@@ -7,8 +7,18 @@
 #
 set -eu
 
+# Cleanup to run whenever we exit
+function cleanup()
+{
+    for n in `jobs -p`
+    do
+        kill $n 2> /dev/null || true
+    done
+    wait
+}
 
-trap "kill `jobs -p` &> /dev/null || true" ERR
+# Install cleanup handler (except for kill -9)
+trap cleanup EXIT
 
 # Exit, with status code "skip" (no 'real' failure)
 function exit_skip() {
@@ -27,7 +37,7 @@ TMP_DIR=`mktemp -d revocation-tmp-XXXXXX`
 export WALLET_DB=wallet-revocation.json
 rm -f $WALLET_DB
 
-# Configuation file will be edited, so we create one
+# Configuration file will be edited, so we create one
 # from the template.
 export CONF=generate-auditor-basedb-revocation.conf
 cp generate-auditor-basedb-template.conf $CONF
@@ -131,23 +141,37 @@ done
 
 if [ 1 != $OK ]
 then
-    kill `jobs -p`
-    wait
+    cleanup
     exit_skip "Failed to launch services"
 fi
 echo " DONE"
 
+# Setup merchant
+
+curl -H "Content-Type: application/json" -X POST -d '{"payto_uris":["payto://x-taler-bank/localhost/43"],"id":"default","name":"default","address":{},"jurisdiction":{},"default_max_wire_fee":"TESTKUDOS:1", "default_max_deposit_fee":"TESTKUDOS:1","default_wire_fee_amortization":1,"default_wire_transfer_delay":{"d_ms" : 3600000},"default_pay_delay":{"d_ms": 3600000}}' http://localhost:9966/private/instances
+
+
 # run wallet CLI
 echo "Running wallet"
-taler-wallet-cli --wallet-db=$WALLET_DB --no-throttle \
-                 testing withdraw \
-                 -e $EXCHANGE_URL \
-                 -b $BANK_URL \
-                 -a TESTKUDOS:8
 
+taler-wallet-cli --no-throttle --wallet-db=$WALLET_DB api 'withdrawTestBalance' \
+  "$(jq -n '
+    {
+      amount: "TESTKUDOS:8",
+      bankBaseUrl: $BANK_URL,
+      exchangeBaseUrl: $EXCHANGE_URL,
+    }' \
+    --arg BANK_URL $BANK_URL \
+    --arg EXCHANGE_URL $EXCHANGE_URL
+  )"
+
+taler-wallet-cli --no-throttle --wallet-db=$WALLET_DB run-until-done
 
 export coins=$(taler-wallet-cli --wallet-db=$WALLET_DB advanced dump-coins)
 
+echo -n "COINS are:"
+echo $coins
+
 # Find coin we want to revoke
 export rc=$(echo "$coins" | jq -r '[.coins[] | select((.denom_value == "TESTKUDOS:2"))][0] | .coin_pub')
 # Find the denom
@@ -159,7 +183,7 @@ export susp=$(echo "$coins" | jq --arg rc "$rc" '[.coins[] | select(.coin_pub !=
 # Do the revocation
 taler-exchange-keyup -o e2a2.dat -c $CONF -r $rd
 taler-auditor-sign -c $CONF -u $AUDITOR_URL -r e2a2.dat -o a2e2.dat -m $MASTER_PUB
-rm e2a2.dat
+rm -f e2a2.dat
 mv a2e2.dat $ABD
 
 # Restart the exchange...
@@ -179,9 +203,17 @@ taler-wallet-cli --wallet-db=$WALLET_DB run-until-done
 
 # Now we buy something, only the coins resulting from recouped will be
 # used, as other ones are suspended
-taler-wallet-cli --wallet-db=$WALLET_DB testing test-pay \
-                 -m $MERCHANT_URL -k sandbox \
-                 -a "TESTKUDOS:1" -s "foo"
+taler-wallet-cli --no-throttle --wallet-db=$WALLET_DB api 'testPay' \
+  "$(jq -n '
+    {
+      amount: "TESTKUDOS:1",
+      merchantApiKey: "sandbox",
+      merchantBaseUrl: $MERCHANT_URL,
+      summary: "foo",
+    }' \
+    --arg MERCHANT_URL $MERCHANT_URL
+  )"
+
 taler-wallet-cli --wallet-db=$WALLET_DB run-until-done
 
 echo "Purchase with recoup'ed coin (via reserve) done"
@@ -245,7 +277,7 @@ export susp=$(echo "$coins" | jq --arg freshc "$freshc" '[.coins[] | select(.coi
 echo "Revoking ${fresh_denom} (to affect coin ${freshc})"
 taler-exchange-keyup -c $CONF -o e2a3.dat -r $fresh_denom
 taler-auditor-sign -c $CONF -u $AUDITOR_URL -r e2a3.dat -o a2e3.dat -m $MASTER_PUB
-rm e2a3.dat
+rm -f e2a3.dat
 mv a2e3.dat $ABD
 
 # Restart the exchange...
@@ -281,16 +313,22 @@ done
 
 # Now we buy something, only the coins resulting from recoup+refresh will be
 # used, as other ones are suspended
-taler-wallet-cli $TIMETRAVEL --wallet-db=$WALLET_DB testing test-pay \
-                 -m $MERCHANT_URL -k sandbox \
-                 -a "TESTKUDOS:0.02" -s "bar"
+taler-wallet-cli $TIMETRAVEL --no-throttle --wallet-db=$WALLET_DB api 'testPay' \
+  "$(jq -n '
+    {
+      amount: "TESTKUDOS:0.02",
+      merchantApiKey: "sandbox",
+      merchantBaseUrl: $MERCHANT_URL,
+      summary: "bar",
+    }' \
+    --arg MERCHANT_URL $MERCHANT_URL
+  )"
 taler-wallet-cli $TIMETRAVEL --wallet-db=$WALLET_DB run-until-done
 
 echo "Bought something with refresh-recouped coin"
 
 echo "Shutting down services"
-kill `jobs -p`
-wait
+cleanup
 
 
 # Dump database
@@ -305,10 +343,10 @@ date +%s > ${BASEDB}.age
 
 
 # clean up
-echo "Final clean up (disabled)"
+echo "Final clean up"
 dropdb $TARGET_DB
-rm -r $DATA_DIR || true
-rm $CONF
+rm -rf $DATA_DIR || true
+rm -f $CONF
 rm -r $TMP_DIR
 
 echo "====================================="
