@@ -72,6 +72,36 @@ struct GNUNET_TIME_Absolute start_time;
  */
 static struct GNUNET_CONTAINER_MultiHashMap *denominations;
 
+/**
+ * Flag that is raised to 'true' if the user
+ * presses CTRL-C to abort the audit.
+ */
+static volatile bool abort_flag;
+
+/**
+ * Context for the SIG-INT (ctrl-C) handler.
+ */
+static struct GNUNET_SIGNAL_Context *sig_int;
+
+/**
+ * Context for the SIGTERM handler.
+ */
+static struct GNUNET_SIGNAL_Context *sig_term;
+
+
+/**
+ * Test if the audit should be aborted because the user
+ * pressed CTRL-C.
+ *
+ * @return false to continue the audit, true to terminate
+ *         cleanly as soon as possible
+ */
+bool
+TALER_ARL_do_abort (void)
+{
+  return abort_flag;
+}
+
 
 /**
  * Convert absolute time to human-readable JSON string.
@@ -417,6 +447,204 @@ test_master_present (void *cls,
 }
 
 
+/**
+ * Perform addition of amounts.  If the addition fails, logs
+ * a detailed error and calls exit() to terminate the process (!).
+ *
+ * Do not call this function directly, use #TALER_ARL_amount_add().
+ *
+ * @param[out] sum where to store @a a1 + @a a2, set to "invalid" on overflow
+ * @param a1 first amount to add
+ * @param a2 second amount to add
+ * @param filename where is the addition called
+ * @param functionname name of the function where the addition is called
+ * @param line line number of the addition
+ */
+void
+TALER_ARL_amount_add_ (struct TALER_Amount *sum,
+                       const struct TALER_Amount *a1,
+                       const struct TALER_Amount *a2,
+                       const char *filename,
+                       const char *functionname,
+                       unsigned int line)
+{
+  enum TALER_AmountArithmeticResult aar;
+  const char *msg;
+  char *a2s;
+
+  aar = TALER_amount_add (sum,
+                          a1,
+                          a2);
+  if (aar >= 0)
+    return;
+  switch (aar)
+  {
+  case TALER_AAR_INVALID_RESULT_OVERFLOW:
+    msg =
+      "arithmetic overflow in amount addition (likely the database is corrupt, see manual)";
+    break;
+  case TALER_AAR_INVALID_NORMALIZATION_FAILED:
+    msg =
+      "normalization failed in amount addition (likely the database is corrupt, see manual)";
+    break;
+  case TALER_AAR_INVALID_CURRENCIES_INCOMPATIBLE:
+    msg =
+      "incompatible currencies in amount addition (likely bad configuration and auditor code missing a sanity check, see manual)";
+    break;
+  default:
+    GNUNET_assert (0); /* should be impossible */
+  }
+  a2s = TALER_amount_to_string (a2);
+  fprintf (stderr,
+           "Aborting audit due to fatal error in function %s at %s:%d trying to add %s to %s: %s\n",
+           functionname,
+           filename,
+           line,
+           TALER_amount2s (a1),
+           a2s,
+           msg);
+  GNUNET_free (a2s);
+  exit (42);
+}
+
+
+/**
+ * Perform subtraction of amounts. If the subtraction fails, logs
+ * a detailed error and calls exit() to terminate the process (!).
+ *
+ * Do not call this function directly, use #TALER_ARL_amount_subtract().
+ *
+ * @param[out] diff where to store (@a a1 - @a a2)
+ * @param a1 amount to subtract from
+ * @param a2 amount to subtract
+ * @param filename where is the addition called
+ * @param functionname name of the function where the addition is called
+ * @param line line number of the addition
+ */
+void
+TALER_ARL_amount_subtract_ (struct TALER_Amount *diff,
+                            const struct TALER_Amount *a1,
+                            const struct TALER_Amount *a2,
+                            const char *filename,
+                            const char *functionname,
+                            unsigned int line)
+{
+  enum TALER_AmountArithmeticResult aar;
+  const char *msg;
+  char *a2s;
+
+  aar = TALER_amount_subtract (diff,
+                               a1,
+                               a2);
+  if (aar >= 0)
+    return;
+  switch (aar)
+  {
+  case TALER_AAR_INVALID_NEGATIVE_RESULT:
+    msg =
+      "negative result in amount subtraction (likely the database is corrupt, see manual)";
+    break;
+  case TALER_AAR_INVALID_NORMALIZATION_FAILED:
+    msg =
+      "normalization failed in amount subtraction (likely the database is corrupt, see manual)";
+    break;
+  case TALER_AAR_INVALID_CURRENCIES_INCOMPATIBLE:
+    msg =
+      "currencies incompatible in amount subtraction (likely bad configuration and auditor code missing a sanity check, see manual)";
+    break;
+  default:
+    GNUNET_assert (0); /* should be impossible */
+  }
+  a2s = TALER_amount_to_string (a2);
+  fprintf (stderr,
+           "Aborting audit due to fatal error in function %s at %s:%d trying to subtract %s from %s: %s\n",
+           functionname,
+           filename,
+           line,
+           a2s,
+           TALER_amount2s (a1),
+           msg);
+  GNUNET_free (a2s);
+  exit (42);
+}
+
+
+/**
+ * Perform subtraction of amounts. Negative results should be signalled by the
+ * return value (leaving @a diff set to 'invalid'). If the subtraction fails
+ * for other reasons (currency mismatch, normalization failure), logs a
+ * detailed error and calls exit() to terminate the process (!).
+ *
+ * Do not call this function directly, use #TALER_ARL_amount_subtract_neg().
+ *
+ * @param[out] diff where to store (@a a1 - @a a2)
+ * @param a1 amount to subtract from
+ * @param a2 amount to subtract
+ * @param filename where is the addition called
+ * @param functionname name of the function where the addition is called
+ * @param line line number of the addition
+ * @return #TALER_ARL_SR_INVALID_NEGATIVE if the result was negative (and @a diff is now invalid),
+ *         #TALER_ARL_SR_ZERO if the result was zero,
+ *         #TALER_ARL_SR_POSITIVE if the result is positive
+ */
+enum TALER_ARL_SubtractionResult
+TALER_ARL_amount_subtract_neg_ (struct TALER_Amount *diff,
+                                const struct TALER_Amount *a1,
+                                const struct TALER_Amount *a2,
+                                const char *filename,
+                                const char *functionname,
+                                unsigned int line)
+{
+  enum TALER_AmountArithmeticResult aar;
+  const char *msg;
+  char *a2s;
+
+  aar = TALER_amount_subtract (diff,
+                               a1,
+                               a2);
+  switch (aar)
+  {
+  case TALER_AAR_RESULT_POSITIVE:
+    return TALER_ARL_SR_POSITIVE;
+  case TALER_AAR_RESULT_ZERO:
+    return TALER_ARL_SR_ZERO;
+  case TALER_AAR_INVALID_NEGATIVE_RESULT:
+    return TALER_ARL_SR_INVALID_NEGATIVE;
+  case TALER_AAR_INVALID_NORMALIZATION_FAILED:
+    msg =
+      "normalization failed in amount subtraction (likely the database is corrupt, see manual)";
+    break;
+  case TALER_AAR_INVALID_CURRENCIES_INCOMPATIBLE:
+    msg =
+      "currencies incompatible in amount subtraction (likely bad configuration and auditor code missing a sanity check, see manual)";
+    break;
+  default:
+    GNUNET_assert (0); /* should be impossible */
+  }
+  a2s = TALER_amount_to_string (a2);
+  fprintf (stderr,
+           "Aborting audit due to fatal error in function %s at %s:%d trying to subtract %s from %s: %s\n",
+           functionname,
+           filename,
+           line,
+           a2s,
+           TALER_amount2s (a1),
+           msg);
+  GNUNET_free (a2s);
+  exit (42);
+}
+
+
+/**
+ * Signal handler called for signals that should cause us to shutdown.
+ */
+static void
+handle_sigint (void)
+{
+  abort_flag = true;
+}
+
+
 /**
  * Setup global variables based on configuration.
  *
@@ -428,7 +656,7 @@ TALER_ARL_init (const struct GNUNET_CONFIGURATION_Handle *c)
 {
   TALER_ARL_cfg = c;
   start_time = GNUNET_TIME_absolute_get ();
-  if (0 == GNUNET_is_zero (&TALER_ARL_master_pub))
+  if (GNUNET_YES == GNUNET_is_zero (&TALER_ARL_master_pub))
   {
     /* -m option not given, try configuration */
     char *TALER_ARL_master_public_key_str;
@@ -473,22 +701,43 @@ TALER_ARL_init (const struct GNUNET_CONFIGURATION_Handle *c)
     return GNUNET_SYSERR;
   }
   {
-    if (GNUNET_OK !=
-        TALER_config_get_amount (TALER_ARL_cfg,
-                                 "taler",
-                                 "CURRENCY_ROUND_UNIT",
-                                 &TALER_ARL_currency_round_unit))
+    if ( (GNUNET_OK !=
+          TALER_config_get_amount (TALER_ARL_cfg,
+                                   "taler",
+                                   "CURRENCY_ROUND_UNIT",
+                                   &TALER_ARL_currency_round_unit)) ||
+         ( (0 != TALER_ARL_currency_round_unit.fraction) &&
+           (0 != TALER_ARL_currency_round_unit.value) ) )
     {
       GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
-                  "Invalid or missing amount in `TALER' under `CURRENCY_ROUND_UNIT'\n");
+                  "Need non-zero value in section `TALER' under `CURRENCY_ROUND_UNIT'\n");
       return GNUNET_SYSERR;
     }
   }
+  sig_int = GNUNET_SIGNAL_handler_install (SIGINT,
+                                           &handle_sigint);
+  if (NULL == sig_int)
+  {
+    GNUNET_log_strerror (GNUNET_ERROR_TYPE_ERROR,
+                         "signal");
+    TALER_ARL_done (NULL);
+    return GNUNET_SYSERR;
+  }
+  sig_term = GNUNET_SIGNAL_handler_install (SIGTERM,
+                                            &handle_sigint);
+  if (NULL == sig_term)
+  {
+    GNUNET_log_strerror (GNUNET_ERROR_TYPE_ERROR,
+                         "signal");
+    TALER_ARL_done (NULL);
+    return GNUNET_SYSERR;
+  }
   if (NULL ==
       (TALER_ARL_edb = TALER_EXCHANGEDB_plugin_load (TALER_ARL_cfg)))
   {
     GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                 "Failed to initialize exchange database plugin.\n");
+    TALER_ARL_done (NULL);
     return GNUNET_SYSERR;
   }
   if (NULL ==
@@ -539,6 +788,16 @@ TALER_ARL_done (json_t *report)
 {
   GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
               "Audit complete\n");
+  if (NULL != sig_int)
+  {
+    GNUNET_SIGNAL_handler_uninstall (sig_int);
+    sig_int = NULL;
+  }
+  if (NULL != sig_term)
+  {
+    GNUNET_SIGNAL_handler_uninstall (sig_term);
+    sig_term = NULL;
+  }
   if (NULL != TALER_ARL_adb)
   {
     TALER_AUDITORDB_plugin_unload (TALER_ARL_adb);
