@@ -75,16 +75,16 @@ struct TAH_RequestHandler
    * @param[in,out] upload_data_size number of bytes (left) in @a upload_data
    * @return MHD result code
    */
-  int (*handler)(struct TAH_RequestHandler *rh,
-                 struct MHD_Connection *connection,
-                 void **connection_cls,
-                 const char *upload_data,
-                 size_t *upload_data_size);
+  MHD_RESULT (*handler)(struct TAH_RequestHandler *rh,
+                        struct MHD_Connection *connection,
+                        void **connection_cls,
+                        const char *upload_data,
+                        size_t *upload_data_size);
 
   /**
    * Default response code.
    */
-  int response_code;
+  unsigned int response_code;
 };
 
 
