@@ -174,19 +174,17 @@ report_amount_arithmetic_inconsistency (
                             auditor))
   {
     /* exchange > auditor */
-    GNUNET_assert (GNUNET_OK ==
-                   TALER_amount_subtract (&delta,
-                                          exchange,
-                                          auditor));
+    TALER_ARL_amount_subtract (&delta,
+                               exchange,
+                               auditor);
   }
   else
   {
     /* auditor < exchange */
     profitable = -profitable;
-    GNUNET_assert (GNUNET_OK ==
-                   TALER_amount_subtract (&delta,
-                                          auditor,
-                                          exchange));
+    TALER_ARL_amount_subtract (&delta,
+                               auditor,
+                               exchange);
   }
   TALER_ARL_report (report_amount_arithmetic_inconsistencies,
                     json_pack ("{s:s, s:I, s:o, s:o, s:I}",
@@ -200,10 +198,9 @@ report_amount_arithmetic_inconsistency (
     target = (1 == profitable)
              ? &total_arithmetic_delta_plus
              : &total_arithmetic_delta_minus;
-    GNUNET_assert (GNUNET_OK ==
-                   TALER_amount_add (target,
-                                     target,
-                                     &delta));
+    TALER_ARL_amount_add (target,
+                          target,
+                          &delta);
   }
 }
 
@@ -263,7 +260,7 @@ struct ReserveSummary
    * Previous balance of the reserve as remembered by the auditor.
    * (updated based on @e total_in and @e total_out at the end).
    */
-  struct TALER_Amount a_balance;
+  struct TALER_Amount balance_at_previous_audit;
 
   /**
    * Previous withdraw fee balance of the reserve, as remembered by the auditor.
@@ -312,7 +309,7 @@ load_auditor_reserve_summary (struct ReserveSummary *rs)
                                         &rs->reserve_pub,
                                         &TALER_ARL_master_pub,
                                         &rowid,
-                                        &rs->a_balance,
+                                        &rs->balance_at_previous_audit,
                                         &rs->a_withdraw_fee_balance,
                                         &rs->a_expiration_date,
                                         &rs->sender_account);
@@ -326,23 +323,23 @@ load_auditor_reserve_summary (struct ReserveSummary *rs)
     rs->had_ri = GNUNET_NO;
     GNUNET_assert (GNUNET_OK ==
                    TALER_amount_get_zero (rs->total_in.currency,
-                                          &rs->a_balance));
+                                          &rs->balance_at_previous_audit));
     GNUNET_assert (GNUNET_OK ==
                    TALER_amount_get_zero (rs->total_in.currency,
                                           &rs->a_withdraw_fee_balance));
     GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
                 "Creating fresh reserve `%s' with starting balance %s\n",
                 TALER_B2S (&rs->reserve_pub),
-                TALER_amount2s (&rs->a_balance));
+                TALER_amount2s (&rs->balance_at_previous_audit));
     return GNUNET_DB_STATUS_SUCCESS_NO_RESULTS;
   }
   rs->had_ri = GNUNET_YES;
   if ( (GNUNET_YES !=
-        TALER_amount_cmp_currency (&rs->a_balance,
+        TALER_amount_cmp_currency (&rs->balance_at_previous_audit,
                                    &rs->a_withdraw_fee_balance)) ||
        (GNUNET_YES !=
         TALER_amount_cmp_currency (&rs->total_in,
-                                   &rs->a_balance)) )
+                                   &rs->balance_at_previous_audit)) )
   {
     GNUNET_break (0);
     return GNUNET_DB_STATUS_HARD_ERROR;
@@ -350,7 +347,7 @@ load_auditor_reserve_summary (struct ReserveSummary *rs)
   GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
               "Auditor remembers reserve `%s' has balance %s\n",
               TALER_B2S (&rs->reserve_pub),
-              TALER_amount2s (&rs->a_balance));
+              TALER_amount2s (&rs->balance_at_previous_audit));
   return GNUNET_DB_STATUS_SUCCESS_ONE_RESULT;
 }
 
@@ -445,10 +442,9 @@ handle_reserve_in (void *cls,
   }
   else
   {
-    GNUNET_assert (GNUNET_OK ==
-                   TALER_amount_add (&rs->total_in,
-                                     &rs->total_in,
-                                     credit));
+    TALER_ARL_amount_add (&rs->total_in,
+                          &rs->total_in,
+                          credit);
     if (NULL == rs->sender_account)
       rs->sender_account = GNUNET_strdup (sender_account_details);
   }
@@ -460,6 +456,8 @@ handle_reserve_in (void *cls,
                                      idle_reserve_expiration_time);
   rs->a_expiration_date = GNUNET_TIME_absolute_max (rs->a_expiration_date,
                                                     expiry);
+  if (TALER_ARL_do_abort ())
+    return GNUNET_SYSERR;
   return GNUNET_OK;
 }
 
@@ -493,6 +491,8 @@ handle_reserve_out (void *cls,
   struct ReserveSummary *rs;
   const struct TALER_DenominationKeyValidityPS *issue;
   struct TALER_Amount withdraw_fee;
+  struct TALER_Amount auditor_value;
+  struct TALER_Amount auditor_amount_with_fee;
   struct GNUNET_TIME_Absolute valid_start;
   struct GNUNET_TIME_Absolute expire_withdraw;
   enum GNUNET_DB_QueryStatus qs;
@@ -528,6 +528,8 @@ handle_reserve_out (void *cls,
     report_row_inconsistency ("withdraw",
                               rowid,
                               "denomination key not found");
+    if (TALER_ARL_do_abort ())
+      return GNUNET_SYSERR;
     return GNUNET_OK;
   }
 
@@ -554,12 +556,11 @@ handle_reserve_out (void *cls,
   }
 
   /* check reserve_sig (first: setup remaining members of wsrd) */
-  wsrd.withdraw_fee = issue->fee_withdraw;
   TALER_amount_hton (&wsrd.amount_with_fee,
                      amount_with_fee);
   if (GNUNET_OK !=
       GNUNET_CRYPTO_eddsa_verify (TALER_SIGNATURE_WALLET_RESERVE_WITHDRAW,
-                                  &wsrd.purpose,
+                                  &wsrd,
                                   &reserve_sig->eddsa_signature,
                                   &reserve_pub->eddsa_pub))
   {
@@ -571,13 +572,31 @@ handle_reserve_out (void *cls,
                                    amount_with_fee),
                                  "key_pub", GNUNET_JSON_from_data_auto (
                                    reserve_pub)));
-    GNUNET_assert (GNUNET_OK ==
-                   TALER_amount_add (&total_bad_sig_loss,
-                                     &total_bad_sig_loss,
-                                     amount_with_fee));
-    return GNUNET_OK;   /* exit here, we cannot add this to the legitimate withdrawals */
+    TALER_ARL_amount_add (&total_bad_sig_loss,
+                          &total_bad_sig_loss,
+                          amount_with_fee);
+    if (TALER_ARL_do_abort ())
+      return GNUNET_SYSERR;
+    return GNUNET_OK;   /* exit function here, we cannot add this to the legitimate withdrawals */
   }
 
+  TALER_amount_ntoh (&withdraw_fee,
+                     &issue->fee_withdraw);
+  TALER_amount_ntoh (&auditor_value,
+                     &issue->value);
+  TALER_ARL_amount_add (&auditor_amount_with_fee,
+                        &auditor_value,
+                        &withdraw_fee);
+  if (0 !=
+      TALER_amount_cmp (&auditor_amount_with_fee,
+                        amount_with_fee))
+  {
+    report_row_inconsistency ("withdraw",
+                              rowid,
+                              "amount with fee from exchange does not match denomination value plus fee");
+  }
+
+
   GNUNET_CRYPTO_hash (reserve_pub,
                       sizeof (*reserve_pub),
                       &key);
@@ -587,7 +606,7 @@ handle_reserve_out (void *cls,
   {
     rs = GNUNET_new (struct ReserveSummary);
     rs->reserve_pub = *reserve_pub;
-    rs->total_out = *amount_with_fee;
+    rs->total_out = auditor_amount_with_fee;
     GNUNET_assert (GNUNET_OK ==
                    TALER_amount_get_zero (amount_with_fee->currency,
                                           &rs->total_in));
@@ -610,24 +629,22 @@ handle_reserve_out (void *cls,
   }
   else
   {
-    GNUNET_assert (GNUNET_OK ==
-                   TALER_amount_add (&rs->total_out,
-                                     &rs->total_out,
-                                     amount_with_fee));
+    TALER_ARL_amount_add (&rs->total_out,
+                          &rs->total_out,
+                          &auditor_amount_with_fee);
   }
   GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
               "Reserve `%s' reduced by %s from withdraw\n",
               TALER_B2S (reserve_pub),
-              TALER_amount2s (amount_with_fee));
-  TALER_amount_ntoh (&withdraw_fee,
-                     &issue->fee_withdraw);
+              TALER_amount2s (&auditor_amount_with_fee));
   GNUNET_log (GNUNET_ERROR_TYPE_INFO,
               "Increasing withdraw profits by fee %s\n",
               TALER_amount2s (&withdraw_fee));
-  GNUNET_assert (GNUNET_OK ==
-                 TALER_amount_add (&rs->total_fee,
-                                   &rs->total_fee,
-                                   &withdraw_fee));
+  TALER_ARL_amount_add (&rs->total_fee,
+                        &rs->total_fee,
+                        &withdraw_fee);
+  if (TALER_ARL_do_abort ())
+    return GNUNET_SYSERR;
   return GNUNET_OK;
 }
 
@@ -686,7 +703,7 @@ handle_recoup_by_reserve (
 
     if (GNUNET_OK !=
         GNUNET_CRYPTO_eddsa_verify (TALER_SIGNATURE_WALLET_COIN_RECOUP,
-                                    &pr.purpose,
+                                    &pr,
                                     &coin_sig->eddsa_signature,
                                     &coin->coin_pub.eddsa_pub))
     {
@@ -697,10 +714,9 @@ handle_recoup_by_reserve (
                                    "loss", TALER_JSON_from_amount (amount),
                                    "key_pub", GNUNET_JSON_from_data_auto (
                                      &coin->coin_pub)));
-      GNUNET_assert (GNUNET_OK ==
-                     TALER_amount_add (&total_bad_sig_loss,
-                                       &total_bad_sig_loss,
-                                       amount));
+      TALER_ARL_amount_add (&total_bad_sig_loss,
+                            &total_bad_sig_loss,
+                            amount);
     }
   }
 
@@ -725,10 +741,9 @@ handle_recoup_by_reserve (
       report_row_inconsistency ("recoup",
                                 rowid,
                                 "denomination key not in revocation set");
-      GNUNET_assert (GNUNET_OK ==
-                     TALER_amount_add (&total_irregular_recoups,
-                                       &total_irregular_recoups,
-                                       amount));
+      TALER_ARL_amount_add (&total_irregular_recoups,
+                            &total_irregular_recoups,
+                            amount);
     }
     else
     {
@@ -743,7 +758,7 @@ handle_recoup_by_reserve (
       if (GNUNET_OK !=
           GNUNET_CRYPTO_eddsa_verify (
             TALER_SIGNATURE_MASTER_DENOMINATION_KEY_REVOKED,
-            &kr.purpose,
+            &kr,
             &msig.eddsa_signature,
             &TALER_ARL_master_pub.eddsa_pub))
       {
@@ -774,10 +789,9 @@ handle_recoup_by_reserve (
                                  "loss", TALER_JSON_from_amount (amount),
                                  "key_pub", GNUNET_JSON_from_data_auto (
                                    &TALER_ARL_master_pub)));
-    GNUNET_assert (GNUNET_OK ==
-                   TALER_amount_add (&total_bad_sig_loss,
-                                     &total_bad_sig_loss,
-                                     amount));
+    TALER_ARL_amount_add (&total_bad_sig_loss,
+                          &total_bad_sig_loss,
+                          amount);
   }
 
   GNUNET_CRYPTO_hash (reserve_pub,
@@ -812,10 +826,9 @@ handle_recoup_by_reserve (
   }
   else
   {
-    GNUNET_assert (GNUNET_OK ==
-                   TALER_amount_add (&rs->total_in,
-                                     &rs->total_in,
-                                     amount));
+    TALER_ARL_amount_add (&rs->total_in,
+                          &rs->total_in,
+                          amount);
   }
   GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
               "Additional /recoup value to for reserve `%s' of %s\n",
@@ -825,6 +838,8 @@ handle_recoup_by_reserve (
                                      idle_reserve_expiration_time);
   rs->a_expiration_date = GNUNET_TIME_absolute_max (rs->a_expiration_date,
                                                     expiry);
+  if (TALER_ARL_do_abort ())
+    return GNUNET_SYSERR;
   return GNUNET_OK;
 }
 
@@ -949,14 +964,12 @@ handle_reserve_closed (
   {
     struct TALER_Amount expected_fee;
 
-    GNUNET_assert (GNUNET_OK ==
-                   TALER_amount_add (&rs->total_out,
-                                     &rs->total_out,
-                                     amount_with_fee));
-    GNUNET_assert (GNUNET_OK ==
-                   TALER_amount_add (&rs->total_fee,
-                                     &rs->total_fee,
-                                     closing_fee));
+    TALER_ARL_amount_add (&rs->total_out,
+                          &rs->total_out,
+                          amount_with_fee);
+    TALER_ARL_amount_add (&rs->total_fee,
+                          &rs->total_fee,
+                          closing_fee);
     /* verify closing_fee is correct! */
     if (GNUNET_OK !=
         get_closing_fee (receiver_account,
@@ -994,6 +1007,8 @@ handle_reserve_closed (
               "Additional closing operation for reserve `%s' of %s\n",
               TALER_B2S (reserve_pub),
               TALER_amount2s (amount_with_fee));
+  if (TALER_ARL_do_abort ())
+    return GNUNET_SYSERR;
   return GNUNET_OK;
 }
 
@@ -1025,25 +1040,22 @@ verify_reserve_balance (void *cls,
   ret = GNUNET_OK;
   /* Check our reserve summary balance calculation shows that
      the reserve balance is acceptable (i.e. non-negative) */
-  GNUNET_assert (GNUNET_OK ==
-                 TALER_amount_add (&balance,
-                                   &rs->total_in,
-                                   &rs->a_balance));
-  if (GNUNET_SYSERR ==
-      TALER_amount_subtract (&nbalance,
-                             &balance,
-                             &rs->total_out))
+  TALER_ARL_amount_add (&balance,
+                        &rs->total_in,
+                        &rs->balance_at_previous_audit);
+  if (TALER_ARL_SR_INVALID_NEGATIVE ==
+      TALER_ARL_amount_subtract_neg (&nbalance,
+                                     &balance,
+                                     &rs->total_out))
   {
     struct TALER_Amount loss;
 
-    GNUNET_assert (GNUNET_SYSERR !=
-                   TALER_amount_subtract (&loss,
-                                          &rs->total_out,
-                                          &balance));
-    GNUNET_assert (GNUNET_OK ==
-                   TALER_amount_add (&total_balance_insufficient_loss,
-                                     &total_balance_insufficient_loss,
-                                     &loss));
+    TALER_ARL_amount_subtract (&loss,
+                               &rs->total_out,
+                               &balance);
+    TALER_ARL_amount_add (&total_balance_insufficient_loss,
+                          &total_balance_insufficient_loss,
+                          &loss);
     TALER_ARL_report (report_reserve_balance_insufficient_inconsistencies,
                       json_pack ("{s:o, s:o}",
                                  "reserve_pub",
@@ -1095,26 +1107,22 @@ verify_reserve_balance (void *cls,
                                 &reserve.balance))
       {
         /* balance > reserve.balance */
-        GNUNET_assert (GNUNET_OK ==
-                       TALER_amount_subtract (&delta,
-                                              &nbalance,
-                                              &reserve.balance));
-        GNUNET_assert (GNUNET_OK ==
-                       TALER_amount_add (&total_balance_summary_delta_plus,
-                                         &total_balance_summary_delta_plus,
-                                         &delta));
+        TALER_ARL_amount_subtract (&delta,
+                                   &nbalance,
+                                   &reserve.balance);
+        TALER_ARL_amount_add (&total_balance_summary_delta_plus,
+                              &total_balance_summary_delta_plus,
+                              &delta);
       }
       else
       {
         /* balance < reserve.balance */
-        GNUNET_assert (GNUNET_OK ==
-                       TALER_amount_subtract (&delta,
-                                              &reserve.balance,
-                                              &nbalance));
-        GNUNET_assert (GNUNET_OK ==
-                       TALER_amount_add (&total_balance_summary_delta_minus,
-                                         &total_balance_summary_delta_minus,
-                                         &delta));
+        TALER_ARL_amount_subtract (&delta,
+                                   &reserve.balance,
+                                   &nbalance);
+        TALER_ARL_amount_add (&total_balance_summary_delta_minus,
+                              &total_balance_summary_delta_minus,
+                              &delta);
       }
       TALER_ARL_report (report_reserve_balance_summary_wrong_inconsistencies,
                         json_pack ("{s:o, s:o, s:o}",
@@ -1147,10 +1155,9 @@ verify_reserve_balance (void *cls,
                                  &cfee))
       {
         /* remaining balance (according to us) exceeds closing fee */
-        GNUNET_assert (GNUNET_OK ==
-                       TALER_amount_add (&total_balance_reserve_not_closed,
-                                         &total_balance_reserve_not_closed,
-                                         &nbalance));
+        TALER_ARL_amount_add (&total_balance_reserve_not_closed,
+                              &total_balance_reserve_not_closed,
+                              &nbalance);
         TALER_ARL_report (report_reserve_not_closed_inconsistencies,
                           json_pack ("{s:o, s:o, s:o}",
                                      "reserve_pub",
@@ -1166,10 +1173,9 @@ verify_reserve_balance (void *cls,
     else
     {
       /* We failed to determine the closing fee, complain! */
-      GNUNET_assert (GNUNET_OK ==
-                     TALER_amount_add (&total_balance_reserve_not_closed,
-                                       &total_balance_reserve_not_closed,
-                                       &nbalance));
+      TALER_ARL_amount_add (&total_balance_reserve_not_closed,
+                            &total_balance_reserve_not_closed,
+                            &nbalance);
       TALER_ARL_report (report_reserve_not_closed_inconsistencies,
                         json_pack ("{s:o, s:o, s:o, s:s}",
                                    "reserve_pub",
@@ -1190,25 +1196,22 @@ verify_reserve_balance (void *cls,
               "Reserve reserve `%s' made %s in withdraw fees\n",
               TALER_B2S (&rs->reserve_pub),
               TALER_amount2s (&rs->total_fee));
-  GNUNET_assert (GNUNET_OK ==
-                 TALER_amount_add (&rs->a_withdraw_fee_balance,
-                                   &rs->a_withdraw_fee_balance,
-                                   &rs->total_fee));
-  GNUNET_assert (GNUNET_OK ==
-                 TALER_amount_add (&total_escrow_balance,
-                                   &total_escrow_balance,
-                                   &rs->total_in));
-  GNUNET_assert (GNUNET_YES ==
-                 TALER_amount_add (&total_withdraw_fee_income,
-                                   &total_withdraw_fee_income,
-                                   &rs->total_fee));
+  TALER_ARL_amount_add (&rs->a_withdraw_fee_balance,
+                        &rs->a_withdraw_fee_balance,
+                        &rs->total_fee);
+  TALER_ARL_amount_add (&total_escrow_balance,
+                        &total_escrow_balance,
+                        &rs->total_in);
+  TALER_ARL_amount_add (&total_withdraw_fee_income,
+                        &total_withdraw_fee_income,
+                        &rs->total_fee);
   {
     struct TALER_Amount r;
 
-    if (GNUNET_SYSERR ==
-        TALER_amount_subtract (&r,
-                               &total_escrow_balance,
-                               &rs->total_out))
+    if (TALER_ARL_SR_INVALID_NEGATIVE ==
+        TALER_ARL_amount_subtract_neg (&r,
+                                       &total_escrow_balance,
+                                       &rs->total_out))
     {
       /* We could not reduce our total balance, i.e. exchange allowed IN TOTAL (!)
          to be withdrawn more than it was IN TOTAL ever given (exchange balance
@@ -1295,7 +1298,7 @@ verify_reserve_balance (void *cls,
                  GNUNET_CONTAINER_multihashmap_remove (rc->reserves,
                                                        key,
                                                        rs));
-  GNUNET_free_non_null (rs->sender_account);
+  GNUNET_free (rs->sender_account);
   GNUNET_free (rs);
   return ret;
 }
