@@ -222,18 +222,18 @@ static json_t *report_reserve_in_inconsistencies;
 static json_t *report_missattribution_in_inconsistencies;
 
 /**
- * Array of reports about row inconcistencies.
+ * Array of reports about row inconsistencies.
  */
 static json_t *report_row_inconsistencies;
 
 /**
- * Array of reports about inconcistencies in the database about
+ * Array of reports about inconsistencies in the database about
  * the incoming wire transfers (exchange is not exactly to blame).
  */
 static json_t *report_wire_format_inconsistencies;
 
 /**
- * Array of reports about minor row inconcistencies.
+ * Array of reports about minor row inconsistencies.
  */
 static json_t *report_row_minor_inconsistencies;
 
@@ -629,10 +629,9 @@ check_pending_rc (void *cls,
 
   (void) cls;
   (void) key;
-  GNUNET_assert (GNUNET_OK ==
-                 TALER_amount_add (&total_closure_amount_lag,
-                                   &total_closure_amount_lag,
-                                   &rc->amount));
+  TALER_ARL_amount_add (&total_closure_amount_lag,
+                        &total_closure_amount_lag,
+                        &rc->amount);
   if ( (0 != rc->amount.value) ||
        (0 != rc->amount.fraction) )
     TALER_ARL_report (report_closure_lags,
@@ -835,10 +834,9 @@ wire_missing_cb (void *cls,
                  /* bool? */ int done)
 {
   (void) cls;
-  GNUNET_assert (GNUNET_OK ==
-                 TALER_amount_add (&total_amount_lag,
-                                   &total_amount_lag,
-                                   amount));
+  TALER_ARL_amount_add (&total_amount_lag,
+                        &total_amount_lag,
+                        amount);
   if ( (GNUNET_YES == tiny) &&
        (0 > TALER_amount_cmp (amount,
                               &tiny_amount)) )
@@ -1001,10 +999,11 @@ wire_out_cb (void *cls,
                                    date),
                                  "diagnostic", "wire transfer not made (yet?)",
                                  "account_section", wa->section_name));
-    GNUNET_assert (GNUNET_OK ==
-                   TALER_amount_add (&total_bad_amount_out_minus,
-                                     &total_bad_amount_out_minus,
-                                     amount));
+    TALER_ARL_amount_add (&total_bad_amount_out_minus,
+                          &total_bad_amount_out_minus,
+                          amount);
+    if (TALER_ARL_do_abort ())
+      return GNUNET_SYSERR;
     return GNUNET_OK;
   }
   {
@@ -1018,7 +1017,7 @@ wire_out_cb (void *cls,
          we should count the wire transfer as entirely spurious, and
          additionally consider the justified wire transfer as missing. */
       TALER_ARL_report (report_wire_out_inconsistencies,
-                        json_pack ("{s:I, s:o, s:o, s:o, s:o, s:s, s:s}",
+                        json_pack ("{s:I, s:o, s:o, s:o, s:o, s:s, s:s, s:s}",
                                    "row", (json_int_t) rowid,
                                    "amount_wired", TALER_JSON_from_amount (
                                      &roi->details.amount),
@@ -1027,14 +1026,14 @@ wire_out_cb (void *cls,
                                    "wtid", GNUNET_JSON_from_data_auto (wtid),
                                    "timestamp", TALER_ARL_json_from_time_abs (
                                      date),
-                                   "diagnostic", "recevier account mismatch",
+                                   "diagnostic", "receiver account mismatch",
+                                   "target", payto_uri,
                                    "account_section", wa->section_name));
-      GNUNET_assert (GNUNET_OK ==
-                     TALER_amount_add (&total_bad_amount_out_plus,
-                                       &total_bad_amount_out_plus,
-                                       &roi->details.amount));
+      TALER_ARL_amount_add (&total_bad_amount_out_plus,
+                            &total_bad_amount_out_plus,
+                            &roi->details.amount);
       TALER_ARL_report (report_wire_out_inconsistencies,
-                        json_pack ("{s:I, s:o, s:o, s:o, s:o, s:s, s:s}",
+                        json_pack ("{s:I, s:o, s:o, s:o, s:o, s:s, s:s, s:s}",
                                    "row", (json_int_t) rowid,
                                    "amount_wired", TALER_JSON_from_amount (
                                      &zero),
@@ -1044,11 +1043,11 @@ wire_out_cb (void *cls,
                                    "timestamp", TALER_ARL_json_from_time_abs (
                                      date),
                                    "diagnostic", "receiver account mismatch",
+                                   "target", roi->details.credit_account_url,
                                    "account_section", wa->section_name));
-      GNUNET_assert (GNUNET_OK ==
-                     TALER_amount_add (&total_bad_amount_out_minus,
-                                       &total_bad_amount_out_minus,
-                                       amount));
+      TALER_ARL_amount_add (&total_bad_amount_out_minus,
+                            &total_bad_amount_out_minus,
+                            amount);
       GNUNET_free (payto_uri);
       goto cleanup;
     }
@@ -1075,28 +1074,24 @@ wire_out_cb (void *cls,
       /* amount > roi->details.amount: wire transfer was smaller than it should have been */
       struct TALER_Amount delta;
 
-      GNUNET_assert (GNUNET_OK ==
-                     TALER_amount_subtract (&delta,
-                                            amount,
-                                            &roi->details.amount));
-      GNUNET_assert (GNUNET_OK ==
-                     TALER_amount_add (&total_bad_amount_out_minus,
-                                       &total_bad_amount_out_minus,
-                                       &delta));
+      TALER_ARL_amount_subtract (&delta,
+                                 amount,
+                                 &roi->details.amount);
+      TALER_ARL_amount_add (&total_bad_amount_out_minus,
+                            &total_bad_amount_out_minus,
+                            &delta);
     }
     else
     {
       /* roi->details.amount < amount: wire transfer was larger than it should have been */
       struct TALER_Amount delta;
 
-      GNUNET_assert (GNUNET_OK ==
-                     TALER_amount_subtract (&delta,
-                                            &roi->details.amount,
-                                            amount));
-      GNUNET_assert (GNUNET_OK ==
-                     TALER_amount_add (&total_bad_amount_out_plus,
-                                       &total_bad_amount_out_plus,
-                                       &delta));
+      TALER_ARL_amount_subtract (&delta,
+                                 &roi->details.amount,
+                                 amount);
+      TALER_ARL_amount_add (&total_bad_amount_out_plus,
+                            &total_bad_amount_out_plus,
+                            &delta);
     }
     goto cleanup;
   }
@@ -1111,6 +1106,8 @@ cleanup:
                            &key,
                            roi));
   wa->pp.last_wire_out_serial_id = rowid + 1;
+  if (TALER_ARL_do_abort ())
+    return GNUNET_SYSERR;
   return GNUNET_OK;
 }
 
@@ -1217,10 +1214,9 @@ complain_out_not_found (void *cls,
                                wa->section_name,
                                "diagnostic",
                                "justification for wire transfer not found"));
-  GNUNET_assert (GNUNET_OK ==
-                 TALER_amount_add (&total_bad_amount_out_plus,
-                                   &total_bad_amount_out_plus,
-                                   &roi->details.amount));
+  TALER_ARL_amount_add (&total_bad_amount_out_plus,
+                        &total_bad_amount_out_plus,
+                        &roi->details.amount);
   return GNUNET_OK;
 }
 
@@ -1350,10 +1346,9 @@ history_debit_cb (void *cls,
     GNUNET_asprintf (&diagnostic,
                      "duplicate subject hash `%s'",
                      TALER_B2S (&roi->subject_hash));
-    GNUNET_assert (GNUNET_OK ==
-                   TALER_amount_add (&total_wire_format_amount,
-                                     &total_wire_format_amount,
-                                     &details->amount));
+    TALER_ARL_amount_add (&total_wire_format_amount,
+                          &total_wire_format_amount,
+                          &details->amount);
     TALER_ARL_report (report_wire_format_inconsistencies,
                       json_pack ("{s:o, s:I, s:s}",
                                  "amount", TALER_JSON_from_amount (
@@ -1502,9 +1497,13 @@ reserve_in_cb (void *cls,
                                    &rii->row_off_hash),
                                  "diagnostic", "duplicate wire offset"));
     GNUNET_free (rii);
+    if (TALER_ARL_do_abort ())
+      return GNUNET_SYSERR;
     return GNUNET_OK;
   }
   wa->pp.last_reserve_in_serial_id = rowid + 1;
+  if (TALER_ARL_do_abort ())
+    return GNUNET_SYSERR;
   return GNUNET_OK;
 }
 
@@ -1540,10 +1539,9 @@ complain_in_not_found (void *cls,
                                "account", wa->section_name,
                                "diagnostic",
                                "incoming wire transfer claimed by exchange not found"));
-  GNUNET_assert (GNUNET_OK ==
-                 TALER_amount_add (&total_bad_amount_in_minus,
-                                   &total_bad_amount_in_minus,
-                                   &rii->details.amount));
+  TALER_ARL_amount_add (&total_bad_amount_in_minus,
+                        &total_bad_amount_in_minus,
+                        &rii->details.amount);
   return GNUNET_OK;
 }
 
@@ -1653,10 +1651,9 @@ history_credit_cb (void *cls,
                                  "timestamp", TALER_ARL_json_from_time_abs (
                                    rii->details.execution_date),
                                  "diagnostic", "wire subject does not match"));
-    GNUNET_assert (GNUNET_OK ==
-                   TALER_amount_add (&total_bad_amount_in_minus,
-                                     &total_bad_amount_in_minus,
-                                     &rii->details.amount));
+    TALER_ARL_amount_add (&total_bad_amount_in_minus,
+                          &total_bad_amount_in_minus,
+                          &rii->details.amount);
     TALER_ARL_report (report_reserve_in_inconsistencies,
                       json_pack ("{s:I, s:I, s:o, s:o, s:o, s:o, s:s}",
                                  "row", (json_int_t) rii->rowid,
@@ -1672,10 +1669,9 @@ history_credit_cb (void *cls,
                                    details->execution_date),
                                  "diagnostic", "wire subject does not match"));
 
-    GNUNET_assert (GNUNET_OK ==
-                   TALER_amount_add (&total_bad_amount_in_plus,
-                                     &total_bad_amount_in_plus,
-                                     &details->amount));
+    TALER_ARL_amount_add (&total_bad_amount_in_plus,
+                          &total_bad_amount_in_plus,
+                          &details->amount);
     goto cleanup;
   }
   if (0 != TALER_amount_cmp (&rii->details.amount,
@@ -1701,28 +1697,24 @@ history_credit_cb (void *cls,
       /* details->amount > rii->details.amount: wire transfer was larger than it should have been */
       struct TALER_Amount delta;
 
-      GNUNET_assert (GNUNET_OK ==
-                     TALER_amount_subtract (&delta,
-                                            &details->amount,
-                                            &rii->details.amount));
-      GNUNET_assert (GNUNET_OK ==
-                     TALER_amount_add (&total_bad_amount_in_plus,
-                                       &total_bad_amount_in_plus,
-                                       &delta));
+      TALER_ARL_amount_subtract (&delta,
+                                 &details->amount,
+                                 &rii->details.amount);
+      TALER_ARL_amount_add (&total_bad_amount_in_plus,
+                            &total_bad_amount_in_plus,
+                            &delta);
     }
     else
     {
       /* rii->details.amount < details->amount: wire transfer was smaller than it should have been */
       struct TALER_Amount delta;
 
-      GNUNET_assert (GNUNET_OK ==
-                     TALER_amount_subtract (&delta,
-                                            &rii->details.amount,
-                                            &details->amount));
-      GNUNET_assert (GNUNET_OK ==
-                     TALER_amount_add (&total_bad_amount_in_minus,
-                                       &total_bad_amount_in_minus,
-                                       &delta));
+      TALER_ARL_amount_subtract (&delta,
+                                 &rii->details.amount,
+                                 &details->amount);
+      TALER_ARL_amount_add (&total_bad_amount_in_minus,
+                            &total_bad_amount_in_minus,
+                            &delta);
     }
     goto cleanup;
   }
@@ -1737,10 +1729,9 @@ history_credit_cb (void *cls,
                                  "bank_row", (json_int_t) row_off,
                                  "reserve_pub", GNUNET_JSON_from_data_auto (
                                    &rii->details.reserve_pub)));
-    GNUNET_assert (GNUNET_OK ==
-                   TALER_amount_add (&total_missattribution_in,
-                                     &total_missattribution_in,
-                                     &rii->details.amount));
+    TALER_ARL_amount_add (&total_missattribution_in,
+                          &total_missattribution_in,
+                          &rii->details.amount);
   }
   if (details->execution_date.abs_value_us !=
       rii->details.execution_date.abs_value_us)
@@ -1867,10 +1858,10 @@ reserve_closed_cb (void *cls,
 
   (void) cls;
   rc = GNUNET_new (struct ReserveClosure);
-  if (GNUNET_SYSERR ==
-      TALER_amount_subtract (&rc->amount,
-                             amount_with_fee,
-                             closing_fee))
+  if (TALER_ARL_SR_INVALID_NEGATIVE ==
+      TALER_ARL_amount_subtract_neg (&rc->amount,
+                                     amount_with_fee,
+                                     closing_fee))
   {
     TALER_ARL_report (report_row_inconsistencies,
                       json_pack ("{s:s, s:I, s:o, s:o, s:o, s:s}",
@@ -1885,6 +1876,8 @@ reserve_closed_cb (void *cls,
                                  "diagnostic",
                                  "closing fee above total amount"));
     GNUNET_free (rc);
+    if (TALER_ARL_do_abort ())
+      return GNUNET_SYSERR;
     return GNUNET_OK;
   }
   pp.last_reserve_close_uuid
@@ -1901,6 +1894,8 @@ reserve_closed_cb (void *cls,
                                             &key,
                                             rc,
                                             GNUNET_CONTAINER_MULTIHASHMAPOPTION_MULTIPLE);
+  if (TALER_ARL_do_abort ())
+    return GNUNET_SYSERR;
   return GNUNET_OK;
 }
 
@@ -2083,10 +2078,6 @@ run (void *cls,
                                "TINY_AMOUNT",
                                &tiny_amount))
   {
-    GNUNET_log_config_invalid (GNUNET_ERROR_TYPE_ERROR,
-                               "auditor",
-                               "TINY_AMOUNT",
-                               "invalid amount");
     global_ret = 1;
     return;
   }
