@@ -154,7 +154,7 @@ handle_debit_history_finished (void *cls,
   switch (response_code)
   {
   case 0:
-    ec = TALER_EC_INVALID_RESPONSE;
+    ec = TALER_EC_GENERIC_INVALID_RESPONSE;
     break;
   case MHD_HTTP_OK:
     if (GNUNET_OK !=
@@ -163,7 +163,7 @@ handle_debit_history_finished (void *cls,
     {
       GNUNET_break_op (0);
       response_code = 0;
-      ec = TALER_EC_INVALID_RESPONSE;
+      ec = TALER_EC_GENERIC_INVALID_RESPONSE;
       break;
     }
     response_code = MHD_HTTP_NO_CONTENT; /* signal end of list */
