@@ -25,6 +25,7 @@
 #include "taler_fakebank_lib.h"
 #include "taler_bank_service.h"
 #include "taler_mhd_lib.h"
+#include <gnunet/gnunet_mhd_compat.h>
 
 /**
  * Maximum POST request size (for /admin/add-incoming)
@@ -585,7 +586,7 @@ handle_mhd_completion_callback (void *cls,
  * @param con_cls closure for request (a `struct Buffer *`)
  * @return MHD result code
  */
-static int
+static MHD_RESULT
 handle_admin_add_incoming (struct TALER_FAKEBANK_Handle *h,
                            struct MHD_Connection *connection,
                            const char *account,
@@ -680,7 +681,7 @@ handle_admin_add_incoming (struct TALER_FAKEBANK_Handle *h,
  * @param con_cls closure for request (a `struct Buffer *`)
  * @return MHD result code
  */
-static int
+static MHD_RESULT
 handle_transfer (struct TALER_FAKEBANK_Handle *h,
                  struct MHD_Connection *connection,
                  const char *account,
@@ -759,14 +760,19 @@ handle_transfer (struct TALER_FAKEBANK_Handle *h,
                                           &row_id);
       if (GNUNET_OK != ret)
       {
+        MHD_RESULT res;
+        char *uids;
+
         GNUNET_break (0);
+        uids = GNUNET_STRINGS_data_to_string_alloc (&uuid,
+                                                    sizeof (uuid));
         json_decref (json);
-        return TALER_MHD_reply_with_error (connection,
-                                           MHD_HTTP_CONFLICT,
-                                           TALER_EC_BANK_TRANSFER_REQUEST_UID_REUSED,
-                                           "transfer request UID was reused");
-
-
+        res = TALER_MHD_reply_with_error (connection,
+                                          MHD_HTTP_CONFLICT,
+                                          TALER_EC_BANK_TRANSFER_REQUEST_UID_REUSED,
+                                          uids);
+        GNUNET_free (uids);
+        return res;
       }
       GNUNET_log (GNUNET_ERROR_TYPE_INFO,
                   "Receiving incoming wire transfer: %s->%s, subject: %s, amount: %s, from %s\n",
@@ -800,12 +806,12 @@ handle_transfer (struct TALER_FAKEBANK_Handle *h,
  * @param con_cls place to store state, not used
  * @return MHD result code
  */
-static int
+static MHD_RESULT
 handle_home_page (struct TALER_FAKEBANK_Handle *h,
                   struct MHD_Connection *connection,
                   void **con_cls)
 {
-  int ret;
+  MHD_RESULT ret;
   struct MHD_Response *resp;
 #define HELLOMSG "Hello, Fakebank!"
 
@@ -867,7 +873,7 @@ struct HistoryArgs
  *
  * @param connection MHD connection.
  * @param[out] ha will contain the parsed values.
- * @return #GNUNET_OK only if the parsing succeedes.
+ * @return #GNUNET_OK only if the parsing succeeds.
  */
 static int
 parse_history_common_args (struct MHD_Connection *connection,
@@ -936,7 +942,7 @@ parse_history_common_args (struct MHD_Connection *connection,
  * @param account which account the request is about
  * @return MHD result code
  */
-static int
+static MHD_RESULT
 handle_debit_history (struct TALER_FAKEBANK_Handle *h,
                       struct MHD_Connection *connection,
                       const char *account)
@@ -1051,7 +1057,7 @@ handle_debit_history (struct TALER_FAKEBANK_Handle *h,
  * @param account which account the request is about
  * @return MHD result code
  */
-static int
+static MHD_RESULT
 handle_credit_history (struct TALER_FAKEBANK_Handle *h,
                        struct MHD_Connection *connection,
                        const char *account)
@@ -1198,7 +1204,7 @@ handle_credit_history (struct TALER_FAKEBANK_Handle *h,
  * @param con_cls closure for request (a `struct Buffer *`)
  * @return MHD result code
  */
-static int
+static MHD_RESULT
 serve (struct TALER_FAKEBANK_Handle *h,
        struct MHD_Connection *connection,
        const char *account,
@@ -1279,7 +1285,7 @@ serve (struct TALER_FAKEBANK_Handle *h,
  * @param con_cls closure for request (a `struct Buffer *`)
  * @return MHD result code
  */
-static int
+static MHD_RESULT
 handle_mhd_request (void *cls,
                     struct MHD_Connection *connection,
                     const char *url,
@@ -1292,7 +1298,7 @@ handle_mhd_request (void *cls,
   struct TALER_FAKEBANK_Handle *h = cls;
   char *account = NULL;
   char *end;
-  int ret;
+  MHD_RESULT ret;
 
   (void) version;
   GNUNET_log (GNUNET_ERROR_TYPE_INFO,
@@ -1313,7 +1319,7 @@ handle_mhd_request (void *cls,
                upload_data,
                upload_data_size,
                con_cls);
-  GNUNET_free_non_null (account);
+  GNUNET_free (account);
   return ret;
 }
 
