@@ -235,7 +235,7 @@ static struct GNUNET_TIME_Relative max_duration_spend;
 static struct GNUNET_HashCode revoke_dkh;
 
 /**
- * Which RSA key size should we use for replacment keys after revocation?
+ * Which RSA key size should we use for replacement keys after revocation?
  * (Useful because maybe that's the one option one might usefully want to
  * change when replacing a key.)
  */
@@ -481,12 +481,9 @@ create_signkey_issue_priv (
   struct GNUNET_TIME_Absolute end,
   struct TALER_EXCHANGEDB_PrivateSigningKeyInformationP *pi)
 {
-  struct GNUNET_CRYPTO_EddsaPrivateKey *priv;
   struct TALER_ExchangeSigningKeyValidityPS *issue = &pi->issue;
 
-  priv = GNUNET_CRYPTO_eddsa_key_create ();
-  pi->signkey_priv.eddsa_priv = *priv;
-  GNUNET_free (priv);
+  GNUNET_CRYPTO_eddsa_key_create (&pi->signkey_priv.eddsa_priv);
   issue->master_public_key = master_public_key;
   issue->start = GNUNET_TIME_absolute_hton (start);
   issue->expire = GNUNET_TIME_absolute_hton (GNUNET_TIME_absolute_add (start,
@@ -497,10 +494,9 @@ create_signkey_issue_priv (
   issue->purpose.purpose = htonl (TALER_SIGNATURE_MASTER_SIGNING_KEY_VALIDITY);
   issue->purpose.size = htonl (sizeof (struct
                                        TALER_ExchangeSigningKeyValidityPS));
-  GNUNET_assert (GNUNET_OK ==
-                 GNUNET_CRYPTO_eddsa_sign (&master_priv.eddsa_priv,
-                                           &issue->purpose,
-                                           &pi->master_sig.eddsa_signature));
+  GNUNET_CRYPTO_eddsa_sign (&master_priv.eddsa_priv,
+                            issue,
+                            &pi->master_sig.eddsa_signature);
 }
 
 
@@ -683,9 +679,6 @@ get_denomination_type_params (const char *ct,
                                "VALUE",
                                &params->value))
   {
-    GNUNET_log_config_missing (GNUNET_ERROR_TYPE_ERROR,
-                               ct,
-                               "VALUE");
     return GNUNET_SYSERR;
   }
   if (GNUNET_OK !=
@@ -694,9 +687,6 @@ get_denomination_type_params (const char *ct,
                                "FEE_WITHDRAW",
                                &params->fee_withdraw))
   {
-    GNUNET_log_config_missing (GNUNET_ERROR_TYPE_ERROR,
-                               ct,
-                               "FEE_WITHDRAW");
     return GNUNET_SYSERR;
   }
   if (GNUNET_OK !=
@@ -705,9 +695,6 @@ get_denomination_type_params (const char *ct,
                                "FEE_DEPOSIT",
                                &params->fee_deposit))
   {
-    GNUNET_log_config_missing (GNUNET_ERROR_TYPE_ERROR,
-                               ct,
-                               "FEE_DEPOSIT");
     return GNUNET_SYSERR;
   }
   if (GNUNET_OK !=
@@ -716,9 +703,6 @@ get_denomination_type_params (const char *ct,
                                "FEE_REFRESH",
                                &params->fee_refresh))
   {
-    GNUNET_log_config_missing (GNUNET_ERROR_TYPE_ERROR,
-                               ct,
-                               "FEE_REFRESH");
     return GNUNET_SYSERR;
   }
   if (GNUNET_OK !=
@@ -727,9 +711,6 @@ get_denomination_type_params (const char *ct,
                                "fee_refund",
                                &params->fee_refund))
   {
-    GNUNET_log_config_missing (GNUNET_ERROR_TYPE_ERROR,
-                               ct,
-                               "FEE_REFUND");
     return GNUNET_SYSERR;
   }
 
@@ -796,10 +777,9 @@ create_denomkey_issue (
     = htonl (TALER_SIGNATURE_MASTER_DENOMINATION_KEY_VALIDITY);
   dki->issue.properties.purpose.size
     = htonl (sizeof (struct TALER_DenominationKeyValidityPS));
-  GNUNET_assert (GNUNET_OK ==
-                 GNUNET_CRYPTO_eddsa_sign (&master_priv.eddsa_priv,
-                                           &dki->issue.properties.purpose,
-                                           &dki->issue.signature.eddsa_signature));
+  GNUNET_CRYPTO_eddsa_sign (&master_priv.eddsa_priv,
+                            &dki->issue.properties,
+                            &dki->issue.signature.eddsa_signature);
 }
 
 
@@ -948,10 +928,9 @@ sign_af (struct TALER_EXCHANGEDB_AggregateFees *af,
   TALER_EXCHANGEDB_fees_2_wf (method,
                               af,
                               &wf);
-  GNUNET_assert (GNUNET_OK ==
-                 GNUNET_CRYPTO_eddsa_sign (priv,
-                                           &wf.purpose,
-                                           &af->master_sig.eddsa_signature));
+  GNUNET_CRYPTO_eddsa_sign (priv,
+                            &wf,
+                            &af->master_sig.eddsa_signature);
 }
 
 
@@ -1022,9 +1001,10 @@ create_wire_fee_for_method (void *cls,
                            af->wire_fee.currency)) )
     {
       GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
-                  "Invalid or missing amount in `%s' under `%s'\n",
-                  wiremethod,
-                  opt);
+                  "Need amount with currency `%s' for option `%s' in section `%s'\n",
+                  currency,
+                  opt,
+                  section);
       *ret = GNUNET_SYSERR;
       GNUNET_free (opt);
       break;
@@ -1044,9 +1024,10 @@ create_wire_fee_for_method (void *cls,
                            af->closing_fee.currency)) )
     {
       GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
-                  "Invalid or missing amount in `%s' under `%s'\n",
-                  wiremethod,
-                  opt);
+                  "Need amount with currency `%s' for option `%s' in section `%s'\n",
+                  currency,
+                  opt,
+                  section);
       *ret = GNUNET_SYSERR;
       GNUNET_free (opt);
       break;
@@ -1151,7 +1132,7 @@ check_revocation_regeneration (
   if (now.abs_value_us >= withdraw_end.abs_value_us)
   {
     GNUNET_log (GNUNET_ERROR_TYPE_INFO,
-                "Revoked denomination key has expired, no need to create a replacment\n");
+                "Revoked denomination key has expired, no need to create a replacement\n");
     return GNUNET_NO;
   }
 
@@ -1302,6 +1283,11 @@ run (void *cls,
     /* The user gave "--now", use it! */
     now = now_tmp;
   }
+  else
+  {
+    /* get current time again, we may be timetraveling! */
+    now = GNUNET_TIME_absolute_get ();
+  }
   GNUNET_TIME_round_abs (&now);
   if (GNUNET_OK !=
       GNUNET_CONFIGURATION_get_value_time (kcfg,
@@ -1405,7 +1391,7 @@ run (void *cls,
 
 
   /* finally, do actual work */
-  if (0 != GNUNET_is_zero (&revoke_dkh))
+  if (GNUNET_NO == GNUNET_is_zero (&revoke_dkh))
   {
     if (GNUNET_OK != revoke_denomination (&revoke_dkh))
     {
