@@ -23,44 +23,23 @@
 
 #include <microhttpd.h>
 #include "taler_exchangedb_plugin.h"
+#include <gnunet/gnunet_mhd_compat.h>
 
 
 /**
- * Type of closure for #TEH_DB_know_coin_transaction.
- */
-struct TEH_DB_KnowCoinContext
-{
-  /**
-   * The coin to make sure it is known.
-   */
-  const struct TALER_CoinPublicInfo *coin;
-
-  /**
-   * MHD connection to queue errors with.
-   */
-  struct MHD_Connection *connection;
-};
-
-
-/**
- * Execute database transaction to ensure coin is known. Run the transaction
- * logic; IF it returns a non-error code, the transaction logic MUST
- * NOT queue a MHD response.  IF it returns an hard error, the
- * transaction logic MUST queue a MHD response and set @a mhd_ret.  IF
- * it returns the soft error code, the function MAY be called again to
- * retry and MUST not queue a MHD response.
+ * Ensure coin is known in the database, and handle conflicts and errors.
  *
- * @param cls a `struct TEH_DB_KnowCoinContext`
- * @param connection MHD request context, must not be NULL
+ * @param coin the coin to make known
+ * @param connection MHD request context
  * @param session database session and transaction to use
- * @param[out] mhd_ret set to MHD status on error, must not be NULL
- * @return transaction status
+ * @param[out] mhd_ret set to MHD status on error
+ * @return transaction status, negative on error (@a mhd_ret will be set in this case)
  */
 enum GNUNET_DB_QueryStatus
-TEH_DB_know_coin_transaction (void *cls,
-                              struct MHD_Connection *connection,
-                              struct TALER_EXCHANGEDB_Session *session,
-                              int *mhd_ret);
+TEH_make_coin_known (const struct TALER_CoinPublicInfo *coin,
+                     struct MHD_Connection *connection,
+                     struct TALER_EXCHANGEDB_Session *session,
+                     MHD_RESULT *mhd_ret);
 
 
 /**
@@ -82,7 +61,7 @@ typedef enum GNUNET_DB_QueryStatus
 (*TEH_DB_TransactionCallback)(void *cls,
                               struct MHD_Connection *connection,
                               struct TALER_EXCHANGEDB_Session *session,
-                              int *mhd_ret);
+                              MHD_RESULT *mhd_ret);
 
 
 /**
@@ -101,10 +80,10 @@ typedef enum GNUNET_DB_QueryStatus
  * @param cb_cls closure for @a cb, must be read-only!
  * @return #GNUNET_OK on success, #GNUNET_SYSERR on failure
  */
-int
+enum GNUNET_GenericReturnValue
 TEH_DB_run_transaction (struct MHD_Connection *connection,
                         const char *name,
-                        int *mhd_ret,
+                        MHD_RESULT *mhd_ret,
                         TEH_DB_TransactionCallback cb,
                         void *cb_cls);
 
