@@ -191,7 +191,7 @@ struct DenomKeyPair
 /**
  * Destroy a denomination key pair.  The key is not necessarily removed from the DB.
  *
- * @param dkp the keypair to destroy
+ * @param dkp the key pair to destroy
  */
 static void
 destroy_denom_key_pair (struct DenomKeyPair *dkp)
@@ -559,7 +559,7 @@ test_melting (struct TALER_EXCHANGEDB_Session *session)
                             session,
                             &refresh_session.rc,
                             &ret_refresh_session));
-  FAILIF (GNUNET_DB_STATUS_SUCCESS_ONE_RESULT !=
+  FAILIF (TALER_EXCHANGEDB_CKS_ADDED !=
           plugin->ensure_coin_known (plugin->cls,
                                      session,
                                      &refresh_session.coin));
@@ -706,12 +706,12 @@ drop:
   destroy_denom_key_pair (dkp);
   GNUNET_CRYPTO_rsa_signature_free (
     refresh_session.coin.denom_sig.rsa_signature);
-  GNUNET_free_non_null (new_denom_pubs);
+  GNUNET_free (new_denom_pubs);
   for (unsigned int cnt = 0;
        (NULL != new_dkp) && (cnt < MELT_NEW_COINS) && (NULL != new_dkp[cnt]);
        cnt++)
     destroy_denom_key_pair (new_dkp[cnt]);
-  GNUNET_free_non_null (new_dkp);
+  GNUNET_free (new_dkp);
   return ret;
 }
 
@@ -833,19 +833,23 @@ static uint64_t deposit_rowid;
  * @param cls closure a `struct TALER_EXCHANGEDB_Deposit *`
  * @param rowid unique ID for the deposit in our DB, used for marking
  *              it as 'tiny' or 'done'
+ * @param exchange_timestamp when did the deposit happen
+ * @param wallet_timestamp when did the wallet sign the contract
  * @param merchant_pub public key of the merchant
  * @param coin_pub public key of the coin
  * @param amount_with_fee amount that was deposited including fee
  * @param deposit_fee amount the exchange gets to keep as transaction fees
  * @param h_contract_terms hash of the proposal data known to merchant and customer
- * @param wire_deadline by which the merchant adviced that he would like the
+ * @param wire_deadline by which the merchant advised that he would like the
  *        wire transfer to be executed
- * @param wire wire details for the merchant, NULL from iterate_matching_deposits()
+ * @param wire wire details for the merchant
  * @return transaction status code, #GNUNET_DB_STATUS_SUCCESS_ONE_RESULT to continue to iterate
  */
 static enum GNUNET_DB_QueryStatus
 deposit_cb (void *cls,
             uint64_t rowid,
+            struct GNUNET_TIME_Absolute exchange_timestamp,
+            struct GNUNET_TIME_Absolute wallet_timestamp,
             const struct TALER_MerchantPublicKeyP *merchant_pub,
             const struct TALER_CoinSpendPublicKeyP *coin_pub,
             const struct TALER_Amount *amount_with_fee,
@@ -858,10 +862,9 @@ deposit_cb (void *cls,
   struct GNUNET_HashCode h_wire;
 
   deposit_rowid = rowid;
-  if (NULL != wire)
-    GNUNET_assert (GNUNET_OK ==
-                   TALER_JSON_merchant_wire_signature_hash (wire,
-                                                            &h_wire));
+  GNUNET_assert (GNUNET_OK ==
+                 TALER_JSON_merchant_wire_signature_hash (wire,
+                                                          &h_wire));
   if ( (0 != GNUNET_memcmp (merchant_pub,
                             &deposit->merchant_pub)) ||
        (0 != TALER_amount_cmp (amount_with_fee,
@@ -873,9 +876,51 @@ deposit_cb (void *cls,
        (0 != memcmp (coin_pub,
                      &deposit->coin.coin_pub,
                      sizeof (struct TALER_CoinSpendPublicKeyP))) ||
-       ( (NULL != wire) &&
-         (0 != GNUNET_memcmp (&h_wire,
-                              &deposit->h_wire)) ) )
+       (0 != GNUNET_memcmp (&h_wire,
+                            &deposit->h_wire)) )
+  {
+    GNUNET_break (0);
+    return GNUNET_DB_STATUS_HARD_ERROR;
+  }
+
+  return GNUNET_DB_STATUS_SUCCESS_ONE_RESULT;
+}
+
+
+/**
+ * Function called with details about deposits that
+ * have been made.  Called in the test on the
+ * deposit given in @a cls.
+ *
+ * @param cls closure a `struct TALER_EXCHANGEDB_Deposit *`
+ * @param rowid unique ID for the deposit in our DB, used for marking
+ *              it as 'tiny' or 'done'
+ * @param coin_pub public key of the coin
+ * @param amount_with_fee amount that was deposited including fee
+ * @param deposit_fee amount the exchange gets to keep as transaction fees
+ * @param h_contract_terms hash of the proposal data known to merchant and customer
+ * @return transaction status code, #GNUNET_DB_STATUS_SUCCESS_ONE_RESULT to continue to iterate
+ */
+static enum GNUNET_DB_QueryStatus
+matching_deposit_cb (void *cls,
+                     uint64_t rowid,
+                     const struct TALER_CoinSpendPublicKeyP *coin_pub,
+                     const struct TALER_Amount *amount_with_fee,
+                     const struct TALER_Amount *deposit_fee,
+                     const struct GNUNET_HashCode *h_contract_terms)
+{
+  struct TALER_EXCHANGEDB_Deposit *deposit = cls;
+
+  deposit_rowid = rowid;
+  if ( (0 != TALER_amount_cmp (amount_with_fee,
+                               &deposit->amount_with_fee)) ||
+       (0 != TALER_amount_cmp (deposit_fee,
+                               &deposit->deposit_fee)) ||
+       (0 != GNUNET_memcmp (h_contract_terms,
+                            &deposit->h_contract_terms)) ||
+       (0 != memcmp (coin_pub,
+                     &deposit->coin.coin_pub,
+                     sizeof (struct TALER_CoinSpendPublicKeyP))) )
   {
     GNUNET_break (0);
     return GNUNET_DB_STATUS_HARD_ERROR;
@@ -890,16 +935,17 @@ deposit_cb (void *cls,
  *
  * @param cls closure
  * @param rowid unique serial ID for the deposit in our DB
- * @param timestamp when did the deposit happen
+ * @param exchange_timestamp when did the deposit happen
+ * @param wallet_timestamp when did the wallet sign the contract
  * @param merchant_pub public key of the merchant
  * @param denom_pub denomination of the @a coin_pub
  * @param coin_pub public key of the coin
  * @param coin_sig signature from the coin
  * @param amount_with_fee amount that was deposited including fee
  * @param h_contract_terms hash of the proposal data known to merchant and customer
- * @param refund_deadline by which the merchant adviced that he might want
+ * @param refund_deadline by which the merchant advised that he might want
  *        to get a refund
- * @param wire_deadline by which the merchant adviced that he would like the
+ * @param wire_deadline by which the merchant advised that he would like the
  *        wire transfer to be executed
  * @param receiver_wire_account wire details for the merchant, NULL from iterate_matching_deposits()
  * @param done flag set if the deposit was already executed (or not)
@@ -908,7 +954,8 @@ deposit_cb (void *cls,
 static int
 audit_deposit_cb (void *cls,
                   uint64_t rowid,
-                  struct GNUNET_TIME_Absolute timestamp,
+                  struct GNUNET_TIME_Absolute exchange_timestamp,
+                  struct GNUNET_TIME_Absolute wallet_timestamp,
                   const struct TALER_MerchantPublicKeyP *merchant_pub,
                   const struct TALER_DenominationPublicKey *denom_pub,
                   const struct TALER_CoinSpendPublicKeyP *coin_pub,
@@ -1264,7 +1311,7 @@ test_wire_out (struct TALER_EXCHANGEDB_Session *session,
 
   coin_value_wt = deposit->amount_with_fee;
   coin_fee_wt = fee_deposit;
-  GNUNET_assert (GNUNET_OK ==
+  GNUNET_assert (0 <
                  TALER_amount_subtract (&transfer_value_wt,
                                         &coin_value_wt,
                                         &coin_fee_wt));
@@ -1699,7 +1746,7 @@ run (void *cls)
   deposit.coin.denom_sig = cbc.sig;
   deadline = GNUNET_TIME_absolute_get ();
   (void) GNUNET_TIME_round_abs (&deadline);
-  FAILIF (GNUNET_DB_STATUS_SUCCESS_ONE_RESULT !=
+  FAILIF (TALER_EXCHANGEDB_CKS_ADDED !=
           plugin->ensure_coin_known (plugin->cls,
                                      session,
                                      &deposit.coin));
@@ -1728,7 +1775,7 @@ run (void *cls)
             plugin->reserves_get (plugin->cls,
                                   session,
                                   &post_reserve));
-    FAILIF (GNUNET_OK !=
+    FAILIF (0 >=
             TALER_amount_subtract (&delta,
                                    &post_reserve.balance,
                                    &pre_reserve.balance));
@@ -1743,7 +1790,7 @@ run (void *cls)
                                                  &recoup_cb,
                                                  &coin_blind));
 
-  GNUNET_assert (GNUNET_OK ==
+  GNUNET_assert (0 <=
                  TALER_amount_add (&amount_with_fee,
                                    &value,
                                    &value));
@@ -1785,7 +1832,7 @@ run (void *cls)
       FAILIF (0 != memcmp (&bt->reserve_pub,
                            &reserve_pub,
                            sizeof (reserve_pub)));
-      /* this is the amount we trasferred twice*/
+      /* this is the amount we transferred twice*/
       FAILIF (1 != bt->amount.value);
       FAILIF (1000 != bt->amount.fraction);
       FAILIF (0 != strcmp (CURRENCY, bt->amount.currency));
@@ -1874,19 +1921,31 @@ run (void *cls)
   deposit.refund_deadline = deadline;
   deposit.wire_deadline = deadline;
   result = 8;
-  FAILIF (GNUNET_DB_STATUS_SUCCESS_ONE_RESULT !=
+  FAILIF (TALER_EXCHANGEDB_CKS_ADDED !=
           plugin->ensure_coin_known (plugin->cls,
                                      session,
                                      &deposit.coin));
-  FAILIF (GNUNET_DB_STATUS_SUCCESS_ONE_RESULT !=
-          plugin->insert_deposit (plugin->cls,
+  {
+    struct GNUNET_TIME_Absolute now;
+    struct GNUNET_TIME_Absolute r;
+    struct TALER_Amount deposit_fee;
+
+    now = GNUNET_TIME_absolute_get ();
+    GNUNET_TIME_round_abs (&now);
+    FAILIF (GNUNET_DB_STATUS_SUCCESS_ONE_RESULT !=
+            plugin->insert_deposit (plugin->cls,
+                                    session,
+                                    now,
+                                    &deposit));
+    FAILIF (1 !=
+            plugin->have_deposit (plugin->cls,
                                   session,
-                                  &deposit));
-  FAILIF (1 !=
-          plugin->have_deposit (plugin->cls,
-                                session,
-                                &deposit,
-                                GNUNET_YES));
+                                  &deposit,
+                                  GNUNET_YES,
+                                  &deposit_fee,
+                                  &r));
+    FAILIF (now.abs_value_us != r.abs_value_us);
+  }
   {
     struct GNUNET_TIME_Absolute start_range;
     struct GNUNET_TIME_Absolute end_range;
@@ -1918,7 +1977,7 @@ run (void *cls)
                                              session,
                                              &deposit.h_wire,
                                              &deposit.merchant_pub,
-                                             &deposit_cb,
+                                             &matching_deposit_cb,
                                              &deposit,
                                              2));
   sleep (2); /* giv deposit time to be ready */
@@ -1983,18 +2042,27 @@ run (void *cls)
                          session,
                          "test-2"));
   RND_BLK (&deposit2.merchant_pub); /* should fail if merchant is different */
-  FAILIF (GNUNET_DB_STATUS_SUCCESS_NO_RESULTS !=
-          plugin->have_deposit (plugin->cls,
-                                session,
-                                &deposit2,
-                                GNUNET_YES));
-  deposit2.merchant_pub = deposit.merchant_pub;
-  RND_BLK (&deposit2.coin.coin_pub); /* should fail if coin is different */
-  FAILIF (GNUNET_DB_STATUS_SUCCESS_NO_RESULTS !=
-          plugin->have_deposit (plugin->cls,
-                                session,
-                                &deposit2,
-                                GNUNET_YES));
+  {
+    struct GNUNET_TIME_Absolute r;
+    struct TALER_Amount deposit_fee;
+
+    FAILIF (GNUNET_DB_STATUS_SUCCESS_NO_RESULTS !=
+            plugin->have_deposit (plugin->cls,
+                                  session,
+                                  &deposit2,
+                                  GNUNET_YES,
+                                  &deposit_fee,
+                                  &r));
+    deposit2.merchant_pub = deposit.merchant_pub;
+    RND_BLK (&deposit2.coin.coin_pub); /* should fail if coin is different */
+    FAILIF (GNUNET_DB_STATUS_SUCCESS_NO_RESULTS !=
+            plugin->have_deposit (plugin->cls,
+                                  session,
+                                  &deposit2,
+                                  GNUNET_YES,
+                                  &deposit_fee,
+                                  &r));
+  }
   FAILIF (GNUNET_OK !=
           test_melting (session));
   FAILIF (GNUNET_DB_STATUS_SUCCESS_NO_RESULTS !=
