@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2014-2018 Taler Systems SA
+  Copyright (C) 2014-2020 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU Affero General Public License as published by the Free Software
@@ -49,7 +49,7 @@ struct TALER_AUDITOR_VersionInformation
    * semantics of GNU libtool.  See
    * https://www.gnu.org/software/libtool/manual/html_node/Versioning.html#Versioning
    */
-  char *version;
+  const char *version;
 
 };
 
@@ -106,18 +106,59 @@ enum TALER_AUDITOR_VersionCompatibility
 };
 
 
+/**
+ * General information about the HTTP response we obtained
+ * from the auditor for a request.
+ */
+struct TALER_AUDITOR_HttpResponse
+{
+
+  /**
+   * The complete JSON reply. NULL if we failed to parse the
+   * reply (too big, invalid JSON).
+   */
+  const json_t *reply;
+
+  /**
+   * Set to the human-readable 'hint' that is optionally
+   * provided by the exchange together with errors. NULL
+   * if no hint was provided or if there was no error.
+   */
+  const char *hint;
+
+  /**
+   * HTTP status code for the response.  0 if the
+   * HTTP request failed and we did not get any answer, or
+   * if the answer was invalid and we set @a ec to a
+   * client-side error code.
+   */
+  unsigned int http_status;
+
+  /**
+   * Taler error code.  #TALER_EC_NONE if everything was
+   * OK.  Usually set to the "code" field of an error
+   * response, but may be set to values created at the
+   * client side, for example when the response was
+   * not in JSON format or was otherwise ill-formed.
+   */
+  enum TALER_ErrorCode ec;
+
+};
+
+
 /**
  * Function called with information about the auditor.
  *
  * @param cls closure
+ * @param hr HTTP response data
  * @param vi basic information about the auditor
  * @param compat protocol compatibility information
  */
 typedef void
 (*TALER_AUDITOR_VersionCallback) (
   void *cls,
-  const struct
-  TALER_AUDITOR_VersionInformation *vi,
+  const struct TALER_AUDITOR_HttpResponse *hr,
+  const struct TALER_AUDITOR_VersionInformation *vi,
   enum TALER_AUDITOR_VersionCompatibility compat);
 
 
@@ -169,15 +210,12 @@ struct TALER_AUDITOR_DepositConfirmationHandle;
  * auditor's /deposit-confirmation handler.
  *
  * @param cls closure
- * @param http_status HTTP status code, 200 on success
- * @param ec taler protocol error status code, 0 on success
- * @param json raw json response
+ * @param hr HTTP response data
  */
 typedef void
-(*TALER_AUDITOR_DepositConfirmationResultCallback)(void *cls,
-                                                   unsigned int http_status,
-                                                   enum TALER_ErrorCode ec,
-                                                   const json_t *json);
+(*TALER_AUDITOR_DepositConfirmationResultCallback)(
+  void *cls,
+  const struct TALER_AUDITOR_HttpResponse *hr);
 
 
 /**
@@ -196,7 +234,7 @@ typedef void
  * @param auditor the auditor handle; the auditor must be ready to operate
  * @param h_wire hash of merchant wire details
  * @param h_contract_terms hash of the contact of the merchant with the customer (further details are never disclosed to the auditor)
- * @param timestamp timestamp when the contract was finalized, must not be too far in the future
+ * @param exchange_timestamp timestamp when the contract was finalized, must not be too far in the future
  * @param refund_deadline date until which the merchant can issue a refund to the customer via the auditor (can be zero if refunds are not allowed); must not be after the @a wire_deadline
  * @param amount_without_fee the amount confirmed to be wired by the exchange to the merchant
  * @param coin_pub coin’s public key
@@ -271,23 +309,19 @@ struct TALER_AUDITOR_ExchangeInfo
 
 
 /**
- * Function called with the result from /exchagnes.
+ * Function called with the result from /exchanges.
  *
  * @param cls closure
- * @param http_status the HTTP status code, 200 on success
- * @param ec detailed Taler error status code, #TALER_EC_NONE on success
+ * @param hr HTTP response data
  * @param num_exchanges length of array at @a ei
  * @param ei information about exchanges returned by the auditor
- * @param raw_response raw JSON response
  */
 typedef void
 (*TALER_AUDITOR_ListExchangesResultCallback)(
   void *cls,
-  unsigned int http_status,
-  enum TALER_ErrorCode ec,
+  const struct TALER_AUDITOR_HttpResponse *hr,
   unsigned int num_exchanges,
-  const struct TALER_AUDITOR_ExchangeInfo *ei,
-  const json_t *raw_response);
+  const struct TALER_AUDITOR_ExchangeInfo *ei);
 
 /**
  * Submit an /exchanges request to the auditor and get the
