@@ -300,9 +300,9 @@ struct TALER_AUDITORDB_DepositConfirmation
   struct GNUNET_HashCode h_wire;
 
   /**
-   * Time when this confirmation was generated.
+   * Time when this deposit confirmation was generated by the exchange.
    */
-  struct GNUNET_TIME_Absolute timestamp;
+  struct GNUNET_TIME_Absolute exchange_timestamp;
 
   /**
    * How much time does the @e merchant have to issue a refund
@@ -364,8 +364,9 @@ struct TALER_AUDITORDB_DepositConfirmation
  * @param cls closure
  * @param serial_id location of the @a dc in the database
  * @param dc the deposit confirmation itself
+ * @return #GNUNET_OK to continue to iterate, #GNUNET_SYSERR to stop iterating
  */
-typedef void
+typedef int
 (*TALER_AUDITORDB_DepositConfirmationCallback)(
   void *cls,
   uint64_t serial_id,
