@@ -232,7 +232,7 @@ TALER_BANK_transfer (struct GNUNET_CURL_Context *ctx,
  * succeed.
  *
  * The caller MUST run #TALER_BANK_transfer() again for the same request as
- * soon as possilbe, to ensure that the request either ultimately succeeds or
+ * soon as possible, to ensure that the request either ultimately succeeds or
  * ultimately fails. Until this has been done, the transaction is in limbo
  * (i.e. may or may not have been committed).
  *
@@ -384,16 +384,16 @@ struct TALER_BANK_DebitDetails
   const char *exchange_base_url;
 
   /**
-   * payto://-URL of the source account that
+   * payto://-URI of the source account that
    * send the funds.
    */
-  const char *debit_account_url;
+  const char *debit_account_url; // FIXME: rename: url->uri
 
   /**
-   * payto://-URL of the target account that
+   * payto://-URI of the target account that
    * received the funds.
    */
-  const char *credit_account_url;
+  const char *credit_account_url; // FIXME: rename: url->uri
 
 };
 
