@@ -296,6 +296,19 @@ struct TALER_RefreshCommitmentP
 };
 
 
+/**
+ * Token used for access control to the merchant's unclaimed
+ * orders.
+ */
+struct TALER_ClaimTokenP
+{
+  /**
+   * The token is a 128-bit UUID.
+   */
+  struct GNUNET_Uuid token;
+};
+
+
 GNUNET_NETWORK_STRUCT_END
 
 
@@ -398,7 +411,7 @@ struct TALER_TrackTransferDetails
  * @param denom_pub denomination key, must match @a coin_public_info's `denom_pub_hash`
  * @return #GNUNET_YES if the coin is valid,
  *         #GNUNET_NO if it is invalid
- *         #GNUNET_SYSERR if an internal error occured
+ *         #GNUNET_SYSERR if an internal error occurred
  */
 int
 TALER_test_coin_valid (const struct TALER_CoinPublicInfo *coin_public_info,
@@ -445,13 +458,6 @@ struct TALER_PlanchetDetail
    */
   struct GNUNET_HashCode denom_pub_hash;
 
-  /**
-   * Hash of the coin's public key.  Kept around so we do not need to
-   * compute it again.  Can be recomputed by hashing the public key
-   * of @a coin_priv if storage is at a premium.
-   */
-  struct GNUNET_HashCode c_hash;
-
   /**
    * Blinded coin (see GNUNET_CRYPTO_rsa_blind()).  Note: is malloc()'ed!
    */
@@ -589,6 +595,7 @@ TALER_planchet_setup_random (struct TALER_PlanchetSecretsP *ps);
  *
  * @param dk denomination key for the coin to be created
  * @param ps secret planchet internals (for #TALER_planchet_to_coin)
+ * @param[out] c_hash set to the hash of the public key of the coin (needed later)
  * @param[out] pd set to the planchet detail for TALER_MERCHANT_tip_pickup() and
  *               other withdraw operations
  * @return #GNUNET_OK on success
@@ -596,6 +603,7 @@ TALER_planchet_setup_random (struct TALER_PlanchetSecretsP *ps);
 int
 TALER_planchet_prepare (const struct TALER_DenominationPublicKey *dk,
                         const struct TALER_PlanchetSecretsP *ps,
+                        struct GNUNET_HashCode *c_hash,
                         struct TALER_PlanchetDetail *pd);
 
 
