@@ -351,11 +351,52 @@ enum TALER_EXCHANGE_VersionCompatibility
 };
 
 
+/**
+ * General information about the HTTP response we obtained
+ * from the exchange for a request.
+ */
+struct TALER_EXCHANGE_HttpResponse
+{
+
+  /**
+   * The complete JSON reply. NULL if we failed to parse the
+   * reply (too big, invalid JSON).
+   */
+  const json_t *reply;
+
+  /**
+   * Set to the human-readable 'hint' that is optionally
+   * provided by the exchange together with errors. NULL
+   * if no hint was provided or if there was no error.
+   */
+  const char *hint;
+
+  /**
+   * HTTP status code for the response.  0 if the
+   * HTTP request failed and we did not get any answer, or
+   * if the answer was invalid and we set @a ec to a
+   * client-side error code.
+   */
+  unsigned int http_status;
+
+  /**
+   * Taler error code.  #TALER_EC_NONE if everything was
+   * OK.  Usually set to the "code" field of an error
+   * response, but may be set to values created at the
+   * client side, for example when the response was
+   * not in JSON format or was otherwise ill-formed.
+   */
+  enum TALER_ErrorCode ec;
+
+};
+
+
 /**
  * Function called with information about who is auditing
  * a particular exchange and what key the exchange is using.
  *
  * @param cls closure
+ * @param hr HTTP response data
  * @param keys information about the various keys used
  *        by the exchange, NULL if /keys failed
  * @param compat protocol compatibility information
@@ -363,6 +404,7 @@ enum TALER_EXCHANGE_VersionCompatibility
 typedef void
 (*TALER_EXCHANGE_CertificationCallback) (
   void *cls,
+  const struct TALER_EXCHANGE_HttpResponse *hr,
   const struct TALER_EXCHANGE_Keys *keys,
   enum TALER_EXCHANGE_VersionCompatibility compat);
 
@@ -461,18 +503,50 @@ TALER_EXCHANGE_set_last_denom (struct TALER_EXCHANGE_Handle *exchange,
                                struct GNUNET_TIME_Absolute last_denom_new);
 
 
+/**
+ * Flags for #TALER_EXCHANGE_check_keys_current().
+ */
+enum TALER_EXCHANGE_CheckKeysFlags
+{
+  /**
+   * No special options.
+   */
+  TALER_EXCHANGE_CKF_NONE,
+
+  /**
+   * Force downloading /keys now, even if /keys is still valid
+   * (that is, the period advertised by the exchange for re-downloads
+   * has not yet expired).
+   */
+  TALER_EXCHANGE_CKF_FORCE_DOWNLOAD = 1,
+
+  /**
+   * Pull all keys again, resetting the client state to the original state.
+   * Using this flag disables the incremental download, and also prevents using
+   * the context until the re-download has completed.
+   */
+  TALER_EXCHANGE_CKF_PULL_ALL_KEYS = 2,
+
+  /**
+   * Force downloading all keys now.
+   */
+  TALER_EXCHANGE_CKF_FORCE_ALL_NOW = TALER_EXCHANGE_CKF_FORCE_DOWNLOAD
+                                     | TALER_EXCHANGE_CKF_PULL_ALL_KEYS
+
+};
+
+
 /**
  * Check if our current response for /keys is valid, and if
  * not, trigger /keys download.
  *
  * @param exchange exchange to check keys for
- * @param force_download #GNUNET_YES to force download even if /keys is still valid
- * @return until when the response is current, 0 if we are re-downloading
+ * @param flags options controlling when to download what
+ * @return until when the existing response is current, 0 if we are re-downloading now
  */
 struct GNUNET_TIME_Absolute
 TALER_EXCHANGE_check_keys_current (struct TALER_EXCHANGE_Handle *exchange,
-                                   int force_download,
-                                   int pull_all_keys);
+                                   enum TALER_EXCHANGE_CheckKeysFlags flags);
 
 
 /**
@@ -644,17 +718,14 @@ struct TALER_EXCHANGE_WireAccount
  * exchange, @a http_status will also be zero.
  *
  * @param cls closure
- * @param http_status HTTP response code, #MHD_HTTP_OK (200) for successful request;
- *                    0 if the exchange's reply is bogus (fails to follow the protocol)
- * @param ec taler-specific error code, #TALER_EC_NONE on success
+ * @param hr HTTP response data
  * @param accounts_len length of the @a accounts array
  * @param accounts list of wire accounts of the exchange, NULL on error
  */
 typedef void
 (*TALER_EXCHANGE_WireCallback) (
   void *cls,
-  unsigned int http_status,
-  enum TALER_ErrorCode ec,
+  const struct TALER_EXCHANGE_HttpResponse *hr,
   unsigned int accounts_len,
   const struct TALER_EXCHANGE_WireAccount *accounts);
 
@@ -703,6 +774,34 @@ TALER_EXCHANGE_wire_cancel (struct TALER_EXCHANGE_WireHandle *wh);
 /* *********************  /coins/$COIN_PUB/deposit *********************** */
 
 
+/**
+ * Sign a deposit permission.  Function for wallets.
+ *
+ * @param amount the amount to be deposited
+ * @param deposit_fee the deposit fee we expect to pay
+ * @param h_wire hash of the merchant’s account details
+ * @param h_contract_terms hash of the contact of the merchant with the customer (further details are never disclosed to the exchange)
+ * @param h_denom_pub hash of the coin denomination's public key
+ * @param coin_priv coin’s private key
+ * @param wallet_timestamp timestamp when the contract was finalized, must not be too far in the future
+ * @param merchant_pub the public key of the merchant (used to identify the merchant for refund requests)
+ * @param refund_deadline date until which the merchant can issue a refund to the customer via the exchange (can be zero if refunds are not allowed); must not be after the @a wire_deadline
+ * @param[out] coin_sig set to the signature made with purpose #TALER_SIGNATURE_WALLET_COIN_DEPOSIT
+ */
+void
+TALER_EXCHANGE_deposit_permission_sign (
+  const struct TALER_Amount *amount,
+  const struct TALER_Amount *deposit_fee,
+  const struct GNUNET_HashCode *h_wire,
+  const struct GNUNET_HashCode *h_contract_terms,
+  const struct GNUNET_HashCode *h_denom_pub,
+  const struct TALER_CoinSpendPrivateKeyP *coin_priv,
+  struct GNUNET_TIME_Absolute wallet_timestamp,
+  const struct TALER_MerchantPublicKeyP *merchant_pub,
+  struct GNUNET_TIME_Absolute refund_deadline,
+  struct TALER_CoinSpendSignatureP *coin_sig);
+
+
 /**
  * @brief A Deposit Handle
  */
@@ -714,21 +813,18 @@ struct TALER_EXCHANGE_DepositHandle;
  * deposit permission request to a exchange.
  *
  * @param cls closure
- * @param http_status HTTP response code, #MHD_HTTP_OK (200) for successful deposit;
- *                    0 if the exchange's reply is bogus (fails to follow the protocol)
+ * @param hr HTTP response data
+ * @param deposit_timestamp time when the exchange generated the deposit confirmation
  * @param exchange_sig signature provided by the exchange
- * @param sign_key exchange key used to sign @a obj, or NULL
- * @param obj the received JSON reply, should be kept as proof (and, in case of errors,
- *            be forwarded to the customer)
+ * @param exchange_pub exchange key used to sign @a obj, or NULL
  */
 typedef void
 (*TALER_EXCHANGE_DepositResultCallback) (
   void *cls,
-  unsigned int http_status,
-  enum TALER_ErrorCode ec,
+  const struct TALER_EXCHANGE_HttpResponse *hr,
+  struct GNUNET_TIME_Absolute deposit_timestamp,
   const struct TALER_ExchangeSignatureP *exchange_sig,
-  const struct TALER_ExchangePublicKeyP *sign_key,
-  const json_t *obj);
+  const struct TALER_ExchangePublicKeyP *exchange_pub);
 
 
 /**
@@ -814,30 +910,25 @@ struct TALER_EXCHANGE_RefundHandle;
  * refund request to an exchange.
  *
  * @param cls closure
- * @param http_status HTTP response code, #MHD_HTTP_OK (200) for successful deposit;
- *                    0 if the exchange's reply is bogus (fails to follow the protocol)
- * @param ec taler-specific error code, #TALER_EC_NONE on success
+ * @param hr HTTP response data
  * @param sign_key exchange key used to sign @a obj, or NULL
- * @param obj the received JSON reply, should be kept as proof (and, in particular,
- *            be forwarded to the customer)
+ * @param signature the actual signature, or NULL on error
  */
 typedef void
 (*TALER_EXCHANGE_RefundCallback) (
   void *cls,
-  unsigned int http_status,
-  enum TALER_ErrorCode ec,
+  const struct TALER_EXCHANGE_HttpResponse *hr,
   const struct TALER_ExchangePublicKeyP *sign_key,
-  const json_t *obj);
+  const struct TALER_ExchangeSignatureP *signature);
 
 
 /**
- * Submit a refund request to the exchange and get the exchange's
- * response.  This API is used by a merchant.  Note that
- * while we return the response verbatim to the caller for further
- * processing, we do already verify that the response is well-formed
- * (i.e. that signatures included in the response are all valid).  If
- * the exchange's reply is not well-formed, we return an HTTP status code
- * of zero to @a cb.
+ * Submit a refund request to the exchange and get the exchange's response.
+ * This API is used by a merchant.  Note that while we return the response
+ * verbatim to the caller for further processing, we do already verify that
+ * the response is well-formed (i.e. that signatures included in the response
+ * are all valid).  If the exchange's reply is not well-formed, we return an
+ * HTTP status code of zero to @a cb.
  *
  * The @a exchange must be ready to operate (i.e.  have
  * finished processing the /keys reply).  If this check fails, we do
@@ -847,7 +938,6 @@ typedef void
  * @param amount the amount to be refunded; must be larger than the refund fee
  *        (as that fee is still being subtracted), and smaller than the amount
  *        (with deposit fee) of the original deposit contribution of this coin
- * @param refund_fee fee applicable to this coin for the refund
  * @param h_contract_terms hash of the contact of the merchant with the customer that is being refunded
  * @param coin_pub coin’s public key of the coin from the original deposit operation
  * @param rtransaction_id transaction id for the transaction between merchant and customer (of refunding operation);
@@ -863,7 +953,6 @@ typedef void
 struct TALER_EXCHANGE_RefundHandle *
 TALER_EXCHANGE_refund (struct TALER_EXCHANGE_Handle *exchange,
                        const struct TALER_Amount *amount,
-                       const struct TALER_Amount *refund_fee,
                        const struct GNUNET_HashCode *h_contract_terms,
                        const struct TALER_CoinSpendPublicKeyP *coin_pub,
                        uint64_t rtransaction_id,
@@ -872,50 +961,6 @@ TALER_EXCHANGE_refund (struct TALER_EXCHANGE_Handle *exchange,
                        void *cb_cls);
 
 
-/**
- * Submit a refund request to the exchange and get the exchange's
- * response.  This API is used by a merchant.  Note that
- * while we return the response verbatim to the caller for further
- * processing, we do already verify that the response is well-formed
- * (i.e. that signatures included in the response are all valid).  If
- * the exchange's reply is not well-formed, we return an HTTP status code
- * of zero to @a cb.
- *
- * The @a exchange must be ready to operate (i.e.  have
- * finished processing the /keys reply).  If this check fails, we do
- * NOT initiate the transaction with the exchange and instead return NULL.
- *
- * @param exchange the exchange handle; the exchange must be ready to operate
- * @param amount the amount to be refunded; must be larger than the refund fee
- *        (as that fee is still being subtracted), and smaller than the amount
- *        (with deposit fee) of the original deposit contribution of this coin
- * @param refund_fee fee applicable to this coin for the refund
- * @param h_contract_terms hash of the contact of the merchant with the customer that is being refunded
- * @param coin_pub coin’s public key of the coin from the original deposit operation
- * @param rtransaction_id transaction id for the transaction between merchant and customer (of refunding operation);
- *                        this is needed as we may first do a partial refund and later a full refund.  If both
- *                        refunds are also over the same amount, we need the @a rtransaction_id to make the disjoint
- *                        refund requests different (as requests are idempotent and otherwise the 2nd refund might not work).
- * @param merchant_pub public key of the merchant
- * @param merchant_sig signature affirming the refund from the merchant
- * @param cb the callback to call when a reply for this request is available
- * @param cb_cls closure for the above callback
- * @return a handle for this request; NULL if the inputs are invalid (i.e.
- *         signatures fail to verify).  In this case, the callback is not called.
- */
-struct TALER_EXCHANGE_RefundHandle *
-TALER_EXCHANGE_refund2 (struct TALER_EXCHANGE_Handle *exchange,
-                        const struct TALER_Amount *amount,
-                        const struct TALER_Amount *refund_fee,
-                        const struct GNUNET_HashCode *h_contract_terms,
-                        const struct TALER_CoinSpendPublicKeyP *coin_pub,
-                        uint64_t rtransaction_id,
-                        const struct TALER_MerchantPublicKeyP *merchant_pub,
-                        const struct TALER_MerchantSignatureP *merchant_sig,
-                        TALER_EXCHANGE_RefundCallback cb,
-                        void *cb_cls);
-
-
 /**
  * Cancel a refund permission request.  This function cannot be used
  * on a request handle if a response is already served for it.  If
@@ -1112,10 +1157,7 @@ struct TALER_EXCHANGE_ReserveHistory
  * reserve status request to a exchange.
  *
  * @param cls closure
- * @param http_status HTTP response code, #MHD_HTTP_OK (200) for successful status request
- *                    0 if the exchange's reply is bogus (fails to follow the protocol)
- * @param ec taler-specific error code, #TALER_EC_NONE on success
- * @param json original response in JSON format (useful only for diagnostics)
+ * @param hr HTTP response data
  * @param balance current balance in the reserve, NULL on error
  * @param history_length number of entries in the transaction history, 0 on error
  * @param history detailed transaction history, NULL on error
@@ -1123,11 +1165,8 @@ struct TALER_EXCHANGE_ReserveHistory
 typedef void
 (*TALER_EXCHANGE_ReservesGetCallback) (
   void *cls,
-  unsigned int http_status,
-  enum TALER_ErrorCode ec,
-  const json_t *json,
-  const struct
-  TALER_Amount *balance,
+  const struct TALER_EXCHANGE_HttpResponse *hr,
+  const struct TALER_Amount *balance,
   unsigned int history_length,
   const struct TALER_EXCHANGE_ReserveHistory *history);
 
@@ -1181,19 +1220,14 @@ struct TALER_EXCHANGE_WithdrawHandle;
  * withdraw request to a exchange.
  *
  * @param cls closure
- * @param http_status HTTP response code, #MHD_HTTP_OK (200) for successful status request
- *                    0 if the exchange's reply is bogus (fails to follow the protocol)
- * @param ec taler-specific error code, #TALER_EC_NONE on success
+ * @param hr HTTP response data
  * @param sig signature over the coin, NULL on error
- * @param full_response full response from the exchange (for logging, in case of errors)
  */
 typedef void
 (*TALER_EXCHANGE_WithdrawCallback) (
   void *cls,
-  unsigned int http_status,
-  enum TALER_ErrorCode ec,
-  const struct TALER_DenominationSignature *sig,
-  const json_t *full_response);
+  const struct TALER_EXCHANGE_HttpResponse *hr,
+  const struct TALER_DenominationSignature *sig);
 
 
 /**
@@ -1227,10 +1261,45 @@ TALER_EXCHANGE_withdraw (
   void *res_cb_cls);
 
 
+/**
+ * Cancel a withdraw status request.  This function cannot be used
+ * on a request handle if a response is already served for it.
+ *
+ * @param wh the withdraw handle
+ */
+void
+TALER_EXCHANGE_withdraw_cancel (struct TALER_EXCHANGE_WithdrawHandle *wh);
+
+
+/**
+ * Callbacks of this type are used to serve the result of submitting a
+ * withdraw request to a exchange without the (un)blinding factor.
+ *
+ * @param cls closure
+ * @param hr HTTP response data
+ * @param blind_sig blind signature over the coin, NULL on error
+ */
+typedef void
+(*TALER_EXCHANGE_Withdraw2Callback) (
+  void *cls,
+  const struct TALER_EXCHANGE_HttpResponse *hr,
+  const struct GNUNET_CRYPTO_RsaSignature *blind_sig);
+
+
+/**
+ * @brief A /reserves/$RESERVE_PUB/withdraw Handle, 2nd variant.
+ * This variant does not do the blinding/unblinding and only
+ * fetches the blind signature on the already blinded planchet.
+ * Used internally by the `struct TALER_EXCHANGE_WithdrawHandle`
+ * implementation as well as for the tipping logic of merchants.
+ */
+struct TALER_EXCHANGE_Withdraw2Handle;
+
+
 /**
  * Withdraw a coin from the exchange using a /reserves/$RESERVE_PUB/withdraw
- * request.  This API is typically used by a wallet to withdraw a tip
- * where the reserve's signature was created by the merchant already.
+ * request.  This API is typically used by a merchant to withdraw a tip
+ * where the blinding factor is unknown to the merchant.
  *
  * Note that to ensure that no money is lost in case of hardware
  * failures, the caller must have committed (most of) the arguments to
@@ -1238,26 +1307,20 @@ TALER_EXCHANGE_withdraw (
  * same arguments in case of failures.
  *
  * @param exchange the exchange handle; the exchange must be ready to operate
- * @param pk kind of coin to create
- * @param reserve_sig signature from the reserve authorizing the withdrawal
- * @param reserve_pub public key of the reserve to withdraw from
- * @param ps secrets of the planchet
- *        caller must have committed this value to disk before the call (with @a pk)
+ * @param pd planchet details of the planchet to withdraw
+ * @param reserve_priv private key of the reserve to withdraw from
  * @param res_cb the callback to call when the final result for this request is available
  * @param res_cb_cls closure for @a res_cb
  * @return NULL
  *         if the inputs are invalid (i.e. denomination key not with this exchange).
  *         In this case, the callback is not called.
  */
-struct TALER_EXCHANGE_WithdrawHandle *
-TALER_EXCHANGE_withdraw2 (
-  struct TALER_EXCHANGE_Handle *exchange,
-  const struct TALER_EXCHANGE_DenomPublicKey *pk,
-  const struct TALER_ReserveSignatureP *reserve_sig,
-  const struct TALER_ReservePublicKeyP *reserve_pub,
-  const struct TALER_PlanchetSecretsP *ps,
-  TALER_EXCHANGE_WithdrawCallback res_cb,
-  void *res_cb_cls);
+struct TALER_EXCHANGE_Withdraw2Handle *
+TALER_EXCHANGE_withdraw2 (struct TALER_EXCHANGE_Handle *exchange,
+                          const struct TALER_PlanchetDetail *pd,
+                          const struct TALER_ReservePrivateKeyP *reserve_priv,
+                          TALER_EXCHANGE_Withdraw2Callback res_cb,
+                          void *res_cb_cls);
 
 
 /**
@@ -1267,7 +1330,7 @@ TALER_EXCHANGE_withdraw2 (
  * @param wh the withdraw handle
  */
 void
-TALER_EXCHANGE_withdraw_cancel (struct TALER_EXCHANGE_WithdrawHandle *wh);
+TALER_EXCHANGE_withdraw2_cancel (struct TALER_EXCHANGE_Withdraw2Handle *wh);
 
 
 /* ********************* /refresh/melt+reveal ***************************** */
@@ -1337,21 +1400,17 @@ struct TALER_EXCHANGE_MeltHandle;
  * #TALER_EXCHANGE_refreshes_reveal().
  *
  * @param cls closure
- * @param http_status HTTP response code, never #MHD_HTTP_OK (200) as for successful intermediate response this callback is skipped.
- *                    0 if the exchange's reply is bogus (fails to follow the protocol)
- * @param ec taler-specific error code, #TALER_EC_NONE on success
+ * @param hr HTTP response data
  * @param noreveal_index choice by the exchange in the cut-and-choose protocol,
  *                    UINT32_MAX on error
  * @param sign_key exchange key used to sign @a full_response, or NULL
- * @param full_response full response from the exchange (for logging, in case of errors)
  */
 typedef void
-(*TALER_EXCHANGE_MeltCallback) (void *cls,
-                                unsigned int http_status,
-                                enum TALER_ErrorCode ec,
-                                uint32_t noreveal_index,
-                                const struct TALER_ExchangePublicKeyP *sign_key,
-                                const json_t *full_response);
+(*TALER_EXCHANGE_MeltCallback) (
+  void *cls,
+  const struct TALER_EXCHANGE_HttpResponse *hr,
+  uint32_t noreveal_index,
+  const struct TALER_ExchangePublicKeyP *sign_key);
 
 
 /**
@@ -1404,23 +1463,18 @@ TALER_EXCHANGE_melt_cancel (struct TALER_EXCHANGE_MeltHandle *mh);
  * the original request specified the respective denomination keys.
  *
  * @param cls closure
- * @param http_status HTTP response code, #MHD_HTTP_OK (200) for successful status request
- *                    0 if the exchange's reply is bogus (fails to follow the protocol)
- * @param ec taler-specific error code, #TALER_EC_NONE on success
+ * @param hr HTTP response data
  * @param num_coins number of fresh coins created, length of the @a sigs and @a coin_privs arrays, 0 if the operation failed
  * @param coin_privs array of @a num_coins private keys for the coins that were created, NULL on error
  * @param sigs array of signature over @a num_coins coins, NULL on error
- * @param full_response full response from the exchange (for logging, in case of errors)
  */
 typedef void
 (*TALER_EXCHANGE_RefreshesRevealCallback)(
   void *cls,
-  unsigned int http_status,
-  enum TALER_ErrorCode ec,
+  const struct TALER_EXCHANGE_HttpResponse *hr,
   unsigned int num_coins,
   const struct TALER_PlanchetSecretsP *coin_privs,
-  const struct TALER_DenominationSignature *sigs,
-  const json_t *full_response);
+  const struct TALER_DenominationSignature *sigs);
 
 
 /**
@@ -1488,25 +1542,20 @@ struct TALER_EXCHANGE_LinkHandle;
  * created when the original coin was melted.
  *
  * @param cls closure
- * @param http_status HTTP response code, #MHD_HTTP_OK (200) for successful status request
- *                    0 if the exchange's reply is bogus (fails to follow the protocol)
- * @param ec taler-specific error code, #TALER_EC_NONE on success
+ * @param hr HTTP response data
  * @param num_coins number of fresh coins created, length of the @a sigs and @a coin_privs arrays, 0 if the operation failed
  * @param coin_privs array of @a num_coins private keys for the coins that were created, NULL on error
  * @param sigs array of signature over @a num_coins coins, NULL on error
  * @param pubs array of public keys for the @a sigs, NULL on error
- * @param full_response full response from the exchange (for logging, in case of errors)
  */
 typedef void
 (*TALER_EXCHANGE_LinkCallback) (
   void *cls,
-  unsigned int http_status,
-  enum TALER_ErrorCode ec,
+  const struct TALER_EXCHANGE_HttpResponse *hr,
   unsigned int num_coins,
   const struct TALER_CoinSpendPrivateKeyP *coin_privs,
   const struct TALER_DenominationSignature *sigs,
-  const struct TALER_DenominationPublicKey *pubs,
-  const json_t *full_response);
+  const struct TALER_DenominationPublicKey *pubs);
 
 
 /**
@@ -1547,37 +1596,68 @@ TALER_EXCHANGE_link_cancel (struct TALER_EXCHANGE_LinkHandle *lh);
 struct TALER_EXCHANGE_TransfersGetHandle;
 
 
+/**
+ * Information the exchange returns per wire transfer.
+ */
+struct TALER_EXCHANGE_TransferData
+{
+
+  /**
+   * exchange key used to sign
+   */
+  struct TALER_ExchangePublicKeyP exchange_pub;
+
+  /**
+   * exchange signature over the transfer data
+   */
+  struct TALER_ExchangeSignatureP exchange_sig;
+
+  /**
+   * hash of the wire transfer address the transfer went to
+   */
+  struct GNUNET_HashCode h_wire;
+
+  /**
+   * time when the exchange claims to have performed the wire transfer
+   */
+  struct GNUNET_TIME_Absolute execution_time;
+
+  /**
+   * Actual amount of the wire transfer, excluding the wire fee.
+   */
+  struct TALER_Amount total_amount;
+
+  /**
+   * wire fee that was charged by the exchange
+   */
+  struct TALER_Amount wire_fee;
+
+  /**
+   * length of the @e details array
+   */
+  unsigned int details_length;
+
+  /**
+   * array with details about the combined transactions
+   */
+  const struct TALER_TrackTransferDetails *details;
+
+};
+
+
 /**
  * Function called with detailed wire transfer data, including all
  * of the coin transactions that were combined into the wire transfer.
  *
  * @param cls closure
- * @param http_status HTTP status code we got, 0 on exchange protocol violation
- * @param ec taler-specific error code, #TALER_EC_NONE on success
- * @param sign_key exchange key used to sign @a json, or NULL
- * @param json original json reply (may include signatures, those have then been
- *        validated already)
- * @param h_wire hash of the wire transfer address the transfer went to, or NULL on error
- * @param execution_time time when the exchange claims to have performed the wire transfer
- * @param total_amount total amount of the wire transfer, or NULL if the exchange could
- *             not provide any @a wtid (set only if @a http_status is #MHD_HTTP_OK)
- * @param wire_fee wire fee that was charged by the exchange
- * @param details_length length of the @a details array
- * @param details array with details about the combined transactions
+ * @param hr HTTP response data
+ * @param ta transfer data, (set only if @a http_status is #MHD_HTTP_OK, otherwise NULL)
  */
 typedef void
 (*TALER_EXCHANGE_TransfersGetCallback)(
   void *cls,
-  unsigned int http_status,
-  enum TALER_ErrorCode ec,
-  const struct TALER_ExchangePublicKeyP *sign_key,
-  const json_t *json,
-  const struct GNUNET_HashCode *h_wire,
-  struct GNUNET_TIME_Absolute execution_time,
-  const struct TALER_Amount *total_amount,
-  const struct TALER_Amount *wire_fee,
-  unsigned int details_length,
-  const struct TALER_TrackTransferDetails *details);
+  const struct TALER_EXCHANGE_HttpResponse *hr,
+  const struct TALER_EXCHANGE_TransferData *ta);
 
 
 /**
@@ -1618,31 +1698,57 @@ TALER_EXCHANGE_transfers_get_cancel (
 struct TALER_EXCHANGE_DepositGetHandle;
 
 
+/**
+ * Data returned for a successful GET /deposits/ request.  Note that
+ * most fields are only set if the status is #MHD_HTTP_OK.  Only
+ * the @e execution_time is available if the status is #MHD_HTTP_ACCEPTED.
+ */
+struct TALER_EXCHANGE_DepositData
+{
+
+  /**
+   * exchange key used to sign, all zeros if exchange did not
+   * yet execute the transaction
+   */
+  struct TALER_ExchangePublicKeyP exchange_pub;
+
+  /**
+   * signature from the exchange over the deposit data, all zeros if exchange did not
+   * yet execute the transaction
+   */
+  struct TALER_ExchangeSignatureP exchange_sig;
+
+  /**
+   * wire transfer identifier used by the exchange, all zeros if exchange did not
+   * yet execute the transaction
+   */
+  struct TALER_WireTransferIdentifierRawP wtid;
+
+  /**
+   * actual or planned execution time for the wire transfer
+   */
+  struct GNUNET_TIME_Absolute execution_time;
+
+  /**
+   * contribution to the total amount by this coin, all zeros if exchange did not
+   * yet execute the transaction
+   */
+  struct TALER_Amount coin_contribution;
+};
+
+
 /**
  * Function called with detailed wire transfer data.
  *
  * @param cls closure
- * @param http_status HTTP status code we got, 0 on exchange protocol violation
- * @param ec taler-specific error code, #TALER_EC_NONE on success
- * @param sign_key exchange key used to sign @a json, or NULL
- * @param json original json reply (may include signatures, those have then been
- *        validated already)
- * @param wtid wire transfer identifier used by the exchange, NULL if exchange did not
- *                  yet execute the transaction
- * @param execution_time actual or planned execution time for the wire transfer
- * @param coin_contribution contribution to the total amount by this coin (can be NULL)
+ * @param hr HTTP response data
+ * @param dd details about the deposit (NULL on errors)
  */
 typedef void
 (*TALER_EXCHANGE_DepositGetCallback)(
   void *cls,
-  unsigned int http_status,
-  enum TALER_ErrorCode ec,
-  const struct TALER_ExchangePublicKeyP *sign_key,
-  const json_t *json,
-  const struct TALER_WireTransferIdentifierRawP *wtid,
-  struct GNUNET_TIME_Absolute
-  execution_time,
-  const struct TALER_Amount *coin_contribution);
+  const struct TALER_EXCHANGE_HttpResponse *hr,
+  const struct TALER_EXCHANGE_DepositData *dd);
 
 
 /**
@@ -1689,6 +1795,7 @@ TALER_EXCHANGE_deposits_get_cancel (
  * @param currency expected currency for the coin
  * @param coin_pub public key of the coin
  * @param history history of the coin in json encoding
+ * @param[out] h_denom_pub set to the hash of the coin's denomination (if available)
  * @param[out] total how much of the coin has been spent according to @a history
  * @return #GNUNET_OK if @a history is valid, #GNUNET_SYSERR if not
  */
@@ -1698,6 +1805,7 @@ TALER_EXCHANGE_verify_coin_history (
   const char *currency,
   const struct TALER_CoinSpendPublicKeyP *coin_pub,
   json_t *history,
+  struct GNUNET_HashCode *h_denom_pub,
   struct TALER_Amount *total);
 
 
@@ -1758,24 +1866,19 @@ struct TALER_EXCHANGE_RecoupHandle;
  * the original request specified the respective denomination keys.
  *
  * @param cls closure
- * @param http_status HTTP response code, #MHD_HTTP_OK (200) for successful status request
- *                    0 if the exchange's reply is bogus (fails to follow the protocol)
- * @param ec taler-specific error code, #TALER_EC_NONE on success
+ * @param hr HTTP response data
  * @param amount amount the exchange will wire back for this coin,
  *        on error the total balance remaining, or NULL
  * @param timestamp what time did the exchange receive the /recoup request
  * @param reserve_pub public key of the reserve receiving the recoup, NULL if refreshed or on error
  * @param old_coin_pub public key of the dirty coin, NULL if not refreshed or on error
- * @param full_response full response from the exchange (for logging, in case of errors)
  */
 typedef void
 (*TALER_EXCHANGE_RecoupResultCallback) (
   void *cls,
-  unsigned int http_status,
-  enum TALER_ErrorCode ec,
+  const struct TALER_EXCHANGE_HttpResponse *hr,
   const struct TALER_ReservePublicKeyP *reserve_pub,
-  const struct TALER_CoinSpendPublicKeyP *old_coin_pub,
-  const json_t *full_response);
+  const struct TALER_CoinSpendPublicKeyP *old_coin_pub);
 
 
 /**
