@@ -243,7 +243,7 @@ struct TALER_EXCHANGEDB_CollectableBlindcoin
   struct TALER_Amount amount_with_fee;
 
   /**
-   * Withdrawl fee charged by the exchange.  This must match the Exchange's
+   * Withdrawal fee charged by the exchange.  This must match the Exchange's
    * denomination key's withdrawal fee.  If the client puts in an
    * invalid withdrawal fee (too high or too low) that does not match
    * the Exchange's denomination key, the withdraw operation is invalid
@@ -335,6 +335,11 @@ struct TALER_EXCHANGEDB_RecoupListEntry
    */
   struct TALER_CoinSpendSignatureP coin_sig;
 
+  /**
+   * Hash of the public denomination key used to sign the coin.
+   */
+  struct GNUNET_HashCode h_denom_pub;
+
   /**
    * Public key of the reserve the coin was paid back into.
    */
@@ -368,7 +373,7 @@ struct TALER_EXCHANGEDB_RecoupRefreshListEntry
 
   /**
    * Blinding factor supplied to prove to the exchange that
-   * the coin came from this reserve.
+   * the coin came from this @e old_coin_pub.
    */
   struct TALER_DenominationBlindingKeyP coin_blind;
 
@@ -441,7 +446,7 @@ struct TALER_EXCHANGEDB_ReserveHistory
   struct TALER_EXCHANGEDB_ReserveHistory *next;
 
   /**
-   * Type of the event, determins @e details.
+   * Type of the event, determines @e details.
    */
   enum TALER_EXCHANGEDB_ReserveOperation type;
 
@@ -608,6 +613,11 @@ struct TALER_EXCHANGEDB_DepositListEntry
    */
   struct GNUNET_HashCode h_wire;
 
+  /**
+   * Hash of the public denomination key used to sign the coin.
+   */
+  struct GNUNET_HashCode h_denom_pub;
+
   /**
    * Detailed information about the receiver for executing the transaction.
    * Includes URL in payto://-format and salt.
@@ -655,6 +665,11 @@ struct TALER_EXCHANGEDB_DepositListEntry
    */
   struct TALER_Amount deposit_fee;
 
+  /**
+   * Has the deposit been wired?
+   */
+  bool done;
+
 };
 
 
@@ -781,6 +796,11 @@ struct TALER_EXCHANGEDB_MeltListEntry
    */
   struct TALER_RefreshCommitmentP rc;
 
+  /**
+   * Hash of the public denomination key used to sign the coin.
+   */
+  struct GNUNET_HashCode h_denom_pub;
+
   /**
    * How much value is being melted?  This amount includes the fees,
    * so the final amount contributed to the melt is this value minus
@@ -975,21 +995,48 @@ struct TALER_EXCHANGEDB_Session;
  * @param cls closure
  * @param rowid unique ID for the deposit in our DB, used for marking
  *              it as 'tiny' or 'done'
+ * @param coin_pub public key of the coin
+ * @param amount_with_fee amount that was deposited including fee
+ * @param deposit_fee amount the exchange gets to keep as transaction fees
+ * @param h_contract_terms hash of the proposal data known to merchant and customer
+ * @return transaction status code, #GNUNET_DB_STATUS_SUCCESS_ONE_RESULT to continue to iterate
+ */
+typedef enum GNUNET_DB_QueryStatus
+(*TALER_EXCHANGEDB_MatchingDepositIterator)(
+  void *cls,
+  uint64_t rowid,
+  const struct TALER_CoinSpendPublicKeyP *coin_pub,
+  const struct TALER_Amount *amount_with_fee,
+  const struct TALER_Amount *deposit_fee,
+  const struct GNUNET_HashCode *h_contract_terms);
+
+
+/**
+ * Function called with details about deposits that have been made,
+ * with the goal of executing the corresponding wire transaction.
+ *
+ * @param cls closure
+ * @param rowid unique ID for the deposit in our DB, used for marking
+ *              it as 'tiny' or 'done'
+ * @param exchange_timestamp when did the exchange receive the deposit
+ * @param wallet_timestamp when did the wallet sign the contract
  * @param merchant_pub public key of the merchant
  * @param coin_pub public key of the coin
  * @param amount_with_fee amount that was deposited including fee
  * @param deposit_fee amount the exchange gets to keep as transaction fees
  * @param h_contract_terms hash of the proposal data known to merchant and customer
- * @param wire_deadline by which the merchant adviced that he would like the
+ * @param wire_deadline by which the merchant advised that he would like the
  *        wire transfer to be executed
  * @param receiver_wire_account wire details for the merchant, includes
- *        'url' in payto://-format; NULL from iterate_matching_deposits()
+ *        'url' in payto://-format;
  * @return transaction status code, #GNUNET_DB_STATUS_SUCCESS_ONE_RESULT to continue to iterate
  */
 typedef enum GNUNET_DB_QueryStatus
 (*TALER_EXCHANGEDB_DepositIterator)(
   void *cls,
   uint64_t rowid,
+  struct GNUNET_TIME_Absolute exchange_timestamp,
+  struct GNUNET_TIME_Absolute wallet_timestamp,
   const struct TALER_MerchantPublicKeyP *merchant_pub,
   const struct TALER_CoinSpendPublicKeyP *coin_pub,
   const struct TALER_Amount *amount_with_fee,
@@ -1022,16 +1069,17 @@ typedef void
  *
  * @param cls closure
  * @param rowid unique serial ID for the deposit in our DB
- * @param timestamp when did the deposit happen
+ * @param exchange_timestamp when did the deposit happen
+ * @param wallet_timestamp when did the contract happen
  * @param merchant_pub public key of the merchant
  * @param denom_pub denomination public key of @a coin_pub
  * @param coin_pub public key of the coin
  * @param coin_sig signature from the coin
  * @param amount_with_fee amount that was deposited including fee
  * @param h_contract_terms hash of the proposal data known to merchant and customer
- * @param refund_deadline by which the merchant adviced that he might want
+ * @param refund_deadline by which the merchant advised that he might want
  *        to get a refund
- * @param wire_deadline by which the merchant adviced that he would like the
+ * @param wire_deadline by which the merchant advised that he would like the
  *        wire transfer to be executed
  * @param receiver_wire_account wire details for the merchant including 'url' in payto://-format;
  *        NULL from iterate_matching_deposits()
@@ -1042,7 +1090,8 @@ typedef int
 (*TALER_EXCHANGEDB_DepositCallback)(
   void *cls,
   uint64_t rowid,
-  struct GNUNET_TIME_Absolute timestamp,
+  struct GNUNET_TIME_Absolute exchange_timestamp,
+  struct GNUNET_TIME_Absolute wallet_timestamp,
   const struct TALER_MerchantPublicKeyP *merchant_pub,
   const struct TALER_DenominationPublicKey *denom_pub,
   const struct TALER_CoinSpendPublicKeyP *coin_pub,
@@ -1797,7 +1846,33 @@ struct TALER_EXCHANGEDB_Plugin
    * @param coin the coin that must be made known
    * @return database transaction status, non-negative on success
    */
-  enum GNUNET_DB_QueryStatus
+  enum TALER_EXCHANGEDB_CoinKnownStatus
+  {
+    /**
+     * The coin was successfully added.
+     */
+    TALER_EXCHANGEDB_CKS_ADDED = 1,
+
+    /**
+     * The coin was already present.
+     */
+    TALER_EXCHANGEDB_CKS_PRESENT = 0,
+
+    /**
+     * Serialization failure.
+     */
+    TALER_EXCHANGEDB_CKS_SOFT_FAIL = -1,
+
+    /**
+     * Hard database failure.
+     */
+    TALER_EXCHANGEDB_CKS_HARD_FAIL = -2,
+
+    /**
+     * Conflicting coin (different denomination key) already in database.
+     */
+    TALER_EXCHANGEDB_CKS_CONFLICT = -3,
+  }
   (*ensure_coin_known)(void *cls,
                        struct TALER_EXCHANGEDB_Session *session,
                        const struct TALER_CoinPublicInfo *coin);
@@ -1840,7 +1915,9 @@ struct TALER_EXCHANGEDB_Plugin
    * @param cls the @e cls of this struct with the plugin-specific state
    * @param session database connection
    * @param deposit deposit to search for
-   * @param check_extras wether to check extra fields or not
+   * @param check_extras whether to check extra fields or not
+   * @param[out] deposit_fee set to the deposit fee the exchange charged
+   * @param[out] exchange_timestamp set to the time when the exchange received the deposit
    * @return 1 if we know this operation,
    *         0 if this exact deposit is unknown to us,
    *         otherwise transaction error status
@@ -1849,7 +1926,9 @@ struct TALER_EXCHANGEDB_Plugin
   (*have_deposit)(void *cls,
                   struct TALER_EXCHANGEDB_Session *session,
                   const struct TALER_EXCHANGEDB_Deposit *deposit,
-                  int check_extras);
+                  int check_extras,
+                  struct TALER_Amount *deposit_fee,
+                  struct GNUNET_TIME_Absolute *exchange_timestamp);
 
 
   /**
@@ -1857,12 +1936,14 @@ struct TALER_EXCHANGEDB_Plugin
    *
    * @param cls the @e cls of this struct with the plugin-specific state
    * @param session connection to the database
+   * @param exchange_timestamp time the exchange received the deposit request
    * @param deposit deposit information to store
    * @return query result status
    */
   enum GNUNET_DB_QueryStatus
   (*insert_deposit)(void *cls,
                     struct TALER_EXCHANGEDB_Session *session,
+                    struct GNUNET_TIME_Absolute exchange_timestamp,
                     const struct TALER_EXCHANGEDB_Deposit *deposit);
 
 
@@ -2007,7 +2088,7 @@ struct TALER_EXCHANGEDB_Plugin
     struct TALER_EXCHANGEDB_Session *session,
     const struct GNUNET_HashCode *h_wire,
     const struct TALER_MerchantPublicKeyP *merchant_pub,
-    TALER_EXCHANGEDB_DepositIterator deposit_cb,
+    TALER_EXCHANGEDB_MatchingDepositIterator deposit_cb,
     void *deposit_cb_cls,
     uint32_t limit);
 
