@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  (C) 2016 Inria and GNUnet e.V.
+  (C) 2016-2020 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU General Public License as published by the Free Software
@@ -34,24 +34,28 @@ struct TALER_FAKEBANK_Handle;
 
 
 /**
- * Start the fake bank.  The fake bank will, like the normal bank,
- * listen for requests for /admin/add/incoming. However, instead of
- * executing or storing those requests, it will simply allow querying
- * whether such a request has been made via #TALER_FAKEBANK_check().
+ * Start the fake bank.  The fake bank will, like the normal bank, listen for
+ * requests for /admin/add/incoming and /transfer. However, instead of
+ * executing or storing those requests, it will simply allow querying whether
+ * such a request has been made via #TALER_FAKEBANK_check_debit() and
+ * #TALER_FAKEBANK_check_credit() as well as the history API.
  *
  * This is useful for writing testcases to check whether the exchange
  * would have issued the correct wire transfer orders.
  *
  * @param port port to listen to
+ * @param currency which currency should the bank offer
  * @return NULL on error
  */
 struct TALER_FAKEBANK_Handle *
-TALER_FAKEBANK_start (uint16_t port);
+TALER_FAKEBANK_start (uint16_t port,
+                      const char *currency);
 
 
 /**
  * Check that no wire transfers were ordered (or at least none
- * that have not been taken care of via #TALER_FAKEBANK_check()).
+ * that have not been taken care of via #TALER_FAKEBANK_check_debit()
+ * or #TALER_FAKEBANK_check_credit()).
  * If any transactions are onrecord, return #GNUNET_SYSERR.
  *
  * @param h bank instance
@@ -62,7 +66,7 @@ TALER_FAKEBANK_check_empty (struct TALER_FAKEBANK_Handle *h);
 
 
 /**
- * Tell the fakebank to create another wire transfer.
+ * Tell the fakebank to create another wire transfer *from* an exchange.
  *
  * @param h fake bank handle
  * @param debit_account account to debit
@@ -70,15 +74,40 @@ TALER_FAKEBANK_check_empty (struct TALER_FAKEBANK_Handle *h);
  * @param amount amount to transfer
  * @param subject wire transfer subject to use
  * @param exchange_base_url exchange URL
+ * @param request_uid unique number to make the request unique, or NULL to create one
+ * @param[out] ret_row_id pointer to store the row ID of this transaction
+ * @return #GNUNET_YES if the transfer was successful,
+ *         #GNUNET_SYSERR if the request_uid was reused for a different transfer
+ */
+int
+TALER_FAKEBANK_make_transfer (
+  struct TALER_FAKEBANK_Handle *h,
+  const char *debit_account,
+  const char *credit_account,
+  const struct TALER_Amount *amount,
+  const struct TALER_WireTransferIdentifierRawP *subject,
+  const char *exchange_base_url,
+  const struct GNUNET_HashCode *request_uid,
+  uint64_t *ret_row_id);
+
+
+/**
+ * Tell the fakebank to create another wire transfer *to* an exchange.
+ *
+ * @param h fake bank handle
+ * @param debit_account account to debit
+ * @param credit_account account to credit
+ * @param amount amount to transfer
+ * @param reserve_pub reserve public key to use in subject
  * @return serial_id of the transfer
  */
 uint64_t
-TALER_FAKEBANK_make_transfer (struct TALER_FAKEBANK_Handle *h,
-                              uint64_t debit_account,
-                              uint64_t credit_account,
-                              const struct TALER_Amount *amount,
-                              const char *subject,
-                              const char *exchange_base_url);
+TALER_FAKEBANK_make_admin_transfer (
+  struct TALER_FAKEBANK_Handle *h,
+  const char *debit_account,
+  const char *credit_account,
+  const struct TALER_Amount *amount,
+  const struct TALER_ReservePublicKeyP *reserve_pub);
 
 
 /**
@@ -87,39 +116,42 @@ TALER_FAKEBANK_make_transfer (struct TALER_FAKEBANK_Handle *h,
  * to the transfer identifier and remove the transaction from the
  * list.  If the transaction was not recorded, return #GNUNET_SYSERR.
  *
- * Rejected transfers do NOT show with "check".
- *
  * @param h bank instance
  * @param want_amount transfer amount desired
  * @param want_debit account that should have been debited
  * @param want_debit account that should have been credited
  * @param exchange_base_url expected base URL of the exchange,
  *        i.e. "https://example.com/"; may include a port
- * @param[out] subject set to the wire transfer identifier
+ * @param[out] wtid set to the wire transfer identifier
  * @return #GNUNET_OK on success
  */
 int
-TALER_FAKEBANK_check (struct TALER_FAKEBANK_Handle *h,
-                      const struct TALER_Amount *want_amount,
-                      uint64_t want_debit,
-                      uint64_t want_credit,
-                      const char *exchange_base_url,
-                      char **subject);
+TALER_FAKEBANK_check_debit (struct TALER_FAKEBANK_Handle *h,
+                            const struct TALER_Amount *want_amount,
+                            const char *want_debit,
+                            const char *want_credit,
+                            const char *exchange_base_url,
+                            struct TALER_WireTransferIdentifierRawP *wtid);
 
 
 /**
- * Reject incoming wire transfer to account @a credit_account
- * as identified by @a rowid.
+ * Check that the @a want_amount was transferred from the @a want_debit to the
+ * @a want_credit account with the @a subject.  If so, remove the transaction
+ * from the list.  If the transaction was not recorded, return #GNUNET_SYSERR.
  *
- * @param h fake bank handle
- * @param rowid identifies transfer to reject
- * @param credit_account account number of owner of credited account
- * @return #GNUNET_YES on success, #GNUNET_NO if the wire transfer was not found
+ * @param h bank instance
+ * @param want_amount transfer amount desired
+ * @param want_debit account that should have been debited
+ * @param want_debit account that should have been credited
+ * @param reserve_pub reserve public key expected in wire subject
+ * @return #GNUNET_OK on success
  */
 int
-TALER_FAKEBANK_reject_transfer (struct TALER_FAKEBANK_Handle *h,
-                                uint64_t rowid,
-                                uint64_t credit_account);
+TALER_FAKEBANK_check_credit (struct TALER_FAKEBANK_Handle *h,
+                             const struct TALER_Amount *want_amount,
+                             const char *want_debit,
+                             const char *want_credit,
+                             const struct TALER_ReservePublicKeyP *reserve_pub);
 
 
 /**
