@@ -167,11 +167,6 @@
  */
 #define TALER_SIGNATURE_MERCHANT_PAYMENT_OK 1104
 
-/**
- * Signature where the merchant confirms a refund increase
- */
-#define TALER_SIGNATURE_MERCHANT_REFUND_OK 1105
-
 /**
  * Signature where the merchant confirms that the user replayed
  * a payment for a browser session.
@@ -321,19 +316,6 @@ struct TALER_WithdrawRequestPS
    */
   struct TALER_AmountNBO amount_with_fee;
 
-  /**
-   * Withdrawl fee charged by the exchange.  This must match the Exchange's
-   * denomination key's withdrawal fee.  If the client puts in an
-   * invalid withdrawal fee (too high or too low) that does not match
-   * the Exchange's denomination key, the withdraw operation is invalid
-   * and will be rejected by the exchange.  The @e amount_with_fee minus
-   * the @e withdraw_fee is must match the value of the generated
-   * coin.  We include this in what is being signed so that we can
-   * verify a exchange's accounting without needing to access the
-   * respective denomination key information each time.
-   */
-  struct TALER_AmountNBO withdraw_fee;
-
   /**
    * Hash of the denomination public key for the coin that is withdrawn.
    */
@@ -368,6 +350,11 @@ struct TALER_DepositRequestPS
    */
   struct GNUNET_HashCode h_wire GNUNET_PACKED;
 
+  /**
+   * Hash over the denomination public key used to sign the coin.
+   */
+  struct GNUNET_HashCode h_denom_pub GNUNET_PACKED;
+
   /**
    * Time when this request was generated.  Used, for example, to
    * assess when (roughly) the income was achieved for tax purposes.
@@ -381,7 +368,7 @@ struct TALER_DepositRequestPS
    * deposit request in a timely fashion (so back-dating is not
    * prevented).
    */
-  struct GNUNET_TIME_AbsoluteNBO timestamp;
+  struct GNUNET_TIME_AbsoluteNBO wallet_timestamp;
 
   /**
    * How much time does the merchant have to issue a refund request?
@@ -447,9 +434,10 @@ struct TALER_DepositConfirmationPS
   struct GNUNET_HashCode h_wire GNUNET_PACKED;
 
   /**
-   * Time when this confirmation was generated.
+   * Time when this confirmation was generated / when the exchange received
+   * the deposit request.
    */
-  struct GNUNET_TIME_AbsoluteNBO timestamp;
+  struct GNUNET_TIME_AbsoluteNBO exchange_timestamp;
 
   /**
    * How much time does the @e merchant have to issue a refund
@@ -521,18 +509,6 @@ struct TALER_RefundRequestPS
    * exchange to the customer.
    */
   struct TALER_AmountNBO refund_amount;
-
-  /**
-   * Refund fee charged by the exchange.  This must match the
-   * Exchange's denomination key's refund fee.  If the client puts in
-   * an invalid refund fee (too high or too low) that does not match
-   * the Exchange's denomination key, the refund operation is invalid
-   * and will be rejected by the exchange.  The @e amount_with_fee
-   * minus the @e refund_fee is the amount that will be credited to
-   * the original coin.
-   */
-  struct TALER_AmountNBO refund_fee;
-
 };
 
 
@@ -575,18 +551,6 @@ struct TALER_RefundConfirmationPS
    * exchange to the customer.
    */
   struct TALER_AmountNBO refund_amount;
-
-  /**
-   * Refund fee charged by the exchange.  This must match the
-   * Exchange's denomination key's refund fee.  If the client puts in
-   * an invalid refund fee (too high or too low) that does not match
-   * the Exchange's denomination key, the refund operation is invalid
-   * and will be rejected by the exchange.  The @e amount_with_fee
-   * minus the @e refund_fee is the amount that will be credited to
-   * the original coin.
-   */
-  struct TALER_AmountNBO refund_fee;
-
 };
 
 
@@ -607,6 +571,11 @@ struct TALER_RefreshMeltCoinAffirmationPS
    */
   struct TALER_RefreshCommitmentP rc GNUNET_PACKED;
 
+  /**
+   * Hash over the denomination public key used to sign the coin.
+   */
+  struct GNUNET_HashCode h_denom_pub GNUNET_PACKED;
+
   /**
    * How much of the value of the coin should be melted?  This amount
    * includes the fees, so the final amount contributed to the melt is
@@ -1158,10 +1127,8 @@ struct TALER_ProposalDataPS
 struct PaymentResponsePS
 {
   /**
-   * Set to TALER_SIGNATURE_MERCHANT_PAYMENT_OK so far. Note that
-   * unsuccessful payments are usually proven by some exchange's signature,
-   * thus it is unlikely that a merchant needs to set a purpose other than
-   * the above mentioned
+   * Set to #TALER_SIGNATURE_MERCHANT_PAYMENT_OK. Note that
+   * unsuccessful payments are usually proven by some exchange's signature.
    */
   struct GNUNET_CRYPTO_EccSignaturePurpose purpose;
 
@@ -1239,7 +1206,7 @@ struct TALER_RecoupRequestPS
   struct TALER_CoinSpendPublicKeyP coin_pub;
 
   /**
-   * Hash of the denomination public key of the coin.
+   * Hash of the (revoked) denomination public key of the coin.
    */
   struct GNUNET_HashCode h_denom_pub;
 
@@ -1371,26 +1338,6 @@ struct TALER_ReserveCloseConfirmationPS
 };
 
 
-/**
- * Used by the merchant to confirm with a signature that the refund has
- * been successfully done. Even though the frontend doesn't usually do crypto,
- * this signature may turn useful in court.
- */
-struct TALER_MerchantRefundConfirmationPS
-{
-  /**
-   * Set to #TALER_SIGNATURE_MERCHANT_REFUND_OK.
-   */
-  struct GNUNET_CRYPTO_EccSignaturePurpose purpose;
-
-  /**
-   * Hashed order id; in case frontend wants to check it.
-   * Hashed without the 0-termination.
-   */
-  struct GNUNET_HashCode h_order_id GNUNET_PACKED;
-
-};
-
 /**
  * Used by the merchant to confirm to the frontend that
  * the user did a payment replay with the current browser session.
