@@ -111,6 +111,20 @@ struct TALER_TESTING_DatabaseConnection
   struct TALER_EXCHANGEDB_Session *session;
 };
 
+struct TALER_TESTING_LibeufinServices
+{
+  /**
+   * Nexus
+   */
+  struct GNUNET_OS_Process *nexus;
+
+  /**
+   * Sandbox
+   */
+  struct GNUNET_OS_Process *sandbox;
+
+};
+
 /**
  * Prepare launching an exchange.  Checks that the configured
  * port is available, runs taler-exchange-keyup,
@@ -135,11 +149,13 @@ TALER_TESTING_prepare_exchange (const char *config_filename,
  *
  * @param cls closure, typically, the "run" method containing
  *        all the commands to be run, and a closure for it.
+ * @param hr http response details
  * @param keys the exchange's keys.
  * @param compat protocol compatibility information.
  */
 void
 TALER_TESTING_cert_cb (void *cls,
+                       const struct TALER_EXCHANGE_HttpResponse *hr,
                        const struct TALER_EXCHANGE_Keys *keys,
                        enum TALER_EXCHANGE_VersionCompatibility compat);
 
@@ -295,7 +311,7 @@ struct TALER_TESTING_BankConfiguration
 
 /**
  * Prepare launching a fakebank.  Check that the configuration
- * file has the right option, and that the port is avaiable.
+ * file has the right option, and that the port is available.
  * If everything is OK, return the configuration data of the fakebank.
  *
  * @param config_filename configuration file to use
@@ -372,10 +388,9 @@ struct TALER_TESTING_Interpreter
   void *final_cleanup_cb_cls;
 
   /**
-   * Instruction pointer.  Tells #interpreter_run() which
-   * instruction to run next.  Need (signed) int because
-   * it gets -1 when rewinding the interpreter to the first
-   * CMD.
+   * Instruction pointer.  Tells #interpreter_run() which instruction to run
+   * next.  Need (signed) int because it gets -1 when rewinding the
+   * interpreter to the first CMD.
    */
   int ip;
 
@@ -583,7 +598,22 @@ TALER_TESTING_interpreter_fail (struct TALER_TESTING_Interpreter *is);
  * @return a end-command.
  */
 struct TALER_TESTING_Command
-TALER_TESTING_cmd_end ();
+TALER_TESTING_cmd_end (void);
+
+
+/**
+ * Make the instruction pointer point to @a target_label
+ * only if @a counter is greater than zero.
+ *
+ * @param label command label
+ * @param target_label label of the new instruction pointer's destination after the jump;
+ *                     must be before the current instruction
+ * @param counter counts how many times the rewinding is to happen.
+ */
+struct TALER_TESTING_Command
+TALER_TESTING_cmd_rewind_ip (const char *label,
+                             const char *target_label,
+                             unsigned int counter);
 
 
 /**
@@ -800,7 +830,6 @@ TALER_TESTING_setup_with_auditor_and_exchange (TALER_TESTING_Main main_cb,
  * @param config_filename configuration filename.
  * @param bank_url base URL of the bank, used by `wget' to check
  *        that the bank was started right.
- *
  * @return the process, or NULL if the process could not
  *         be started.
  */
@@ -808,6 +837,21 @@ struct GNUNET_OS_Process *
 TALER_TESTING_run_bank (const char *config_filename,
                         const char *bank_url);
 
+/**
+ * Start the (nexus) bank process.  Assume the port
+ * is available and the database is clean.  Use the "prepare
+ * bank" function to do such tasks.  This function is also
+ * responsible to create the exchange EBICS subscriber at
+ * the nexus.
+ *
+ * @param bc bank configuration of the bank
+ * @return the process, or NULL if the process could not
+ *         be started.
+ */
+struct TALER_TESTING_LibeufinServices
+TALER_TESTING_run_libeufin (const struct TALER_TESTING_BankConfiguration *bc);
+
+
 /**
  * Runs the Fakebank by guessing / extracting the portnumber
  * from the base URL.
@@ -837,6 +881,22 @@ TALER_TESTING_prepare_bank (const char *config_filename,
                             const char *config_section,
                             struct TALER_TESTING_BankConfiguration *bc);
 
+/**
+ * Prepare the Nexus execution.  Check if the port is available
+ * and delete old database.
+ *
+ * @param config_filename configuration file name.
+ * @param reset_db should we reset the bank's database
+ * @param config_section section of the configuration with the exchange's account
+ * @param[out] bc set to the bank's configuration data
+ * @return the base url, or NULL upon errors.  Must be freed
+ *         by the caller.
+ */
+int
+TALER_TESTING_prepare_nexus (const char *config_filename,
+                             int reset_db,
+                             const char *config_section,
+                             struct TALER_TESTING_BankConfiguration *bc);
 
 /**
  * Look for substring in a programs' name.
@@ -911,6 +971,17 @@ TALER_TESTING_cmd_transfer (const char *label,
                             const char *exchange_base_url);
 
 
+/**
+ * Modify a transfer command to enable retries when the reserve is not yet
+ * full or we get other transient errors from the bank.
+ *
+ * @param cmd a fakebank transfer command
+ * @return the command with retries enabled
+ */
+struct TALER_TESTING_Command
+TALER_TESTING_cmd_transfer_retry (struct TALER_TESTING_Command cmd);
+
+
 /**
  * Make the "exec-auditor" CMD.
  *
@@ -1042,7 +1113,7 @@ TALER_TESTING_cmd_admin_add_incoming (
  * @param payto_debit_account which account sends money.
  * @param auth authentication data
  * @param ref reference to a command that can offer a reserve
- *        private key.
+ *        private key or public key.
  * @return the command.
  */
 struct TALER_TESTING_Command
@@ -1054,34 +1125,6 @@ TALER_TESTING_cmd_admin_add_incoming_with_ref (
   const char *ref);
 
 
-/**
- * Create "fakebank transfer" CMD, letting the caller specifying
- * the merchant instance.  This version is useful when a tip
- * reserve should be topped up, in fact the interpreter will need
- * the "tipping instance" in order to get the instance public key
- * and make a wire transfer subject out of it.
- *
- * @param label command label.
- * @param amount amount to transfer.
- * @param payto_debit_account which account sends money.
- * @param auth authentication data
- * @param instance the instance that runs the tipping.  Under this
- *        instance, the configuration file will provide the private
- *        key of the tipping reserve.  This data will then used to
- *        construct the wire transfer subject line.
- * @param config_filename configuration file to use.
- * @return the command.
- */
-struct TALER_TESTING_Command
-TALER_TESTING_cmd_admin_add_incoming_with_instance (
-  const char *label,
-  const char *amount,
-  const struct TALER_BANK_AuthenticationData *auth,
-  const char *payto_debit_account,
-  const char *instance,
-  const char *config_filename);
-
-
 /**
  * Modify a fakebank transfer command to enable retries when the
  * reserve is not yet full or we get other transient errors from
@@ -1234,6 +1277,30 @@ TALER_TESTING_cmd_withdraw_amount (const char *label,
                                    unsigned int expected_response_code);
 
 
+/**
+ * Create a withdraw command, letting the caller specify
+ * the desired amount as string and also re-using an existing
+ * coin private key in the process (violating the specification,
+ * which will result in an error when spending the coin!).
+ *
+ * @param label command label.
+ * @param reserve_reference command providing us with a reserve to withdraw from
+ * @param amount how much we withdraw.
+ * @param coin_ref reference to (withdraw/reveal) command of a coin
+ *        from which we should re-use the private key
+ * @param expected_response_code which HTTP response code
+ *        we expect from the exchange.
+ * @return the withdraw command to be executed by the interpreter.
+ */
+struct TALER_TESTING_Command
+TALER_TESTING_cmd_withdraw_amount_reuse_key (
+  const char *label,
+  const char *reserve_reference,
+  const char *amount,
+  const char *coin_ref,
+  unsigned int expected_response_code);
+
+
 /**
  * Create withdraw command, letting the caller specify the
  * amount by a denomination key.
@@ -1298,6 +1365,16 @@ TALER_TESTING_cmd_status (const char *label,
                           const char *expected_balance,
                           unsigned int expected_response_code);
 
+/**
+ * Index of the deposit value trait of a deposit command.
+ */
+#define TALER_TESTING_CMD_DEPOSIT_TRAIT_IDX_DEPOSIT_VALUE 0
+
+/**
+ * Index of the deposit fee trait of a deposit command.
+ */
+#define TALER_TESTING_CMD_DEPOSIT_TRAIT_IDX_DEPOSIT_FEE 1
+
 /**
  * Create a "deposit" command.
  *
@@ -1605,7 +1682,7 @@ TALER_TESTING_cmd_check_bank_transfer_with_ref (const char *label,
 
 
 /**
- * Checks wheter all the wire transfers got "checked"
+ * Checks whether all the wire transfers got "checked"
  * by the "bank check" CMD.
  *
  * @param label command label.
@@ -1623,7 +1700,6 @@ TALER_TESTING_cmd_check_bank_empty (const char *label);
  * @param label command label.
  * @param expected_response_code expected HTTP status code.
  * @param refund_amount the amount to ask a refund for.
- * @param refund_fee expected refund fee.
  * @param coin_reference reference to a command that can
  *        provide a coin to be refunded.
  * @param refund_transaction_id transaction id to use
@@ -1635,7 +1711,6 @@ struct TALER_TESTING_Command
 TALER_TESTING_cmd_refund_with_id (const char *label,
                                   unsigned int expected_response_code,
                                   const char *refund_amount,
-                                  const char *refund_fee,
                                   const char *deposit_reference,
                                   uint64_t refund_transaction_id);
 
@@ -1646,7 +1721,6 @@ TALER_TESTING_cmd_refund_with_id (const char *label,
  * @param label command label.
  * @param expected_response_code expected HTTP status code.
  * @param refund_amount the amount to ask a refund for.
- * @param refund_fee expected refund fee.
  * @param coin_reference reference to a command that can
  *        provide a coin to be refunded.
  *
@@ -1656,7 +1730,6 @@ struct TALER_TESTING_Command
 TALER_TESTING_cmd_refund (const char *label,
                           unsigned int expected_response_code,
                           const char *refund_amount,
-                          const char *refund_fee,
                           const char *deposit_reference);
 
 
@@ -1727,7 +1800,7 @@ TALER_TESTING_cmd_sleep (const char *label,
 
 /**
  * This CMD simply tries to connect via HTTP to the
- * service addressed by @a url.  It attemps 10 times
+ * service addressed by @a url.  It attempts 10 times
  * before giving up and make the test fail.
  *
  * @param label label for the command.
@@ -1843,6 +1916,18 @@ TALER_TESTING_cmd_batch_next (struct TALER_TESTING_Interpreter *is);
 struct TALER_TESTING_Command *
 TALER_TESTING_cmd_batch_get_current (const struct TALER_TESTING_Command *cmd);
 
+
+/**
+ * Set what command the batch should be at.
+ *
+ * @param cmd current batch command
+ * @param new_ip where to move the IP
+ */
+void
+TALER_TESTING_cmd_batch_set_current (const struct TALER_TESTING_Command *cmd,
+                                     unsigned int new_ip);
+
+
 /**
  * Make a serialize-keys CMD.
  *
@@ -1874,6 +1959,7 @@ TALER_TESTING_cmd_connect_with_state (const char *label,
  * @param dbc collects plugin and session handles
  * @param merchant_name Human-readable name of the merchant.
  * @param merchant_account merchant's account name (NOT a payto:// URI)
+ * @param exchange_timestamp when did the exchange receive the deposit
  * @param wire_deadline point in time where the aggregator should have
  *        wired money to the merchant.
  * @param amount_with_fee amount to deposit (inclusive of deposit fee)
@@ -1881,14 +1967,15 @@ TALER_TESTING_cmd_connect_with_state (const char *label,
  * @return the command.
  */
 struct TALER_TESTING_Command
-TALER_TESTING_cmd_insert_deposit (const char *label,
-                                  const struct
-                                  TALER_TESTING_DatabaseConnection *dbc,
-                                  const char *merchant_name,
-                                  const char *merchant_account,
-                                  struct GNUNET_TIME_Relative wire_deadline,
-                                  const char *amount_with_fee,
-                                  const char *deposit_fee);
+TALER_TESTING_cmd_insert_deposit (
+  const char *label,
+  const struct TALER_TESTING_DatabaseConnection *dbc,
+  const char *merchant_name,
+  const char *merchant_account,
+  struct GNUNET_TIME_Absolute exchange_timestamp,
+  struct GNUNET_TIME_Relative wire_deadline,
+  const char *amount_with_fee,
+  const char *deposit_fee);
 
 
 /**
@@ -2292,6 +2379,31 @@ TALER_TESTING_make_trait_denom_sig (
   const struct TALER_DenominationSignature *sig);
 
 
+/**
+ * Offer number trait, 32-bit version.
+ *
+ * @param index the number's index number.
+ * @param n number to offer.
+ */
+struct TALER_TESTING_Trait
+TALER_TESTING_make_trait_uint32 (unsigned int index,
+                                 const uint32_t *n);
+
+
+/**
+ * Obtain a "number" value from @a cmd, 32-bit version.
+ *
+ * @param cmd command to extract the number from.
+ * @param index the number's index number.
+ * @param[out] n set to the number coming from @a cmd.
+ * @return #GNUNET_OK on success.
+ */
+int
+TALER_TESTING_get_trait_uint32 (const struct TALER_TESTING_Command *cmd,
+                                unsigned int index,
+                                const uint32_t **n);
+
+
 /**
  * Offer number trait, 64-bit version.
  *
@@ -2317,6 +2429,31 @@ TALER_TESTING_get_trait_uint64 (const struct TALER_TESTING_Command *cmd,
                                 const uint64_t **n);
 
 
+/**
+ * Offer number trait, 64-bit signed version.
+ *
+ * @param index the number's index number.
+ * @param n number to offer.
+ */
+struct TALER_TESTING_Trait
+TALER_TESTING_make_trait_int64 (unsigned int index,
+                                const int64_t *n);
+
+
+/**
+ * Obtain a "number" value from @a cmd, 64-bit signed version.
+ *
+ * @param cmd command to extract the number from.
+ * @param index the number's index number.
+ * @param[out] n set to the number coming from @a cmd.
+ * @return #GNUNET_OK on success.
+ */
+int
+TALER_TESTING_get_trait_int64 (const struct TALER_TESTING_Command *cmd,
+                               unsigned int index,
+                               const int64_t **n);
+
+
 /**
  * Offer a number.
  *
@@ -2466,6 +2603,33 @@ TALER_TESTING_make_trait_exchange_keys (unsigned int index,
                                         const json_t *keys);
 
 
+/**
+ * Obtain json from @a cmd.
+ *
+ * @param cmd command to extract the json from.
+ * @param index index number associate with the json on offer.
+ * @param[out] json where to write the json.
+ * @return #GNUNET_OK on success.
+ */
+int
+TALER_TESTING_get_trait_json (const struct TALER_TESTING_Command *cmd,
+                              unsigned int index,
+                              const json_t **json);
+
+
+/**
+ * Offer json in a trait.
+ *
+ * @param index index number associate with the json
+ *        on offer.
+ * @param json json to offer.
+ * @return the trait.
+ */
+struct TALER_TESTING_Trait
+TALER_TESTING_make_trait_json (unsigned int index,
+                               const json_t *json);
+
+
 /**
  * Obtain a private key from a "merchant".  Used e.g. to obtain
  * a merchant's priv to sign a /track request.
@@ -2779,6 +2943,64 @@ TALER_TESTING_get_trait_cmd (const struct TALER_TESTING_Command *cmd,
                              struct TALER_TESTING_Command **_cmd);
 
 
+/**
+ * Obtain a uuid from @a cmd.
+ *
+ * @param cmd command to extract the uuid from.
+ * @param index which amount to pick if @a cmd has multiple
+ *        on offer
+ * @param[out] uuid where to write the uuid.
+ * @return #GNUNET_OK on success.
+ */
+int
+TALER_TESTING_get_trait_uuid (const struct TALER_TESTING_Command *cmd,
+                              unsigned int index,
+                              struct GNUNET_Uuid **uuid);
+
+
+/**
+ * Offer a uuid in a trait.
+ *
+ * @param index which uuid to offer, in case there are
+ *        multiple available.
+ * @param uuid the uuid to offer.
+ *
+ * @return the trait.
+ */
+struct TALER_TESTING_Trait
+TALER_TESTING_make_trait_uuid (unsigned int index,
+                               const struct GNUNET_Uuid *uuid);
+
+
+/**
+ * Obtain a claim token from @a cmd.
+ *
+ * @param cmd command to extract the token from.
+ * @param index which amount to pick if @a cmd has multiple
+ *        on offer
+ * @param[out] ct where to write the token.
+ * @return #GNUNET_OK on success.
+ */
+int
+TALER_TESTING_get_trait_claim_token (const struct TALER_TESTING_Command *cmd,
+                                     unsigned int index,
+                                     const struct TALER_ClaimTokenP **ct);
+
+
+/**
+ * Offer a claim token in a trait.
+ *
+ * @param index which token to offer, in case there are
+ *        multiple available.
+ * @param ct the token to offer.
+ *
+ * @return the trait.
+ */
+struct TALER_TESTING_Trait
+TALER_TESTING_make_trait_claim_token (unsigned int index,
+                                      const struct TALER_ClaimTokenP *ct);
+
+
 /**
  * Obtain a absolute time from @a cmd.
  *
@@ -2807,4 +3029,33 @@ TALER_TESTING_make_trait_absolute_time (
   unsigned int index,
   const struct GNUNET_TIME_Absolute *time);
 
+
+/**
+ * Obtain a relative time from @a cmd.
+ *
+ * @param cmd command to extract trait from
+ * @param index which time to pick if
+ *        @a cmd has multiple on offer.
+ * @param[out] time set to the wanted WTID.
+ * @return #GNUNET_OK on success
+ */
+int
+TALER_TESTING_get_trait_relative_time (
+  const struct TALER_TESTING_Command *cmd,
+  unsigned int index,
+  const struct GNUNET_TIME_Relative **time);
+
+
+/**
+ * Offer a relative time.
+ *
+ * @param index associate the object with this index
+ * @param time which object should be returned
+ * @return the trait.
+ */
+struct TALER_TESTING_Trait
+TALER_TESTING_make_trait_relative_time (
+  unsigned int index,
+  const struct GNUNET_TIME_Relative *time);
+
 #endif
