@@ -616,7 +616,7 @@ TALER_JSON_get_error_code (const json_t *json)
   if (NULL == json)
   {
     GNUNET_break_op (0);
-    return TALER_EC_INVALID_RESPONSE;
+    return TALER_EC_GENERIC_INVALID_RESPONSE;
   }
   jc = json_object_get (json, "code");
   /* The caller already knows that the JSON represents an error,
