@@ -281,7 +281,15 @@ validate_iban (const char *iban)
     j++;
   }
   for (j = 0; '\0' != nbuf[j]; j++)
-    GNUNET_assert (isdigit ( (unsigned char) nbuf[j]));
+  {
+    if (! isdigit ( (unsigned char) nbuf[j]))
+    {
+      GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
+                  "IBAN `%s' didn't convert to numeric format\n",
+                  iban);
+      return GNUNET_NO;
+    }
+  }
   GNUNET_assert (sizeof(dividend) >= 8);
   remainder = 0;
   for (unsigned int i = 0; i<j; i += 16)
@@ -335,7 +343,8 @@ validate_payto_iban (const char *account_url)
                         IBAN_PREFIX,
                         strlen (IBAN_PREFIX)))
     return GNUNET_NO;
-  iban = &account_url[strlen (IBAN_PREFIX)];
+
+  iban = strrchr (account_url, '/') + 1;
 #undef IBAN_PREFIX
   q = strchr (iban,
               '?');
@@ -380,7 +389,10 @@ validate_payto (const char *account_url)
     return GNUNET_SYSERR; /* not payto */
 #undef PAYTO_PREFIX
   if (GNUNET_NO != (ret = validate_payto_iban (account_url)))
+  {
+    GNUNET_break_op (GNUNET_SYSERR != ret);
     return ret; /* got a definitive answer */
+  }
   /* Insert other bank account validation methods here later! */
   return GNUNET_NO;
 }
@@ -399,7 +411,9 @@ TALER_JSON_merchant_wire_signature_hash (const json_t *wire_s,
                                          struct GNUNET_HashCode *hc)
 {
   const char *payto_uri;
-  const char *salt;
+  const char *salt; /* Current merchant backend will always make the salt
+                       a `struct GNUNET_HashCode`, but *we* do not insist
+                       on that. */
   struct GNUNET_JSON_Specification spec[] = {
     GNUNET_JSON_spec_string ("payto_uri", &payto_uri),
     GNUNET_JSON_spec_string ("salt", &salt),
@@ -414,6 +428,14 @@ TALER_JSON_merchant_wire_signature_hash (const json_t *wire_s,
     GNUNET_break_op (0);
     return GNUNET_SYSERR;
   }
+  GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
+              "Validating `%s'\n",
+              payto_uri);
+  if (GNUNET_SYSERR == validate_payto (payto_uri))
+  {
+    GNUNET_break_op (0);
+    return GNUNET_SYSERR;
+  }
   TALER_merchant_wire_signature_hash (payto_uri,
                                       salt,
                                       hc);
@@ -430,9 +452,9 @@ TALER_JSON_merchant_wire_signature_hash (const json_t *wire_s,
  * @return #GNUNET_OK if signature is valid
  */
 int
-TALER_JSON_exchange_wire_signature_check (const json_t *wire_s,
-                                          const struct
-                                          TALER_MasterPublicKeyP *master_pub)
+TALER_JSON_exchange_wire_signature_check (
+  const json_t *wire_s,
+  const struct TALER_MasterPublicKeyP *master_pub)
 {
   const char *payto_uri;
   struct TALER_MasterSignatureP master_sig;
@@ -471,9 +493,9 @@ TALER_JSON_exchange_wire_signature_check (const json_t *wire_s,
  * @return NULL if @a payto_uri is malformed
  */
 json_t *
-TALER_JSON_exchange_wire_signature_make (const char *payto_uri,
-                                         const struct
-                                         TALER_MasterPrivateKeyP *master_priv)
+TALER_JSON_exchange_wire_signature_make (
+  const char *payto_uri,
+  const struct TALER_MasterPrivateKeyP *master_priv)
 {
   struct TALER_MasterSignatureP master_sig;
 
