@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2015-2017 Taler Systems SA
+  Copyright (C) 2015-2020 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU General Public License as published by the Free Software
@@ -56,7 +56,6 @@ TALER_EXCHANGE_parse_reserve_history (
   unsigned int uuid_off;
   struct TALER_Amount total_in;
   struct TALER_Amount total_out;
-  size_t off;
 
   GNUNET_assert (GNUNET_OK ==
                  TALER_amount_get_zero (currency,
@@ -65,13 +64,15 @@ TALER_EXCHANGE_parse_reserve_history (
                  TALER_amount_get_zero (currency,
                                         &total_out));
   uuid_off = 0;
-  for (off = 0; off<history_length; off++)
+  for (unsigned int off = 0; off<history_length; off++)
   {
+    struct TALER_EXCHANGE_ReserveHistory *rh = &rhistory[off];
     json_t *transaction;
     struct TALER_Amount amount;
     const char *type;
     struct GNUNET_JSON_Specification hist_spec[] = {
-      GNUNET_JSON_spec_string ("type", &type),
+      GNUNET_JSON_spec_string ("type",
+                               &type),
       TALER_JSON_spec_amount ("amount",
                               &amount),
       /* 'wire' and 'signature' are optional depending on 'type'! */
@@ -89,7 +90,13 @@ TALER_EXCHANGE_parse_reserve_history (
       return GNUNET_SYSERR;
     }
     rhistory[off].amount = amount;
-
+    if (GNUNET_YES !=
+        TALER_amount_cmp_currency (&amount,
+                                   &total_in))
+    {
+      GNUNET_break_op (0);
+      return GNUNET_SYSERR;
+    }
     if (0 == strcasecmp (type,
                          "CREDIT"))
     {
@@ -102,15 +109,15 @@ TALER_EXCHANGE_parse_reserve_history (
         GNUNET_JSON_spec_varsize ("wire_reference",
                                   &wire_reference,
                                   &wire_reference_size),
-        GNUNET_JSON_spec_absolute_time ("timestamp",
-                                        &timestamp),
+        TALER_JSON_spec_absolute_time ("timestamp",
+                                       &timestamp),
         GNUNET_JSON_spec_string ("sender_account_url",
                                  &wire_url),
         GNUNET_JSON_spec_end ()
       };
 
-      rhistory[off].type = TALER_EXCHANGE_RTT_CREDIT;
-      if (GNUNET_OK !=
+      rh->type = TALER_EXCHANGE_RTT_CREDIT;
+      if (0 >
           TALER_amount_add (&total_in,
                             &total_in,
                             &amount))
@@ -127,11 +134,11 @@ TALER_EXCHANGE_parse_reserve_history (
         GNUNET_break_op (0);
         return GNUNET_SYSERR;
       }
-      rhistory[off].details.in_details.sender_url = GNUNET_strdup (wire_url);
-      rhistory[off].details.in_details.wire_reference = wire_reference;
-      rhistory[off].details.in_details.wire_reference_size =
+      rh->details.in_details.sender_url = GNUNET_strdup (wire_url);
+      rh->details.in_details.wire_reference = wire_reference;
+      rh->details.in_details.wire_reference_size =
         wire_reference_size;
-      rhistory[off].details.in_details.timestamp = timestamp;
+      rh->details.in_details.timestamp = timestamp;
       /* end type==DEPOSIT */
     }
     else if (0 == strcasecmp (type,
@@ -139,11 +146,12 @@ TALER_EXCHANGE_parse_reserve_history (
     {
       struct TALER_ReserveSignatureP sig;
       struct TALER_WithdrawRequestPS withdraw_purpose;
+      struct TALER_Amount withdraw_fee;
       struct GNUNET_JSON_Specification withdraw_spec[] = {
         GNUNET_JSON_spec_fixed_auto ("reserve_sig",
                                      &sig),
-        TALER_JSON_spec_amount_nbo ("withdraw_fee",
-                                    &withdraw_purpose.withdraw_fee),
+        TALER_JSON_spec_amount ("withdraw_fee",
+                                &withdraw_fee),
         GNUNET_JSON_spec_fixed_auto ("h_denom_pub",
                                      &withdraw_purpose.h_denomination_pub),
         GNUNET_JSON_spec_fixed_auto ("h_coin_envelope",
@@ -151,7 +159,7 @@ TALER_EXCHANGE_parse_reserve_history (
         GNUNET_JSON_spec_end ()
       };
 
-      rhistory[off].type = TALER_EXCHANGE_RTT_WITHDRAWAL;
+      rh->type = TALER_EXCHANGE_RTT_WITHDRAWAL;
       if (GNUNET_OK !=
           GNUNET_JSON_parse (transaction,
                              withdraw_spec,
@@ -170,7 +178,7 @@ TALER_EXCHANGE_parse_reserve_history (
       /* Check that the signature is a valid withdraw request */
       if (GNUNET_OK !=
           GNUNET_CRYPTO_eddsa_verify (TALER_SIGNATURE_WALLET_RESERVE_WITHDRAW,
-                                      &withdraw_purpose.purpose,
+                                      &withdraw_purpose,
                                       &sig.eddsa_signature,
                                       &reserve_pub->eddsa_pub))
       {
@@ -182,28 +190,25 @@ TALER_EXCHANGE_parse_reserve_history (
       {
         const struct TALER_EXCHANGE_Keys *key_state;
         const struct TALER_EXCHANGE_DenomPublicKey *dki;
-        struct TALER_Amount fee;
 
         key_state = TALER_EXCHANGE_get_keys (exchange);
         dki = TALER_EXCHANGE_get_denomination_key_by_hash (key_state,
                                                            &withdraw_purpose.
                                                            h_denomination_pub);
-        TALER_amount_ntoh (&fee,
-                           &withdraw_purpose.withdraw_fee);
         if ( (GNUNET_YES !=
-              TALER_amount_cmp_currency (&fee,
+              TALER_amount_cmp_currency (&withdraw_fee,
                                          &dki->fee_withdraw)) ||
              (0 !=
-              TALER_amount_cmp (&fee,
+              TALER_amount_cmp (&withdraw_fee,
                                 &dki->fee_withdraw)) )
         {
           GNUNET_break_op (0);
           GNUNET_JSON_parse_free (withdraw_spec);
           return GNUNET_SYSERR;
         }
-        rhistory[off].details.withdraw.fee = fee;
+        rh->details.withdraw.fee = withdraw_fee;
       }
-      rhistory[off].details.withdraw.out_authorization_sig
+      rh->details.withdraw.out_authorization_sig
         = json_object_get (transaction,
                            "signature");
       /* Check check that the same withdraw transaction
@@ -226,7 +231,7 @@ TALER_EXCHANGE_parse_reserve_history (
       }
       uuid_off++;
 
-      if (GNUNET_OK !=
+      if (0 >
           TALER_amount_add (&total_out,
                             &total_out,
                             &amount))
@@ -248,18 +253,16 @@ TALER_EXCHANGE_parse_reserve_history (
         GNUNET_JSON_spec_fixed_auto ("coin_pub",
                                      &pc.coin_pub),
         GNUNET_JSON_spec_fixed_auto ("exchange_sig",
-                                     &rhistory[off].details.recoup_details.
-                                     exchange_sig),
+                                     &rh->details.recoup_details.exchange_sig),
         GNUNET_JSON_spec_fixed_auto ("exchange_pub",
-                                     &rhistory[off].details.recoup_details.
-                                     exchange_pub),
-        GNUNET_JSON_spec_absolute_time_nbo ("timestamp",
-                                            &pc.timestamp),
+                                     &rh->details.recoup_details.exchange_pub),
+        TALER_JSON_spec_absolute_time_nbo ("timestamp",
+                                           &pc.timestamp),
         GNUNET_JSON_spec_end ()
       };
 
-      rhistory[off].type = TALER_EXCHANGE_RTT_RECOUP;
-      rhistory[off].amount = amount;
+      rh->type = TALER_EXCHANGE_RTT_RECOUP;
+      rh->amount = amount;
       if (GNUNET_OK !=
           GNUNET_JSON_parse (transaction,
                              recoup_spec,
@@ -268,39 +271,38 @@ TALER_EXCHANGE_parse_reserve_history (
         GNUNET_break_op (0);
         return GNUNET_SYSERR;
       }
-      rhistory[off].details.recoup_details.coin_pub = pc.coin_pub;
+      rh->details.recoup_details.coin_pub = pc.coin_pub;
       TALER_amount_hton (&pc.recoup_amount,
                          &amount);
       pc.purpose.size = htonl (sizeof (pc));
       pc.purpose.purpose = htonl (TALER_SIGNATURE_EXCHANGE_CONFIRM_RECOUP);
       pc.reserve_pub = *reserve_pub;
       timestamp = GNUNET_TIME_absolute_ntoh (pc.timestamp);
-      rhistory[off].details.recoup_details.timestamp = timestamp;
+      rh->details.recoup_details.timestamp = timestamp;
 
       key_state = TALER_EXCHANGE_get_keys (exchange);
       if (GNUNET_OK !=
           TALER_EXCHANGE_test_signing_key (key_state,
-                                           &rhistory[off].details.
+                                           &rh->details.
                                            recoup_details.exchange_pub))
       {
         GNUNET_break_op (0);
         return GNUNET_SYSERR;
       }
       if (GNUNET_OK !=
-          GNUNET_CRYPTO_eddsa_verify (TALER_SIGNATURE_EXCHANGE_CONFIRM_RECOUP,
-                                      &pc.purpose,
-                                      &rhistory[off].details.recoup_details.
-                                      exchange_sig.eddsa_signature,
-                                      &rhistory[off].details.recoup_details.
-                                      exchange_pub.eddsa_pub))
+          GNUNET_CRYPTO_eddsa_verify (
+            TALER_SIGNATURE_EXCHANGE_CONFIRM_RECOUP,
+            &pc,
+            &rh->details.recoup_details.exchange_sig.eddsa_signature,
+            &rh->details.recoup_details.exchange_pub.eddsa_pub))
       {
         GNUNET_break_op (0);
         return GNUNET_SYSERR;
       }
-      if (GNUNET_OK !=
+      if (0 >
           TALER_amount_add (&total_in,
                             &total_in,
-                            &rhistory[off].amount))
+                            &rh->amount))
       {
         /* overflow in history already!? inconceivable! Bad exchange! */
         GNUNET_break_op (0);
@@ -315,26 +317,24 @@ TALER_EXCHANGE_parse_reserve_history (
       struct TALER_ReserveCloseConfirmationPS rcc;
       struct GNUNET_TIME_Absolute timestamp;
       struct GNUNET_JSON_Specification closing_spec[] = {
-        GNUNET_JSON_spec_string ("receiver_account_details",
-                                 &rhistory[off].details.close_details.
-                                 receiver_account_details),
+        GNUNET_JSON_spec_string (
+          "receiver_account_details",
+          &rh->details.close_details.receiver_account_details),
         GNUNET_JSON_spec_fixed_auto ("wtid",
-                                     &rhistory[off].details.close_details.wtid),
+                                     &rh->details.close_details.wtid),
         GNUNET_JSON_spec_fixed_auto ("exchange_sig",
-                                     &rhistory[off].details.close_details.
-                                     exchange_sig),
+                                     &rh->details.close_details.exchange_sig),
         GNUNET_JSON_spec_fixed_auto ("exchange_pub",
-                                     &rhistory[off].details.close_details.
-                                     exchange_pub),
+                                     &rh->details.close_details.exchange_pub),
         TALER_JSON_spec_amount_nbo ("closing_fee",
                                     &rcc.closing_fee),
-        GNUNET_JSON_spec_absolute_time_nbo ("timestamp",
-                                            &rcc.timestamp),
+        TALER_JSON_spec_absolute_time_nbo ("timestamp",
+                                           &rcc.timestamp),
         GNUNET_JSON_spec_end ()
       };
 
-      rhistory[off].type = TALER_EXCHANGE_RTT_CLOSE;
-      rhistory[off].amount = amount;
+      rh->type = TALER_EXCHANGE_RTT_CLOSE;
+      rh->amount = amount;
       if (GNUNET_OK !=
           GNUNET_JSON_parse (transaction,
                              closing_spec,
@@ -346,42 +346,40 @@ TALER_EXCHANGE_parse_reserve_history (
       TALER_amount_hton (&rcc.closing_amount,
                          &amount);
       GNUNET_CRYPTO_hash (
-        rhistory[off].details.close_details.receiver_account_details,
-        strlen (
-          rhistory[off].details.close_details.receiver_account_details) + 1,
+        rh->details.close_details.receiver_account_details,
+        strlen (rh->details.close_details.receiver_account_details) + 1,
         &rcc.h_wire);
-      rcc.wtid = rhistory[off].details.close_details.wtid;
+      rcc.wtid = rh->details.close_details.wtid;
       rcc.purpose.size = htonl (sizeof (rcc));
       rcc.purpose.purpose = htonl (TALER_SIGNATURE_EXCHANGE_RESERVE_CLOSED);
       rcc.reserve_pub = *reserve_pub;
       timestamp = GNUNET_TIME_absolute_ntoh (rcc.timestamp);
-      rhistory[off].details.close_details.timestamp = timestamp;
-      TALER_amount_ntoh (&rhistory[off].details.close_details.fee,
+      rh->details.close_details.timestamp = timestamp;
+      TALER_amount_ntoh (&rh->details.close_details.fee,
                          &rcc.closing_fee);
       key_state = TALER_EXCHANGE_get_keys (exchange);
       if (GNUNET_OK !=
           TALER_EXCHANGE_test_signing_key (key_state,
-                                           &rhistory[off].details.close_details.
+                                           &rh->details.close_details.
                                            exchange_pub))
       {
         GNUNET_break_op (0);
         return GNUNET_SYSERR;
       }
       if (GNUNET_OK !=
-          GNUNET_CRYPTO_eddsa_verify (TALER_SIGNATURE_EXCHANGE_RESERVE_CLOSED,
-                                      &rcc.purpose,
-                                      &rhistory[off].details.close_details.
-                                      exchange_sig.eddsa_signature,
-                                      &rhistory[off].details.close_details.
-                                      exchange_pub.eddsa_pub))
+          GNUNET_CRYPTO_eddsa_verify (
+            TALER_SIGNATURE_EXCHANGE_RESERVE_CLOSED,
+            &rcc,
+            &rh->details.close_details.exchange_sig.eddsa_signature,
+            &rh->details.close_details.exchange_pub.eddsa_pub))
       {
         GNUNET_break_op (0);
         return GNUNET_SYSERR;
       }
-      if (GNUNET_OK !=
+      if (0 >
           TALER_amount_add (&total_out,
                             &total_out,
-                            &rhistory[off].amount))
+                            &rh->amount))
       {
         /* overflow in history already!? inconceivable! Bad exchange! */
         GNUNET_break_op (0);
@@ -398,7 +396,7 @@ TALER_EXCHANGE_parse_reserve_history (
   }
 
   /* check balance = total_in - total_out < withdraw-amount */
-  if (GNUNET_SYSERR ==
+  if (0 >
       TALER_amount_subtract (balance,
                              &total_in,
                              &total_out))
@@ -427,8 +425,8 @@ TALER_EXCHANGE_free_reserve_history (
     switch (rhistory[i].type)
     {
     case TALER_EXCHANGE_RTT_CREDIT:
-      GNUNET_free_non_null (rhistory[i].details.in_details.wire_reference);
-      GNUNET_free_non_null (rhistory[i].details.in_details.sender_url);
+      GNUNET_free (rhistory[i].details.in_details.wire_reference);
+      GNUNET_free (rhistory[i].details.in_details.sender_url);
       break;
     case TALER_EXCHANGE_RTT_WITHDRAWAL:
       break;
@@ -449,6 +447,7 @@ TALER_EXCHANGE_free_reserve_history (
  * @param currency expected currency for the coin
  * @param coin_pub public key of the coin
  * @param history history of the coin in json encoding
+ * @param[out] h_denom_pub set to the hash of the coin's denomination (if available)
  * @param[out] total how much of the coin has been spent according to @a history
  * @return #GNUNET_OK if @a history is valid, #GNUNET_SYSERR if not
  */
@@ -458,6 +457,7 @@ TALER_EXCHANGE_verify_coin_history (
   const char *currency,
   const struct TALER_CoinSpendPublicKeyP *coin_pub,
   json_t *history,
+  struct GNUNET_HashCode *h_denom_pub,
   struct TALER_Amount *total)
 {
   size_t len;
@@ -505,11 +505,22 @@ TALER_EXCHANGE_verify_coin_history (
       GNUNET_break_op (0);
       return GNUNET_SYSERR;
     }
+    if (GNUNET_YES !=
+        TALER_amount_cmp_currency (&amount,
+                                   &rtotal))
+    {
+      GNUNET_break_op (0);
+      return GNUNET_SYSERR;
+    }
     add = GNUNET_SYSERR;
     if (0 == strcasecmp (type,
                          "DEPOSIT"))
     {
-      struct TALER_DepositRequestPS dr;
+      struct TALER_DepositRequestPS dr = {
+        .purpose.size = htonl (sizeof (dr)),
+        .purpose.purpose = htonl (TALER_SIGNATURE_WALLET_COIN_DEPOSIT),
+        .coin_pub = *coin_pub
+      };
       struct TALER_CoinSpendSignatureP sig;
       struct GNUNET_JSON_Specification spec[] = {
         GNUNET_JSON_spec_fixed_auto ("coin_sig",
@@ -518,10 +529,12 @@ TALER_EXCHANGE_verify_coin_history (
                                      &dr.h_contract_terms),
         GNUNET_JSON_spec_fixed_auto ("h_wire",
                                      &dr.h_wire),
-        GNUNET_JSON_spec_absolute_time_nbo ("timestamp",
-                                            &dr.timestamp),
-        GNUNET_JSON_spec_absolute_time_nbo ("refund_deadline",
-                                            &dr.refund_deadline),
+        GNUNET_JSON_spec_fixed_auto ("h_denom_pub",
+                                     &dr.h_denom_pub),
+        TALER_JSON_spec_absolute_time_nbo ("timestamp",
+                                           &dr.wallet_timestamp),
+        TALER_JSON_spec_absolute_time_nbo ("refund_deadline",
+                                           &dr.refund_deadline),
         TALER_JSON_spec_amount_nbo ("deposit_fee",
                                     &dr.deposit_fee),
         GNUNET_JSON_spec_fixed_auto ("merchant_pub",
@@ -537,20 +550,18 @@ TALER_EXCHANGE_verify_coin_history (
         GNUNET_break_op (0);
         return GNUNET_SYSERR;
       }
-      dr.purpose.size = htonl (sizeof (dr));
-      dr.purpose.purpose = htonl (TALER_SIGNATURE_WALLET_COIN_DEPOSIT);
       TALER_amount_hton (&dr.amount_with_fee,
                          &amount);
-      dr.coin_pub = *coin_pub;
       if (GNUNET_OK !=
           GNUNET_CRYPTO_eddsa_verify (TALER_SIGNATURE_WALLET_COIN_DEPOSIT,
-                                      &dr.purpose,
+                                      &dr,
                                       &sig.eddsa_signature,
                                       &coin_pub->eddsa_pub))
       {
         GNUNET_break_op (0);
         return GNUNET_SYSERR;
       }
+      *h_denom_pub = dr.h_denom_pub;
       if (NULL != dk)
       {
         /* check that deposit fee matches our expectations from /keys! */
@@ -579,6 +590,8 @@ TALER_EXCHANGE_verify_coin_history (
                                      &sig),
         GNUNET_JSON_spec_fixed_auto ("rc",
                                      &rm.rc),
+        GNUNET_JSON_spec_fixed_auto ("h_denom_pub",
+                                     &rm.h_denom_pub),
         TALER_JSON_spec_amount_nbo ("melt_fee",
                                     &rm.melt_fee),
         GNUNET_JSON_spec_end ()
@@ -599,13 +612,14 @@ TALER_EXCHANGE_verify_coin_history (
       rm.coin_pub = *coin_pub;
       if (GNUNET_OK !=
           GNUNET_CRYPTO_eddsa_verify (TALER_SIGNATURE_WALLET_COIN_MELT,
-                                      &rm.purpose,
+                                      &rm,
                                       &sig.eddsa_signature,
                                       &coin_pub->eddsa_pub))
       {
         GNUNET_break_op (0);
         return GNUNET_SYSERR;
       }
+      *h_denom_pub = rm.h_denom_pub;
       if (NULL != dk)
       {
         /* check that melt fee matches our expectations from /keys! */
@@ -627,9 +641,17 @@ TALER_EXCHANGE_verify_coin_history (
     else if (0 == strcasecmp (type,
                               "REFUND"))
     {
-      struct TALER_RefundRequestPS rr;
       struct TALER_MerchantSignatureP sig;
+      struct TALER_Amount refund_fee;
+      struct TALER_Amount sig_amount;
+      struct TALER_RefundRequestPS rr = {
+        .purpose.size = htonl (sizeof (rr)),
+        .purpose.purpose = htonl (TALER_SIGNATURE_MERCHANT_REFUND),
+        .coin_pub = *coin_pub
+      };
       struct GNUNET_JSON_Specification spec[] = {
+        TALER_JSON_spec_amount ("refund_fee",
+                                &refund_fee),
         GNUNET_JSON_spec_fixed_auto ("merchant_sig",
                                      &sig),
         GNUNET_JSON_spec_fixed_auto ("h_contract_terms",
@@ -638,8 +660,6 @@ TALER_EXCHANGE_verify_coin_history (
                                      &rr.merchant),
         GNUNET_JSON_spec_uint64 ("rtransaction_id",
                                  &rr.rtransaction_id),
-        TALER_JSON_spec_amount_nbo ("refund_fee",
-                                    &rr.refund_fee),
         GNUNET_JSON_spec_end ()
       };
 
@@ -651,14 +671,22 @@ TALER_EXCHANGE_verify_coin_history (
         GNUNET_break_op (0);
         return GNUNET_SYSERR;
       }
-      rr.purpose.size = htonl (sizeof (rr));
-      rr.purpose.purpose = htonl (TALER_SIGNATURE_MERCHANT_REFUND);
-      rr.coin_pub = *coin_pub;
+      if (0 >
+          TALER_amount_add (&sig_amount,
+                            &refund_fee,
+                            &amount))
+      {
+        GNUNET_break_op (0);
+        return GNUNET_SYSERR;
+      }
       TALER_amount_hton (&rr.refund_amount,
-                         &amount);
+                         &sig_amount);
+      rr.rtransaction_id = GNUNET_htonll (rr.rtransaction_id);
+      TALER_amount_hton (&rr.refund_amount,
+                         &sig_amount);
       if (GNUNET_OK !=
           GNUNET_CRYPTO_eddsa_verify (TALER_SIGNATURE_MERCHANT_REFUND,
-                                      &rr.purpose,
+                                      &rr,
                                       &sig.eddsa_sig,
                                       &rr.merchant.eddsa_pub))
       {
@@ -676,13 +704,11 @@ TALER_EXCHANGE_verify_coin_history (
       /* check that refund fee matches our expectations from /keys! */
       if (NULL != dk)
       {
-        TALER_amount_ntoh (&fee,
-                           &rr.refund_fee);
         if ( (GNUNET_YES !=
-              TALER_amount_cmp_currency (&fee,
+              TALER_amount_cmp_currency (&refund_fee,
                                          &dk->fee_refund)) ||
              (0 !=
-              TALER_amount_cmp (&fee,
+              TALER_amount_cmp (&refund_fee,
                                 &dk->fee_refund)) )
         {
           GNUNET_break_op (0);
@@ -694,18 +720,36 @@ TALER_EXCHANGE_verify_coin_history (
     else if (0 == strcasecmp (type,
                               "RECOUP"))
     {
-      struct TALER_RecoupConfirmationPS pc;
+      struct TALER_RecoupConfirmationPS pc = {
+        .purpose.size = htonl (sizeof (pc)),
+        .purpose.purpose = htonl (TALER_SIGNATURE_EXCHANGE_CONFIRM_RECOUP),
+        .coin_pub = *coin_pub
+      };
+      struct TALER_RecoupRequestPS rr = {
+        .purpose.size = htonl (sizeof (pc)),
+        .purpose.purpose = htonl (TALER_SIGNATURE_WALLET_COIN_RECOUP),
+        .coin_pub = *coin_pub
+      };
       struct TALER_ExchangePublicKeyP exchange_pub;
       struct TALER_ExchangeSignatureP exchange_sig;
+      struct TALER_CoinSpendSignatureP coin_sig;
       struct GNUNET_JSON_Specification spec[] = {
+        TALER_JSON_spec_amount_nbo ("amount",
+                                    &pc.recoup_amount),
         GNUNET_JSON_spec_fixed_auto ("exchange_sig",
                                      &exchange_sig),
         GNUNET_JSON_spec_fixed_auto ("exchange_pub",
                                      &exchange_pub),
         GNUNET_JSON_spec_fixed_auto ("reserve_pub",
                                      &pc.reserve_pub),
-        GNUNET_JSON_spec_absolute_time_nbo ("timestamp",
-                                            &pc.timestamp),
+        GNUNET_JSON_spec_fixed_auto ("coin_sig",
+                                     &coin_sig),
+        GNUNET_JSON_spec_fixed_auto ("coin_blind",
+                                     &rr.coin_blind),
+        GNUNET_JSON_spec_fixed_auto ("h_denom_pub",
+                                     &rr.h_denom_pub),
+        TALER_JSON_spec_absolute_time_nbo ("timestamp",
+                                           &pc.timestamp),
         GNUNET_JSON_spec_end ()
       };
 
@@ -717,33 +761,160 @@ TALER_EXCHANGE_verify_coin_history (
         GNUNET_break_op (0);
         return GNUNET_SYSERR;
       }
-      pc.purpose.size = htonl (sizeof (pc));
-      pc.purpose.purpose = htonl (TALER_SIGNATURE_EXCHANGE_CONFIRM_RECOUP);
-      pc.coin_pub = *coin_pub;
       TALER_amount_hton (&pc.recoup_amount,
                          &amount);
       if (GNUNET_OK !=
           GNUNET_CRYPTO_eddsa_verify (TALER_SIGNATURE_EXCHANGE_CONFIRM_RECOUP,
-                                      &pc.purpose,
+                                      &pc,
                                       &exchange_sig.eddsa_signature,
                                       &exchange_pub.eddsa_pub))
       {
         GNUNET_break_op (0);
         return GNUNET_SYSERR;
       }
+      if (GNUNET_OK !=
+          GNUNET_CRYPTO_eddsa_verify (TALER_SIGNATURE_WALLET_COIN_RECOUP,
+                                      &rr,
+                                      &coin_sig.eddsa_signature,
+                                      &coin_pub->eddsa_pub))
+      {
+        GNUNET_break_op (0);
+        return GNUNET_SYSERR;
+      }
+      *h_denom_pub = rr.h_denom_pub;
+      add = GNUNET_YES;
+    }
+    else if (0 == strcasecmp (type,
+                              "RECOUP-REFRESH"))
+    {
+      struct TALER_RecoupRefreshConfirmationPS pc = {
+        .purpose.size = htonl (sizeof (pc)),
+        .purpose.purpose = htonl (
+          TALER_SIGNATURE_EXCHANGE_CONFIRM_RECOUP_REFRESH),
+        .coin_pub = *coin_pub
+      };
+      struct TALER_RecoupRequestPS rr = {
+        .purpose.size = htonl (sizeof (pc)),
+        .purpose.purpose = htonl (TALER_SIGNATURE_WALLET_COIN_RECOUP),
+        .coin_pub = *coin_pub
+      };
+      struct TALER_ExchangePublicKeyP exchange_pub;
+      struct TALER_ExchangeSignatureP exchange_sig;
+      struct TALER_CoinSpendSignatureP coin_sig;
+      struct GNUNET_JSON_Specification spec[] = {
+        TALER_JSON_spec_amount_nbo ("amount",
+                                    &pc.recoup_amount),
+        GNUNET_JSON_spec_fixed_auto ("exchange_sig",
+                                     &exchange_sig),
+        GNUNET_JSON_spec_fixed_auto ("exchange_pub",
+                                     &exchange_pub),
+        GNUNET_JSON_spec_fixed_auto ("coin_sig",
+                                     &coin_sig),
+        GNUNET_JSON_spec_fixed_auto ("old_coin_pub",
+                                     &pc.old_coin_pub),
+        GNUNET_JSON_spec_fixed_auto ("coin_blind",
+                                     &rr.coin_blind),
+        GNUNET_JSON_spec_fixed_auto ("h_denom_pub",
+                                     &rr.h_denom_pub),
+        TALER_JSON_spec_absolute_time_nbo ("timestamp",
+                                           &pc.timestamp),
+        GNUNET_JSON_spec_end ()
+      };
+
+      if (GNUNET_OK !=
+          GNUNET_JSON_parse (transaction,
+                             spec,
+                             NULL, NULL))
+      {
+        GNUNET_break_op (0);
+        return GNUNET_SYSERR;
+      }
+      TALER_amount_hton (&pc.recoup_amount,
+                         &amount);
+      if (GNUNET_OK !=
+          GNUNET_CRYPTO_eddsa_verify (
+            TALER_SIGNATURE_EXCHANGE_CONFIRM_RECOUP_REFRESH,
+            &pc,
+            &exchange_sig.eddsa_signature,
+            &exchange_pub.eddsa_pub))
+      {
+        GNUNET_break_op (0);
+        return GNUNET_SYSERR;
+      }
+      if (GNUNET_OK !=
+          GNUNET_CRYPTO_eddsa_verify (TALER_SIGNATURE_WALLET_COIN_RECOUP,
+                                      &rr,
+                                      &coin_sig.eddsa_signature,
+                                      &coin_pub->eddsa_pub))
+      {
+        GNUNET_break_op (0);
+        return GNUNET_SYSERR;
+      }
+      *h_denom_pub = rr.h_denom_pub;
+      add = GNUNET_YES;
+    }
+    else if (0 == strcasecmp (type,
+                              "OLD-COIN-RECOUP"))
+    {
+      struct TALER_RecoupRefreshConfirmationPS pc = {
+        .purpose.size = htonl (sizeof (pc)),
+        .purpose.purpose = htonl (
+          TALER_SIGNATURE_EXCHANGE_CONFIRM_RECOUP_REFRESH),
+        .old_coin_pub = *coin_pub
+      };
+      struct TALER_ExchangePublicKeyP exchange_pub;
+      struct TALER_ExchangeSignatureP exchange_sig;
+      struct GNUNET_JSON_Specification spec[] = {
+        TALER_JSON_spec_amount_nbo ("amount",
+                                    &pc.recoup_amount),
+        GNUNET_JSON_spec_fixed_auto ("exchange_sig",
+                                     &exchange_sig),
+        GNUNET_JSON_spec_fixed_auto ("exchange_pub",
+                                     &exchange_pub),
+        GNUNET_JSON_spec_fixed_auto ("coin_pub",
+                                     &pc.coin_pub),
+        TALER_JSON_spec_absolute_time_nbo ("timestamp",
+                                           &pc.timestamp),
+        GNUNET_JSON_spec_end ()
+      };
+
+      if (GNUNET_OK !=
+          GNUNET_JSON_parse (transaction,
+                             spec,
+                             NULL, NULL))
+      {
+        GNUNET_break_op (0);
+        return GNUNET_SYSERR;
+      }
+      TALER_amount_hton (&pc.recoup_amount,
+                         &amount);
+      if (GNUNET_OK !=
+          GNUNET_CRYPTO_eddsa_verify (
+            TALER_SIGNATURE_EXCHANGE_CONFIRM_RECOUP_REFRESH,
+            &pc,
+            &exchange_sig.eddsa_signature,
+            &exchange_pub.eddsa_pub))
+      {
+        GNUNET_break_op (0);
+        return GNUNET_SYSERR;
+      }
       add = GNUNET_YES;
     }
     else
     {
       /* signature not supported, new version on server? */
       GNUNET_break_op (0);
+      GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                  "Unexpected type `%s' in response\n",
+                  type);
       GNUNET_assert (GNUNET_SYSERR == add);
       return GNUNET_SYSERR;
     }
+
     if (GNUNET_YES == add)
     {
       /* This amount should be added to the total */
-      if (GNUNET_OK !=
+      if (0 >
           TALER_amount_add (total,
                             total,
                             &amount))
@@ -761,7 +932,7 @@ TALER_EXCHANGE_verify_coin_history (
          these negative amounts, as we might get refunds before
          deposits from a semi-evil exchange.  Then, at the end, we do
          the subtraction by calculating "total = total - rtotal" */GNUNET_assert (GNUNET_NO == add);
-      if (GNUNET_OK !=
+      if (0 >
           TALER_amount_add (&rtotal,
                             &rtotal,
                             &amount))
@@ -770,11 +941,13 @@ TALER_EXCHANGE_verify_coin_history (
         GNUNET_break_op (0);
         return GNUNET_SYSERR;
       }
+
     }
   }
 
+
   /* Finally, subtract 'rtotal' from total to handle the subtractions */
-  if (GNUNET_OK !=
+  if (0 >
       TALER_amount_subtract (total,
                              total,
                              &rtotal))
