@@ -61,14 +61,10 @@ TALER_EXCHANGE_free_melt_data_ (struct MeltData *md)
   }
 
   for (unsigned int i = 0; i<TALER_CNC_KAPPA; i++)
-    GNUNET_free_non_null (md->fresh_coins[i]);
-  /* Finally, clean up a bit...
-     (NOTE: compilers might optimize this away, so this is
-     not providing any strong assurances that the key material
-     is purged.) */
-  memset (md,
-          0,
-          sizeof (struct MeltData));
+    GNUNET_free (md->fresh_coins[i]);
+  /* Finally, clean up a bit... */
+  GNUNET_CRYPTO_zero_keys (md,
+                           sizeof (struct MeltData));
 }
 
 
@@ -210,9 +206,9 @@ deserialize_melted_coin (struct MeltedCoin *mc,
  * @param dk information to serialize
  * @param buf buffer to write data in, NULL to just compute
  *            required size
- * @param off offeset at @a buf to use
- * @return number of bytes written to @a buf at @a off, or if
- *        @a buf is NULL, number of bytes required
+ * @param off offset at @a buf to use
+ * @return number of bytes written to @a buf at @a off (in addition to @a off itself), or if
+ *        @a buf is NULL, number of bytes required, excluding @a off
  */
 static size_t
 serialize_denomination_key (const struct TALER_DenominationPublicKey *dk,
@@ -270,7 +266,8 @@ deserialize_denomination_key (struct TALER_DenominationPublicKey *dk,
           buf,
           sizeof (uint32_t));
   pbuf_size = ntohl (be);
-  if (size < sizeof (uint32_t) + pbuf_size)
+  if ( (size < sizeof (uint32_t) + pbuf_size) ||
+       (sizeof (uint32_t) + pbuf_size < pbuf_size) )
   {
     GNUNET_break (0);
     *ok = GNUNET_NO;
@@ -536,11 +533,11 @@ TALER_EXCHANGE_refresh_prepare (
   {
     md.fresh_pks[i].rsa_public_key
       = GNUNET_CRYPTO_rsa_public_key_dup (fresh_pks[i].key.rsa_public_key);
-    if ( (GNUNET_OK !=
+    if ( (0 >
           TALER_amount_add (&total,
                             &total,
                             &fresh_pks[i].value)) ||
-         (GNUNET_OK !=
+         (0 >
           TALER_amount_add (&total,
                             &total,
                             &fresh_pks[i].fee_withdraw)) )
@@ -565,12 +562,8 @@ TALER_EXCHANGE_refresh_prepare (
   /* build up coins */
   for (unsigned int i = 0; i<TALER_CNC_KAPPA; i++)
   {
-    struct GNUNET_CRYPTO_EcdhePrivateKey *tpk;
-
-    tpk = GNUNET_CRYPTO_ecdhe_key_create ();
-    md.melted_coin.transfer_priv[i].ecdhe_priv = *tpk;
-    GNUNET_free (tpk);
-
+    GNUNET_CRYPTO_ecdhe_key_create (
+      &md.melted_coin.transfer_priv[i].ecdhe_priv);
     GNUNET_CRYPTO_ecdhe_key_get_public (
       &md.melted_coin.transfer_priv[i].ecdhe_priv,
       &rce[i].transfer_pub.ecdhe_pub);
@@ -586,6 +579,7 @@ TALER_EXCHANGE_refresh_prepare (
       struct TALER_PlanchetSecretsP *fc = &md.fresh_coins[i][j];
       struct TALER_RefreshCoinData *rcd = &rce[i].new_coins[j];
       struct TALER_PlanchetDetail pd;
+      struct GNUNET_HashCode c_hash;
 
       TALER_planchet_setup_refresh (&trans_sec[i],
                                     j,
@@ -593,6 +587,7 @@ TALER_EXCHANGE_refresh_prepare (
       if (GNUNET_OK !=
           TALER_planchet_prepare (&md.fresh_pks[j],
                                   fc,
+                                  &c_hash,
                                   &pd))
       {
         GNUNET_break_op (0);
@@ -618,8 +613,8 @@ TALER_EXCHANGE_refresh_prepare (
   for (unsigned int i = 0; i < TALER_CNC_KAPPA; i++)
   {
     for (unsigned int j = 0; j < fresh_pks_len; j++)
-      GNUNET_free_non_null (rce[i].new_coins[j].coin_ev);
-    GNUNET_free_non_null (rce[i].new_coins);
+      GNUNET_free (rce[i].new_coins[j].coin_ev);
+    GNUNET_free (rce[i].new_coins);
   }
   TALER_EXCHANGE_free_melt_data_ (&md);
   return buf;
