@@ -211,7 +211,7 @@ handle_refresh_reveal_finished (void *cls,
   switch (response_code)
   {
   case 0:
-    hr.ec = TALER_EC_INVALID_RESPONSE;
+    hr.ec = TALER_EC_GENERIC_INVALID_RESPONSE;
     break;
   case MHD_HTTP_OK:
     {
@@ -225,7 +225,7 @@ handle_refresh_reveal_finished (void *cls,
       if (GNUNET_OK != ret)
       {
         hr.http_status = 0;
-        hr.ec = TALER_EC_REVEAL_REPLY_MALFORMED;
+        hr.ec = TALER_EC_GENERIC_REPLY_MALFORMED;
       }
       else
       {
