@@ -276,7 +276,7 @@ handle_transfers_get_finished (void *cls,
   switch (response_code)
   {
   case 0:
-    hr.ec = TALER_EC_INVALID_RESPONSE;
+    hr.ec = TALER_EC_GENERIC_INVALID_RESPONSE;
     break;
   case MHD_HTTP_OK:
     if (GNUNET_OK ==
@@ -284,7 +284,7 @@ handle_transfers_get_finished (void *cls,
                                          j))
       return;
     GNUNET_break_op (0);
-    hr.ec = TALER_EC_TRANSFERS_GET_REPLY_MALFORMED;
+    hr.ec = TALER_EC_GENERIC_REPLY_MALFORMED;
     hr.http_status = 0;
     break;
   case MHD_HTTP_BAD_REQUEST:
