@@ -81,14 +81,17 @@ do_timeout (void *cls)
  * Function called with information about the auditor.
  *
  * @param cls closure
+ * @param hr http response details
  * @param vi basic information about the auditor
  * @param compat protocol compatibility information
  */
 static void
 version_cb (void *cls,
+            const struct TALER_AUDITOR_HttpResponse *hr,
             const struct TALER_AUDITOR_VersionInformation *vi,
             enum TALER_AUDITOR_VersionCompatibility compat)
 {
+  (void) hr;
   if ( (NULL != vi) &&
        (TALER_AUDITOR_VC_MATCH == compat) )
     global_ret = 0;
@@ -137,8 +140,7 @@ main (int argc,
   GNUNET_log_setup ("test-auditor-api-version",
                     "INFO",
                     NULL);
-  proc = GNUNET_OS_start_process (GNUNET_NO,
-                                  GNUNET_OS_INHERIT_STD_ALL,
+  proc = GNUNET_OS_start_process (GNUNET_OS_INHERIT_STD_ALL,
                                   NULL, NULL, NULL,
                                   "taler-auditor-httpd",
                                   "taler-auditor-httpd",
