@@ -61,8 +61,7 @@ wirewatch_run (void *cls,
   struct WirewatchState *ws = cls;
 
   ws->wirewatch_proc
-    = GNUNET_OS_start_process (GNUNET_NO,
-                               GNUNET_OS_INHERIT_STD_ALL,
+    = GNUNET_OS_start_process (GNUNET_OS_INHERIT_STD_ALL,
                                NULL, NULL, NULL,
                                "taler-exchange-wirewatch",
                                "taler-exchange-wirewatch",
