@@ -69,6 +69,7 @@ TALER_TESTING_interpreter_lookup_command (struct TALER_TESTING_Interpreter *is,
 #define BATCH_INDEX 1
       struct TALER_TESTING_Command *batch;
       struct TALER_TESTING_Command *current;
+      struct TALER_TESTING_Command *icmd;
       const struct TALER_TESTING_Command *match;
 
       current = TALER_TESTING_cmd_batch_get_current (cmd);
@@ -79,15 +80,15 @@ TALER_TESTING_interpreter_lookup_command (struct TALER_TESTING_Interpreter *is,
       /* We must do the loop forward, but we can find the last match */
       match = NULL;
       for (unsigned int j = 0;
-           NULL != (cmd = &batch[j])->label;
+           NULL != (icmd = &batch[j])->label;
            j++)
       {
-        if (current == cmd)
+        if (current == icmd)
           break; /* do not go past current command */
-        if ( (NULL != cmd->label) &&
-             (0 == strcmp (cmd->label,
+        if ( (NULL != icmd->label) &&
+             (0 == strcmp (icmd->label,
                            label)) )
-          match = cmd;
+          match = icmd;
       }
       if (NULL != match)
         return match;
@@ -176,7 +177,7 @@ TALER_TESTING_interpreter_next (struct TALER_TESTING_Interpreter *is)
   struct TALER_TESTING_Command *cmd = &is->commands[is->ip];
 
   if (GNUNET_SYSERR == is->result)
-    return; /* ignore, we already failled! */
+    return; /* ignore, we already failed! */
   if (TALER_TESTING_cmd_is_batch (cmd))
   {
     TALER_TESTING_cmd_batch_next (is);
@@ -354,7 +355,7 @@ do_shutdown (void *cls)
     TALER_FAKEBANK_stop (is->fakebank);
     is->fakebank = NULL;
   }
-  GNUNET_free_non_null (is->commands);
+  GNUNET_free (is->commands);
 }
 
 
@@ -570,7 +571,7 @@ struct MainContext
  * respective handler by writing to the trigger pipe.
  */
 static void
-sighandler_child_death ()
+sighandler_child_death (void)
 {
   static char c;
   int old_errno = errno;  /* back-up errno */
@@ -589,11 +590,13 @@ sighandler_child_death ()
  *
  * @param cls closure, typically, the "run" method containing
  *        all the commands to be run, and a closure for it.
+ * @param hr HTTP response details
  * @param keys the exchange's keys.
  * @param compat protocol compatibility information.
  */
 void
 TALER_TESTING_cert_cb (void *cls,
+                       const struct TALER_EXCHANGE_HttpResponse *hr,
                        const struct TALER_EXCHANGE_Keys *keys,
                        enum TALER_EXCHANGE_VersionCompatibility compat)
 {
@@ -605,25 +608,26 @@ TALER_TESTING_cert_cb (void *cls,
   {
     if (GNUNET_NO == is->working)
     {
-      GNUNET_log
-        (GNUNET_ERROR_TYPE_WARNING,
-        "Got NULL response for /keys"
-        " during startup, retrying!\n");
+      GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
+                  "Got NULL response for /keys during startup (%u/%d), retrying!\n",
+                  hr->http_status,
+                  (int) hr->ec);
       TALER_EXCHANGE_disconnect (is->exchange);
-      GNUNET_assert
-        (NULL != (is->exchange = TALER_EXCHANGE_connect
-                                   (is->ctx,
-                                   main_ctx->exchange_url,
-                                   &TALER_TESTING_cert_cb,
-                                   main_ctx,
-                                   TALER_EXCHANGE_OPTION_END)));
+      GNUNET_assert (NULL != (is->exchange
+                                = TALER_EXCHANGE_connect (is->ctx,
+                                                          main_ctx->exchange_url,
+                                                          &TALER_TESTING_cert_cb,
+                                                          main_ctx,
+                                                          TALER_EXCHANGE_OPTION_END)));
       return;
     }
     else
-      GNUNET_log
-        (GNUNET_ERROR_TYPE_ERROR,
-        "Got NULL response for /keys"
-        " during execution!\n");
+    {
+      GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                  "Got NULL response for /keys during execution (%u/%d)!\n",
+                  hr->http_status,
+                  (int) hr->ec);
+    }
   }
   else
   {
@@ -786,8 +790,7 @@ TALER_TESTING_setup (TALER_TESTING_Main main_cb,
           sizeof (is));
   is.exchanged = exchanged;
   is.cfg = cfg;
-  sigpipe = GNUNET_DISK_pipe (GNUNET_NO, GNUNET_NO,
-                              GNUNET_NO, GNUNET_NO);
+  sigpipe = GNUNET_DISK_pipe (GNUNET_DISK_PF_NONE);
   GNUNET_assert (NULL != sigpipe);
   shc_chld = GNUNET_SIGNAL_handler_install
                (GNUNET_SIGCHLD,
@@ -809,7 +812,7 @@ TALER_TESTING_setup (TALER_TESTING_Main main_cb,
                           &main_ctx);
   if (NULL != is.final_cleanup_cb)
     is.final_cleanup_cb (is.final_cleanup_cb_cls);
-  GNUNET_free_non_null (main_ctx.exchange_url);
+  GNUNET_free (main_ctx.exchange_url);
   GNUNET_SIGNAL_handler_uninstall (shc_chld);
   GNUNET_DISK_pipe_close (sigpipe);
   sigpipe = NULL;
