@@ -52,7 +52,7 @@ invalidate (struct TALER_Amount *a)
  * @return #GNUNET_OK if the string is a valid monetary amount specification,
  *         #GNUNET_SYSERR if it is invalid.
  */
-int
+enum GNUNET_GenericReturnValue
 TALER_string_to_amount (const char *str,
                         struct TALER_Amount *amount)
 {
@@ -126,8 +126,10 @@ TALER_string_to_amount (const char *str,
       return GNUNET_SYSERR;
     }
     n = *value - '0';
-    if ( (amount->value * 10 + n < amount->value) ||
-         (amount->value > MAX_AMOUNT_VALUE) )
+    if ( (amount->value * 10 < amount->value) ||
+         (amount->value * 10 + n < amount->value) ||
+         (amount->value > MAX_AMOUNT_VALUE) ||
+         (amount->value * 10 + n > MAX_AMOUNT_VALUE) )
     {
       GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
                   "Value specified in amount `%s' is too large\n",
@@ -189,7 +191,7 @@ TALER_string_to_amount (const char *str,
  * @return #GNUNET_OK if the string is a valid amount specification,
  *         #GNUNET_SYSERR if it is invalid.
  */
-int
+enum GNUNET_GenericReturnValue
 TALER_string_to_amount_nbo (const char *str,
                             struct TALER_AmountNBO *amount_nbo)
 {
@@ -253,7 +255,7 @@ TALER_amount_ntoh (struct TALER_Amount *res,
  * @return #GNUNET_OK if @a cur is a valid currency specification,
  *         #GNUNET_SYSERR if it is invalid.
  */
-int
+enum GNUNET_GenericReturnValue
 TALER_amount_get_zero (const char *cur,
                        struct TALER_Amount *amount)
 {
@@ -278,10 +280,10 @@ TALER_amount_get_zero (const char *cur,
  * @param amount amount to check
  * @return #GNUNET_OK if @a amount is valid
  */
-int
+enum GNUNET_GenericReturnValue
 TALER_amount_is_valid (const struct TALER_Amount *amount)
 {
-  return ('\0' != amount->currency[0]);
+  return ('\0' != amount->currency[0]) ? GNUNET_OK : GNUNET_NO;
 }
 
 
@@ -292,10 +294,10 @@ TALER_amount_is_valid (const struct TALER_Amount *amount)
  * @return #GNUNET_YES if valid,
  *         #GNUNET_NO if invalid
  */
-static int
+static enum GNUNET_GenericReturnValue
 test_valid_nbo (const struct TALER_AmountNBO *a)
 {
-  return ('\0' != a->currency[0]);
+  return ('\0' != a->currency[0]) ? GNUNET_YES : GNUNET_NO;
 }
 
 
@@ -308,7 +310,7 @@ test_valid_nbo (const struct TALER_AmountNBO *a)
  *         #GNUNET_NO if the currencies are different,
  *         #GNUNET_SYSERR if either amount is invalid
  */
-int
+enum GNUNET_GenericReturnValue
 TALER_amount_cmp_currency (const struct TALER_Amount *a1,
                            const struct TALER_Amount *a2)
 {
@@ -331,7 +333,7 @@ TALER_amount_cmp_currency (const struct TALER_Amount *a1,
  *         #GNUNET_NO if the currencies are different,
  *         #GNUNET_SYSERR if either amount is invalid
  */
-int
+enum GNUNET_GenericReturnValue
 TALER_amount_cmp_currency_nbo (const struct TALER_AmountNBO *a1,
                                const struct TALER_AmountNBO *a2)
 {
@@ -388,18 +390,44 @@ TALER_amount_cmp (const struct TALER_Amount *a1,
 }
 
 
+/**
+ * Compare the value/fraction of two amounts.  Does not compare the currency.
+ * Comparing amounts of different currencies will cause the program to abort().
+ * If unsure, check with #TALER_amount_cmp_currency() first to be sure that
+ * the currencies of the two amounts are identical. NBO variant.
+ *
+ * @param a1 first amount
+ * @param a2 second amount
+ * @return result of the comparison
+ *         -1 if `a1 < a2`
+ *          1 if `a1 > a2`
+ *          0 if `a1 == a2`.
+ */
+int
+TALER_amount_cmp_nbo (const struct TALER_AmountNBO *a1,
+                      const struct TALER_AmountNBO *a2)
+{
+  struct TALER_Amount h1;
+  struct TALER_Amount h2;
+
+  TALER_amount_ntoh (&h1,
+                     a1);
+  TALER_amount_ntoh (&h2,
+                     a2);
+  return TALER_amount_cmp (&h1,
+                           &h2);
+}
+
+
 /**
  * Perform saturating subtraction of amounts.
  *
  * @param[out] diff where to store (@a a1 - @a a2), or invalid if @a a2 > @a a1
  * @param a1 amount to subtract from
  * @param a2 amount to subtract
- * @return #GNUNET_OK if the subtraction worked,
- *         #GNUNET_NO if @a a1 = @a a2
- *         #GNUNET_SYSERR if @a a2 > @a a1 or currencies are incompatible;
- *                        @a diff is set to invalid
+ * @return operation status, negative on failures
  */
-int
+enum TALER_AmountArithmeticResult
 TALER_amount_subtract (struct TALER_Amount *diff,
                        const struct TALER_Amount *a1,
                        const struct TALER_Amount *a2)
@@ -412,7 +440,7 @@ TALER_amount_subtract (struct TALER_Amount *diff,
                                  a2))
   {
     invalidate (diff);
-    return GNUNET_SYSERR;
+    return TALER_AAR_INVALID_CURRENCIES_INCOMPATIBLE;
   }
   /* make local copies to avoid aliasing problems between
      diff and a1/a2 */
@@ -422,7 +450,7 @@ TALER_amount_subtract (struct TALER_Amount *diff,
        (GNUNET_SYSERR == TALER_amount_normalize (&n2)) )
   {
     invalidate (diff);
-    return GNUNET_SYSERR;
+    return TALER_AAR_INVALID_NORMALIZATION_FAILED;
   }
 
   if (n1.fraction < n2.fraction)
@@ -430,7 +458,7 @@ TALER_amount_subtract (struct TALER_Amount *diff,
     if (0 == n1.value)
     {
       invalidate (diff);
-      return GNUNET_SYSERR;
+      return TALER_AAR_INVALID_NEGATIVE_RESULT;
     }
     n1.fraction += TALER_AMOUNT_FRAC_BASE;
     n1.value--;
@@ -438,7 +466,7 @@ TALER_amount_subtract (struct TALER_Amount *diff,
   if (n1.value < n2.value)
   {
     invalidate (diff);
-    return GNUNET_SYSERR;
+    return TALER_AAR_INVALID_NEGATIVE_RESULT;
   }
   GNUNET_assert (GNUNET_OK ==
                  TALER_amount_get_zero (n1.currency,
@@ -449,8 +477,8 @@ TALER_amount_subtract (struct TALER_Amount *diff,
   diff->value = n1.value - n2.value;
   if ( (0 == diff->fraction) &&
        (0 == diff->value) )
-    return GNUNET_NO;
-  return GNUNET_OK;
+    return TALER_AAR_RESULT_ZERO;
+  return TALER_AAR_RESULT_POSITIVE;
 }
 
 
@@ -460,10 +488,9 @@ TALER_amount_subtract (struct TALER_Amount *diff,
  * @param[out] sum where to store @a a1 + @a a2, set to "invalid" on overflow
  * @param a1 first amount to add
  * @param a2 second amount to add
- * @return #GNUNET_OK if the addition worked,
- *         #GNUNET_SYSERR on overflow
+ * @return operation status, negative on failures
  */
-int
+enum TALER_AmountArithmeticResult
 TALER_amount_add (struct TALER_Amount *sum,
                   const struct TALER_Amount *a1,
                   const struct TALER_Amount *a2)
@@ -476,7 +503,7 @@ TALER_amount_add (struct TALER_Amount *sum,
       TALER_amount_cmp_currency (a1, a2))
   {
     invalidate (sum);
-    return GNUNET_SYSERR;
+    return TALER_AAR_INVALID_CURRENCIES_INCOMPATIBLE;
   }
   /* make local copies to avoid aliasing problems between
      diff and a1/a2 */
@@ -486,7 +513,7 @@ TALER_amount_add (struct TALER_Amount *sum,
        (GNUNET_SYSERR == TALER_amount_normalize (&n2)) )
   {
     invalidate (sum);
-    return GNUNET_SYSERR;
+    return TALER_AAR_INVALID_NORMALIZATION_FAILED;
   }
 
   GNUNET_assert (GNUNET_OK ==
@@ -497,13 +524,13 @@ TALER_amount_add (struct TALER_Amount *sum,
   {
     /* integer overflow */
     invalidate (sum);
-    return GNUNET_SYSERR;
+    return TALER_AAR_INVALID_RESULT_OVERFLOW;
   }
   if (res.value > MAX_AMOUNT_VALUE)
   {
     /* too large to be legal */
     invalidate (sum);
-    return GNUNET_SYSERR;
+    return TALER_AAR_INVALID_RESULT_OVERFLOW;
   }
   res.fraction = n1.fraction + n2.fraction;
   if (GNUNET_SYSERR ==
@@ -511,10 +538,13 @@ TALER_amount_add (struct TALER_Amount *sum,
   {
     /* integer overflow via carry from fraction */
     invalidate (sum);
-    return GNUNET_SYSERR;
+    return TALER_AAR_INVALID_RESULT_OVERFLOW;
   }
   *sum = res;
-  return GNUNET_OK;
+  if ( (0 == sum->fraction) &&
+       (0 == sum->value) )
+    return TALER_AAR_RESULT_ZERO;
+  return TALER_AAR_RESULT_POSITIVE;
 }
 
 
@@ -526,29 +556,25 @@ TALER_amount_add (struct TALER_Amount *sum,
  *         #GNUNET_NO if value was already normalized
  *         #GNUNET_SYSERR if value was invalid or could not be normalized
  */
-int
+enum GNUNET_GenericReturnValue
 TALER_amount_normalize (struct TALER_Amount *amount)
 {
-  int ret;
+  uint32_t overflow;
 
   if (GNUNET_YES != TALER_amount_is_valid (amount))
     return GNUNET_SYSERR;
-  ret = GNUNET_NO;
-  while ( (amount->value != UINT64_MAX) &&
-          (amount->fraction >= TALER_AMOUNT_FRAC_BASE) )
+  if (amount->fraction < TALER_AMOUNT_FRAC_BASE)
+    return GNUNET_NO;
+  overflow = amount->fraction / TALER_AMOUNT_FRAC_BASE;
+  amount->fraction %= TALER_AMOUNT_FRAC_BASE;
+  amount->value += overflow;
+  if ( (amount->value < overflow) ||
+       (amount->value > MAX_AMOUNT_VALUE) )
   {
-    amount->fraction -= TALER_AMOUNT_FRAC_BASE;
-    amount->value++;
-    ret = GNUNET_OK;
-  }
-  if (amount->fraction >= TALER_AMOUNT_FRAC_BASE)
-  {
-    /* failed to normalize, adding up fractions caused
-       main value to overflow! */
     invalidate (amount);
     return GNUNET_SYSERR;
   }
-  return ret;
+  return GNUNET_OK;
 }
 
 
@@ -556,7 +582,7 @@ TALER_amount_normalize (struct TALER_Amount *amount)
  * Convert the fraction of @a amount to a string in decimals.
  *
  * @param amount value to convert
- * @param[out] tail where to write the reesult
+ * @param[out] tail where to write the result
  */
 static void
 amount_to_tail (const struct TALER_Amount *amount,
@@ -624,8 +650,9 @@ TALER_amount_to_string (const struct TALER_Amount *amount)
 const char *
 TALER_amount2s (const struct TALER_Amount *amount)
 {
-  /* 12 is sufficient for a uint32_t value in decimal; 3 is for ":.\0" */
-  static char result[TALER_AMOUNT_FRAC_LEN + TALER_CURRENCY_LEN + 3 + 12];
+  /* 24 is sufficient for a uint64_t value in decimal; 3 is for ":.\0" */
+  static GNUNET_THREAD_LOCAL char result[TALER_AMOUNT_FRAC_LEN
+                                         + TALER_CURRENCY_LEN + 3 + 24];
   struct TALER_Amount norm;
 
   if (GNUNET_YES != TALER_amount_is_valid (amount))
@@ -705,7 +732,7 @@ TALER_amount_divide (struct TALER_Amount *result,
  * @return #GNUNET_OK on success, #GNUNET_NO if rounding was unnecessary,
  *         #GNUNET_SYSERR if the amount or currency or @a round_unit was invalid
  */
-int
+enum GNUNET_GenericReturnValue
 TALER_amount_round_down (struct TALER_Amount *amount,
                          const struct TALER_Amount *round_unit)
 {
